@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,12 +18,12 @@ int main(int argc, char* argv[]) {
 
     // Launch the controller passing in command line arguments.
     // Exit program with the controller's return code.
-    try  {
+    try {
         // Instantiate/fetch the application controller singleton.
         DControllerBasePtr& controller = CtrlAgentController::instance();
 
         // 'false' value disables test mode.
-        controller->launch(argc, argv, false);
+        ret = controller->launch(argc, argv, false);
     } catch (const VersionMessage& ex) {
         std::string msg(ex.what());
         if (!msg.empty()) {
@@ -35,10 +35,15 @@ int main(int argc, char* argv[]) {
             std::cerr << msg << std::endl;
         }
         ret = EXIT_FAILURE;
-    } catch (const isc::Exception& ex) {
+    } catch (const std::exception& ex) {
         std::cerr << "Service failed: " << ex.what() << std::endl;
         ret = EXIT_FAILURE;
+    } catch (...) {
+        std::cerr << "Service failed" << std::endl;
+        ret = EXIT_FAILURE;
     }
 
+    CtrlAgentController::instance().reset();
+
     return (ret);
 }
