@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,6 +20,8 @@
 #include <dhcpsrv/cfg_multi_threading.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/db_type.h>
+#include <dhcpsrv/host_mgr.h>
+#include <dhcpsrv/lease_mgr_factory.h>
 #include <hooks/hooks.h>
 #include <hooks/hooks_manager.h>
 #include <stats/stats_mgr.h>
@@ -29,6 +31,7 @@
 
 #include <sstream>
 
+using namespace isc::asiolink;
 using namespace isc::config;
 using namespace isc::data;
 using namespace isc::db;
@@ -108,25 +111,17 @@ ControlledDhcpv4Srv::init(const std::string& file_name) {
     // Set signal handlers. When the SIGHUP is received by the process
     // the server reconfiguration will be triggered. When SIGTERM or
     // SIGINT will be received, the server will start shutting down.
-    signal_set_.reset(new isc::util::SignalSet(SIGINT, SIGHUP, SIGTERM));
-    // Set the pointer to the handler function.
-    signal_handler_ = signalHandler;
+    signal_set_.reset(new IOSignalSet(getIOService(), signalHandler));
+
+    signal_set_->add(SIGINT);
+    signal_set_->add(SIGHUP);
+    signal_set_->add(SIGTERM);
 }
 
 void ControlledDhcpv4Srv::cleanup() {
     // Nothing to do here. No need to disconnect from anything.
 }
 
-/// @brief Configure DHCPv4 server using the configuration file specified.
-///
-/// This function is used to both configure the DHCP server on its startup
-/// and dynamically reconfigure the server when SIGHUP signal is received.
-///
-/// It fetches DHCPv4 server's configuration from the 'Dhcp4' section of
-/// the JSON configuration file.
-///
-/// @param file_name Configuration file location.
-/// @return status of the command
 ConstElementPtr
 ControlledDhcpv4Srv::loadConfigFile(const std::string& file_name) {
     // This is a configuration backend implementation that reads the
@@ -262,11 +257,13 @@ ControlledDhcpv4Srv::commandConfigReloadHandler(const string&,
     std::string file = ControlledDhcpv4Srv::getInstance()->getConfigFile();
     try {
         LOG_INFO(dhcp4_logger, DHCP4_DYNAMIC_RECONFIGURATION).arg(file);
-        return (loadConfigFile(file));
+        auto result = loadConfigFile(file);
+        LOG_INFO(dhcp4_logger, DHCP4_DYNAMIC_RECONFIGURATION_SUCCESS).arg(file);
+        return (result);
     } catch (const std::exception& ex) {
         // Log the unsuccessful reconfiguration. The reason for failure
         // should be already logged. Don't rethrow an exception so as
-        // the control channel perhaps keeps working.
+        // the server keeps working.
         LOG_FATAL(dhcp4_logger, DHCP4_DYNAMIC_RECONFIGURATION_FAIL)
             .arg(file);
         return (createAnswer(CONTROL_RESULT_ERROR,
@@ -412,7 +409,7 @@ ControlledDhcpv4Srv::commandConfigSetHandler(const string&,
     // the logging first in case there's a configuration failure.
     int rcode = 0;
     isc::config::parseAnswer(rcode, result);
-    if (rcode == 0) {
+    if (rcode == CONTROL_RESULT_SUCCESS) {
         CfgMgr::instance().getStagingCfg()->applyLoggingCfg();
 
         // Use new configuration.
@@ -497,8 +494,14 @@ ControlledDhcpv4Srv::commandDhcpDisableHandler(const std::string&,
                                                ConstElementPtr args) {
     std::ostringstream message;
     int64_t max_period = 0;
+    std::string origin;
 
-    // Parse arguments to see if the 'max-period' parameter has been specified.
+    // If the args map does not contain 'origin' parameter, the default type
+    // will be used (user command).
+    NetworkState::Origin type = NetworkState::Origin::USER_COMMAND;
+
+    // Parse arguments to see if the 'max-period' or 'origin' parameters have
+    // been specified.
     if (args) {
         // Arguments must be a map.
         if (args->getType() != Element::map) {
@@ -518,11 +521,26 @@ ControlledDhcpv4Srv::commandDhcpDisableHandler(const std::string&,
                     if (max_period <= 0) {
                         message << "'max-period' must be positive integer";
                     }
+                }
+            }
+            ConstElementPtr origin_element = args->get("origin");
+            // The 'origin' parameter is optional.
+            if (origin_element) {
+                // It must be a string, if specified.
+                if (origin_element->getType() != Element::string) {
+                    message << "'origin' argument must be a string";
 
-                    // The user specified that the DHCP service should resume not
-                    // later than in max-period seconds. If the 'dhcp-enable' command
-                    // is not sent, the DHCP service will resume automatically.
-                    network_state_->delayedEnableAll(static_cast<unsigned>(max_period));
+                } else {
+                    origin = origin_element->stringValue();
+                    if (origin == "ha-partner") {
+                        type = NetworkState::Origin::HA_COMMAND;
+                    } else if (origin != "user") {
+                        if (origin.empty()) {
+                            origin = "(empty string)";
+                        }
+                        message << "invalid value used for 'origin' parameter: "
+                                << origin;
+                    }
                 }
             }
         }
@@ -530,12 +548,18 @@ ControlledDhcpv4Srv::commandDhcpDisableHandler(const std::string&,
 
     // No error occurred, so let's disable the service.
     if (message.tellp() == 0) {
-        network_state_->disableService();
-
         message << "DHCPv4 service disabled";
         if (max_period > 0) {
             message << " for " << max_period << " seconds";
+
+            // The user specified that the DHCP service should resume not
+            // later than in max-period seconds. If the 'dhcp-enable' command
+            // is not sent, the DHCP service will resume automatically.
+            network_state_->delayedEnableAll(static_cast<unsigned>(max_period),
+                                             type);
         }
+        network_state_->disableService(type);
+
         // Success.
         return (config::createAnswer(CONTROL_RESULT_SUCCESS, message.str()));
     }
@@ -545,9 +569,56 @@ ControlledDhcpv4Srv::commandDhcpDisableHandler(const std::string&,
 }
 
 ConstElementPtr
-ControlledDhcpv4Srv::commandDhcpEnableHandler(const std::string&, ConstElementPtr) {
-    network_state_->enableService();
-    return (config::createAnswer(CONTROL_RESULT_SUCCESS, "DHCP service successfully enabled"));
+ControlledDhcpv4Srv::commandDhcpEnableHandler(const std::string&,
+                                              ConstElementPtr args) {
+    std::ostringstream message;
+    std::string origin;
+
+    // If the args map does not contain 'origin' parameter, the default type
+    // will be used (user command).
+    NetworkState::Origin type = NetworkState::Origin::USER_COMMAND;
+
+    // Parse arguments to see if the 'origin' parameter has been specified.
+    if (args) {
+        // Arguments must be a map.
+        if (args->getType() != Element::map) {
+            message << "arguments for the 'dhcp-enable' command must be a map";
+
+        } else {
+            ConstElementPtr origin_element = args->get("origin");
+            // The 'origin' parameter is optional.
+            if (origin_element) {
+                // It must be a string, if specified.
+                if (origin_element->getType() != Element::string) {
+                    message << "'origin' argument must be a string";
+
+                } else {
+                    origin = origin_element->stringValue();
+                    if (origin == "ha-partner") {
+                        type = NetworkState::Origin::HA_COMMAND;
+                    } else if (origin != "user") {
+                        if (origin.empty()) {
+                            origin = "(empty string)";
+                        }
+                        message << "invalid value used for 'origin' parameter: "
+                                << origin;
+                    }
+                }
+            }
+        }
+    }
+
+    // No error occurred, so let's enable the service.
+    if (message.tellp() == 0) {
+        network_state_->enableService(type);
+
+        // Success.
+        return (config::createAnswer(CONTROL_RESULT_SUCCESS,
+                                     "DHCP service successfully enabled"));
+    }
+
+    // Failure.
+    return (config::createAnswer(CONTROL_RESULT_ERROR, message.str()));
 }
 
 ConstElementPtr
@@ -660,12 +731,19 @@ ControlledDhcpv4Srv::commandStatusGetHandler(const string&,
         status->set("reload", Element::create(reload.total_seconds()));
     }
 
-    if (MultiThreadingMgr::instance().getMode()) {
+    auto& mt_mgr = MultiThreadingMgr::instance();
+    if (mt_mgr.getMode()) {
         status->set("multi-threading-enabled", Element::create(true));
         status->set("thread-pool-size", Element::create(static_cast<int32_t>(
                         MultiThreadingMgr::instance().getThreadPoolSize())));
         status->set("packet-queue-size", Element::create(static_cast<int32_t>(
                         MultiThreadingMgr::instance().getPacketQueueSize())));
+        ElementPtr queue_stats = Element::createList();
+        queue_stats->add(Element::create(mt_mgr.getThreadPool().getQueueStat(10)));
+        queue_stats->add(Element::create(mt_mgr.getThreadPool().getQueueStat(100)));
+        queue_stats->add(Element::create(mt_mgr.getThreadPool().getQueueStat(1000)));
+        status->set("packet-queue-statistics", queue_stats);
+
     } else {
         status->set("multi-threading-enabled", Element::create(false));
     }
@@ -774,10 +852,6 @@ ControlledDhcpv4Srv::processCommand(const string& command,
 
 isc::data::ConstElementPtr
 ControlledDhcpv4Srv::processConfig(isc::data::ConstElementPtr config) {
-
-    LOG_DEBUG(dhcp4_logger, DBG_DHCP4_COMMAND, DHCP4_CONFIG_RECEIVED)
-              .arg(config->str());
-
     ControlledDhcpv4Srv* srv = ControlledDhcpv4Srv::getInstance();
 
     // Single stream instance used in all error clauses
@@ -788,6 +862,9 @@ ControlledDhcpv4Srv::processConfig(isc::data::ConstElementPtr config) {
         return (isc::config::createAnswer(1, err.str()));
     }
 
+    LOG_DEBUG(dhcp4_logger, DBG_DHCP4_COMMAND, DHCP4_CONFIG_RECEIVED)
+        .arg(srv->redactConfig(config)->str());
+
     ConstElementPtr answer = configureDhcp4Server(*srv, config);
 
     // Check that configuration was successful. If not, do not reopen sockets
@@ -805,11 +882,20 @@ ControlledDhcpv4Srv::processConfig(isc::data::ConstElementPtr config) {
 
     // Re-open lease and host database with new parameters.
     try {
-        DatabaseConnection::db_lost_callback =
+        DatabaseConnection::db_lost_callback_ =
             std::bind(&ControlledDhcpv4Srv::dbLostCallback, srv, ph::_1);
+
+        DatabaseConnection::db_recovered_callback_ =
+            std::bind(&ControlledDhcpv4Srv::dbRecoveredCallback, srv, ph::_1);
+
+        DatabaseConnection::db_failed_callback_ =
+            std::bind(&ControlledDhcpv4Srv::dbFailedCallback, srv, ph::_1);
+
         CfgDbAccessPtr cfg_db = CfgMgr::instance().getStagingCfg()->getCfgDbAccess();
         cfg_db->setAppendedParameters("universe=4");
         cfg_db->createManagers();
+        // Reset counters related to connections as all managers have been recreated.
+        srv->getNetworkState()->reset(NetworkState::Origin::DB_CONNECTION);
     } catch (const std::exception& ex) {
         err << "Unable to open database: " << ex.what();
         return (isc::config::createAnswer(1, err.str()));
@@ -837,7 +923,7 @@ ControlledDhcpv4Srv::processConfig(isc::data::ConstElementPtr config) {
     // Configure DHCP packet queueing
     try {
         data::ConstElementPtr qc;
-        qc  = CfgMgr::instance().getStagingCfg()->getDHCPQueueControl();
+        qc = CfgMgr::instance().getStagingCfg()->getDHCPQueueControl();
         if (IfaceMgr::instance().configureDHCPPacketQueue(AF_INET, qc)) {
             LOG_INFO(dhcp4_logger, DHCP4_CONFIG_PACKET_QUEUE)
                      .arg(IfaceMgr::instance().getPacketQueue4()->getInfoStr());
@@ -932,9 +1018,6 @@ ControlledDhcpv4Srv::processConfig(isc::data::ConstElementPtr config) {
     // @todo This should be fixed.
     try {
         CfgMultiThreading::apply(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading());
-        if (MultiThreadingMgr::instance().getMode()) {
-            LOG_FATAL(dhcp4_logger, DHCP4_MULTI_THREADING_WARNING);
-        }
     } catch (const std::exception& ex) {
         err << "Error applying multi threading settings: "
             << ex.what();
@@ -948,7 +1031,7 @@ isc::data::ConstElementPtr
 ControlledDhcpv4Srv::checkConfig(isc::data::ConstElementPtr config) {
 
     LOG_DEBUG(dhcp4_logger, DBG_DHCP4_COMMAND, DHCP4_CONFIG_RECEIVED)
-              .arg(config->str());
+        .arg(redactConfig(config)->str());
 
     ControlledDhcpv4Srv* srv = ControlledDhcpv4Srv::getInstance();
 
@@ -965,8 +1048,7 @@ ControlledDhcpv4Srv::checkConfig(isc::data::ConstElementPtr config) {
 
 ControlledDhcpv4Srv::ControlledDhcpv4Srv(uint16_t server_port /*= DHCP4_SERVER_PORT*/,
                                          uint16_t client_port /*= 0*/)
-    : Dhcpv4Srv(server_port, client_port), io_service_(),
-      timer_mgr_(TimerMgr::instance()) {
+    : Dhcpv4Srv(server_port, client_port), timer_mgr_(TimerMgr::instance()) {
     if (getInstance()) {
         isc_throw(InvalidOperation,
                   "There is another Dhcpv4Srv instance already.");
@@ -979,6 +1061,12 @@ ControlledDhcpv4Srv::ControlledDhcpv4Srv(uint16_t server_port /*= DHCP4_SERVER_P
     // CommandMgr uses IO service to run asynchronous socket operations.
     CommandMgr::instance().setIOService(getIOService());
 
+    // LeaseMgr uses IO service to run asynchronous timers.
+    LeaseMgr::setIOService(getIOService());
+
+    // HostMgr uses IO service to run asynchronous timers.
+    HostMgr::setIOService(getIOService());
+
     // These are the commands always supported by the DHCPv4 server.
     // Please keep the list in alphabetic order.
     CommandMgr::instance().registerCommand("build-report",
@@ -1060,17 +1148,21 @@ ControlledDhcpv4Srv::ControlledDhcpv4Srv(uint16_t server_port /*= DHCP4_SERVER_P
 
 void ControlledDhcpv4Srv::shutdownServer(int exit_value) {
     setExitValue(exit_value);
-    io_service_.stop();       // Stop ASIO transmissions
+    getIOService()->stop();   // Stop ASIO transmissions
     shutdown();               // Initiate DHCPv4 shutdown procedure.
 }
 
 ControlledDhcpv4Srv::~ControlledDhcpv4Srv() {
     try {
+        LeaseMgrFactory::destroy();
+        HostMgr::create();
         cleanup();
 
         // The closure captures either a shared pointer (memory leak)
         // or a raw pointer (pointing to a deleted object).
-        DatabaseConnection::db_lost_callback = 0;
+        DatabaseConnection::db_lost_callback_ = 0;
+        DatabaseConnection::db_recovered_callback_ = 0;
+        DatabaseConnection::db_failed_callback_ = 0;
 
         timer_mgr_->unregisterTimers();
 
@@ -1104,6 +1196,11 @@ ControlledDhcpv4Srv::~ControlledDhcpv4Srv() {
         CommandMgr::instance().deregisterCommand("status-get");
         CommandMgr::instance().deregisterCommand("version-get");
 
+        // LeaseMgr uses IO service to run asynchronous timers.
+        LeaseMgr::setIOService(IOServicePtr());
+
+        // HostMgr uses IO service to run asynchronous timers.
+        HostMgr::setIOService(IOServicePtr());
     } catch (...) {
         // Don't want to throw exceptions from the destructor. The server
         // is shutting down anyway.
@@ -1114,14 +1211,6 @@ ControlledDhcpv4Srv::~ControlledDhcpv4Srv() {
                     // at this stage anyway.
 }
 
-void ControlledDhcpv4Srv::sessionReader(void) {
-    // Process one asio event. If there are more events, iface_mgr will call
-    // this callback more than once.
-    if (getInstance()) {
-        getInstance()->io_service_.run_one();
-    }
-}
-
 void
 ControlledDhcpv4Srv::reclaimExpiredLeases(const size_t max_leases,
                                           const uint16_t timeout,
@@ -1146,89 +1235,74 @@ ControlledDhcpv4Srv::deleteExpiredReclaimedLeases(const uint32_t secs) {
     TimerMgr::instance()->setup(CfgExpiration::FLUSH_RECLAIMED_TIMER_NAME);
 }
 
-void
-ControlledDhcpv4Srv::dbReconnect(ReconnectCtlPtr db_reconnect_ctl) {
-    bool reopened = false;
-
-    // We lost at least one of them. Reopen all of them (lease, host, and CB databases).
-    try {
-        CfgDbAccessPtr cfg_db = CfgMgr::instance().getCurrentCfg()->getCfgDbAccess();
-        cfg_db->createManagers();
-
-        auto ctl_info = CfgMgr::instance().getCurrentCfg()->getConfigControlInfo();
-        if (ctl_info) {
-            auto srv_cfg = CfgMgr::instance().getCurrentCfg();
-            server_->getCBControl()->databaseConfigConnect(srv_cfg);
-        }
-
-        reopened = true;
-    } catch (const std::exception& ex) {
-        LOG_ERROR(dhcp4_logger, DHCP4_DB_RECONNECT_ATTEMPT_FAILED).arg(ex.what());
-    }
-
-    if (reopened) {
-        // Cancel the timer.
-        if (TimerMgr::instance()->isTimerRegistered("Dhcp4DbReconnectTimer")) {
-            TimerMgr::instance()->cancel("Dhcp4DbReconnectTimer");
-        }
-
-        // Set network state to service enabled
-        network_state_->enableService();
-
-        // Toss the reconnect control, we're done with it
-        db_reconnect_ctl.reset();
-    } else {
-        if (!db_reconnect_ctl->checkRetries()) {
-            // We're out of retries, log it and initiate shutdown.
-            LOG_ERROR(dhcp4_logger, DHCP4_DB_RECONNECT_RETRIES_EXHAUSTED)
-            .arg(db_reconnect_ctl->maxRetries());
-            shutdownServer(EXIT_FAILURE);
-            return;
-        }
-
-        LOG_INFO(dhcp4_logger, DHCP4_DB_RECONNECT_ATTEMPT_SCHEDULE)
-                .arg(db_reconnect_ctl->maxRetries() - db_reconnect_ctl->retriesLeft() + 1)
-                .arg(db_reconnect_ctl->maxRetries())
-                .arg(db_reconnect_ctl->retryInterval());
-
-        if (!TimerMgr::instance()->isTimerRegistered("Dhcp4DbReconnectTimer")) {
-            TimerMgr::instance()->registerTimer("Dhcp4DbReconnectTimer",
-                            std::bind(&ControlledDhcpv4Srv::dbReconnect, this,
-                                      db_reconnect_ctl),
-                            db_reconnect_ctl->retryInterval(),
-                            asiolink::IntervalTimer::ONE_SHOT);
-        }
-
-        TimerMgr::instance()->setup("Dhcp4DbReconnectTimer");
-    }
-}
-
 bool
 ControlledDhcpv4Srv::dbLostCallback(ReconnectCtlPtr db_reconnect_ctl) {
-    // Disable service until we recover
-    network_state_->disableService();
-
     if (!db_reconnect_ctl) {
-        // This shouldn't never happen
+        // This should never happen
         LOG_ERROR(dhcp4_logger, DHCP4_DB_RECONNECT_NO_DB_CTL);
         return (false);
     }
 
-    // If reconnect isn't enabled log it,
-    // initiate a shutdown and return false.
+    // Disable service until the connection is recovered.
+    if (db_reconnect_ctl->retriesLeft() == db_reconnect_ctl->maxRetries() &&
+        db_reconnect_ctl->alterServiceState()) {
+        network_state_->disableService(NetworkState::Origin::DB_CONNECTION);
+    }
+
+    LOG_INFO(dhcp4_logger, DHCP4_DB_RECONNECT_LOST_CONNECTION);
+
+    // If reconnect isn't enabled log it, initiate a shutdown if needed and
+    // return false.
     if (!db_reconnect_ctl->retriesLeft() ||
         !db_reconnect_ctl->retryInterval()) {
         LOG_INFO(dhcp4_logger, DHCP4_DB_RECONNECT_DISABLED)
             .arg(db_reconnect_ctl->retriesLeft())
             .arg(db_reconnect_ctl->retryInterval());
-        shutdownServer(EXIT_FAILURE);
-        return(false);
+        if (db_reconnect_ctl->exitOnFailure()) {
+            shutdownServer(EXIT_FAILURE);
+        }
+        return (false);
     }
 
-    // Invoke reconnect method
-    dbReconnect(db_reconnect_ctl);
+    return (true);
+}
 
-    return(true);
+bool
+ControlledDhcpv4Srv::dbRecoveredCallback(ReconnectCtlPtr db_reconnect_ctl) {
+    if (!db_reconnect_ctl) {
+        // This should never happen
+        LOG_ERROR(dhcp4_logger, DHCP4_DB_RECONNECT_NO_DB_CTL);
+        return (false);
+    }
+
+    // Enable service after the connection is recovered.
+    if (db_reconnect_ctl->alterServiceState()) {
+        network_state_->enableService(NetworkState::Origin::DB_CONNECTION);
+    }
+
+    LOG_INFO(dhcp4_logger, DHCP4_DB_RECONNECT_SUCCEEDED);
+
+    db_reconnect_ctl->resetRetries();
+
+    return (true);
+}
+
+bool
+ControlledDhcpv4Srv::dbFailedCallback(ReconnectCtlPtr db_reconnect_ctl) {
+    if (!db_reconnect_ctl) {
+        // This should never happen
+        LOG_ERROR(dhcp4_logger, DHCP4_DB_RECONNECT_NO_DB_CTL);
+        return (false);
+    }
+
+    LOG_INFO(dhcp4_logger, DHCP4_DB_RECONNECT_FAILED)
+            .arg(db_reconnect_ctl->maxRetries());
+
+    if (db_reconnect_ctl->exitOnFailure()) {
+        shutdownServer(EXIT_FAILURE);
+    }
+
+    return (true);
 }
 
 void
