@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,6 +26,7 @@
 #include <dhcpsrv/parsers/host_reservation_parser.h>
 #include <dhcpsrv/parsers/host_reservations_list_parser.h>
 #include <dhcpsrv/parsers/ifaces_config_parser.h>
+#include <dhcpsrv/parsers/multi_threading_config_parser.h>
 #include <dhcpsrv/parsers/option_data_parser.h>
 #include <dhcpsrv/parsers/dhcp_queue_control_parser.h>
 #include <dhcpsrv/parsers/simple_parser4.h>
@@ -34,6 +35,7 @@
 #include <dhcpsrv/host_data_source_factory.h>
 #include <dhcpsrv/timer_mgr.h>
 #include <process/config_ctl_parser.h>
+#include <hooks/hooks_manager.h>
 #include <hooks/hooks_parser.h>
 #include <config/command_mgr.h>
 #include <util/encode/hex.h>
@@ -58,7 +60,6 @@ using namespace isc::asiolink;
 using namespace isc::hooks;
 using namespace isc::process;
 using namespace isc::config;
-using namespace isc::db;
 
 namespace {
 
@@ -114,6 +115,23 @@ public:
         cfg->setServerTag(server_tag);
     }
 
+    /// @brief Sets global parameters before other parameters are parsed.
+    ///
+    /// This method sets selected global parameters before other parameters
+    /// are parsed. This is important when the behavior of the parsers
+    /// run later depends on these global parameters.
+    ///
+    /// Currently this method sets the following global parameters:
+    /// - ip-reservations-unique
+    ///
+    /// @param global global configuration scope
+    /// @param cfg Server configuration (parsed parameters will be stored here)
+    void parseEarly(const SrvConfigPtr& cfg, const ConstElementPtr& global) {
+        // Set ip-reservations-unique flag.
+        bool ip_reservations_unique = getBoolean(global, "ip-reservations-unique");
+        cfg->setIPReservationsUnique(ip_reservations_unique);
+    }
+
     /// @brief Copies subnets from shared networks to regular subnets container
     ///
     /// @param from pointer to shared networks container (copy from here)
@@ -163,6 +181,9 @@ public:
     void
     sanityChecks(const SrvConfigPtr& cfg, const ConstElementPtr& global) {
 
+        /// Global lifetime sanity checks
+        cfg->sanityChecksLifetime("valid-lifetime");
+
         /// Shared network sanity checks
         const SharedNetwork4Collection* networks = cfg->getCfgSharedNetworks4()->getAll();
         if (networks) {
@@ -297,7 +318,6 @@ void configureCommandChannel() {
     }
 }
 
-
 isc::data::ConstElementPtr
 configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
                      bool check_only) {
@@ -307,15 +327,16 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
         return (answer);
     }
 
-    LOG_DEBUG(dhcp4_logger, DBG_DHCP4_COMMAND,
-              DHCP4_CONFIG_START).arg(config_set->str());
+    LOG_DEBUG(dhcp4_logger, DBG_DHCP4_COMMAND, DHCP4_CONFIG_START)
+        .arg(server.redactConfig(config_set)->str());
 
     // Before starting any subnet operations, let's reset the subnet-id counter,
     // so newly recreated configuration starts with first subnet-id equal 1.
     Subnet::resetSubnetID();
 
-    // Remove any existing timers.
+    // Close DHCP sockets and remove any existing timers.
     if (!check_only) {
+        IfaceMgr::instance().closeSockets();
         TimerMgr::instance()->unregisterTimers();
         server.discardPackets();
         server.getCBControl()->reset();
@@ -335,10 +356,8 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
     // Rollback informs whether error occurred and original data
     // have to be restored to global storages.
     bool rollback = false;
-    // config_pair holds the details of the current parser when iterating over
-    // the parsers.  It is declared outside the loops so in case of an error,
-    // the name of the failing parser can be retrieved in the "catch" clause.
-    ConfigPair config_pair;
+    // Global parameter name in case of an error.
+    string parameter_name;
     ElementPtr mutable_cfg;
     SrvConfigPtr srv_cfg;
     try {
@@ -350,186 +369,255 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
         // default values and will insert derived values as well.
         mutable_cfg = boost::const_pointer_cast<Element>(config_set);
 
+        // Relocate dhcp-ddns parameters that have moved to global scope.
+        // Rule is that a global value overrides the dhcp-ddns value, so
+        // we need to do this before we apply global defaults.
+        // Note this is done for backward compatibility.
+        srv_cfg->moveDdnsParams(mutable_cfg);
+
+        // Move from reservation mode to new reservations flags.
+        // @todo add warning
+        BaseNetworkParser::moveReservationMode(mutable_cfg);
+
         // Set all default values if not specified by the user.
         SimpleParser4::setAllDefaults(mutable_cfg);
 
         // And now derive (inherit) global parameters to subnets, if not specified.
         SimpleParser4::deriveParameters(mutable_cfg);
 
+        // In principle we could have the following code structured as a series
+        // of long if else if clauses. That would give a marginal performance
+        // boost, but would make the code less readable. We had serious issues
+        // with the parser code debugability, so I decided to keep it as a
+        // series of independent ifs.
+
+        // This parser is used in several places.
+        Dhcp4ConfigParser global_parser;
+
+        // Apply global options in the staging config, e.g. ip-reservations-unique
+        global_parser.parseEarly(srv_cfg, mutable_cfg);
+
         // We need definitions first
         ConstElementPtr option_defs = mutable_cfg->get("option-def");
         if (option_defs) {
+            parameter_name = "option-def";
             OptionDefListParser parser(AF_INET);
             CfgOptionDefPtr cfg_option_def = srv_cfg->getCfgOptionDef();
             parser.parse(cfg_option_def, option_defs);
         }
 
-        // This parser is used in several places, so it should be available
-        // early.
-        Dhcp4ConfigParser global_parser;
+        ConstElementPtr option_datas = mutable_cfg->get("option-data");
+        if (option_datas) {
+            parameter_name = "option-data";
+            OptionDataListParser parser(AF_INET);
+            CfgOptionPtr cfg_option = srv_cfg->getCfgOption();
+            parser.parse(cfg_option, option_datas);
+        }
+
+        ConstElementPtr control_socket = mutable_cfg->get("control-socket");
+        if (control_socket) {
+            parameter_name = "control-socket";
+            ControlSocketParser parser;
+            parser.parse(*srv_cfg, control_socket);
+        }
+
+        ConstElementPtr multi_threading = mutable_cfg->get("multi-threading");
+        if (multi_threading) {
+            parameter_name = "multi-threading";
+            MultiThreadingConfigParser parser;
+            parser.parse(*srv_cfg, multi_threading);
+        }
+
+        ConstElementPtr queue_control = mutable_cfg->get("dhcp-queue-control");
+        if (queue_control) {
+            parameter_name = "dhcp-queue-control";
+            DHCPQueueControlParser parser;
+            srv_cfg->setDHCPQueueControl(parser.parse(queue_control));
+        }
+
+        ConstElementPtr hr_identifiers =
+            mutable_cfg->get("host-reservation-identifiers");
+        if (hr_identifiers) {
+            parameter_name = "host-reservation-identifiers";
+            HostReservationIdsParser4 parser;
+            parser.parse(hr_identifiers);
+        }
+
+        ConstElementPtr ifaces_config = mutable_cfg->get("interfaces-config");
+        if (ifaces_config) {
+            parameter_name = "interfaces-config";
+            IfacesConfigParser parser(AF_INET, check_only);
+            CfgIfacePtr cfg_iface = srv_cfg->getCfgIface();
+            parser.parse(cfg_iface, ifaces_config);
+        }
+
+        ConstElementPtr sanity_checks = mutable_cfg->get("sanity-checks");
+        if (sanity_checks) {
+            parameter_name = "sanity-checks";
+            SanityChecksParser parser;
+            parser.parse(*srv_cfg, sanity_checks);
+        }
+
+        ConstElementPtr expiration_cfg =
+            mutable_cfg->get("expired-leases-processing");
+        if (expiration_cfg) {
+            parameter_name = "expired-leases-processing";
+            ExpirationConfigParser parser;
+            parser.parse(expiration_cfg);
+        }
+
+        // The hooks-libraries configuration must be parsed after parsing
+        // multi-threading configuration so that libraries are checked
+        // for multi-threading compatibility.
+        ConstElementPtr hooks_libraries = mutable_cfg->get("hooks-libraries");
+        if (hooks_libraries) {
+            parameter_name = "hooks-libraries";
+            HooksLibrariesParser hooks_parser;
+            HooksConfig& libraries = srv_cfg->getHooksConfig();
+            hooks_parser.parse(libraries, hooks_libraries);
+            libraries.verifyLibraries(hooks_libraries->getPosition());
+        }
 
         // D2 client configuration.
         D2ClientConfigPtr d2_client_cfg;
 
-        // Make parsers grouping.
-        const std::map<std::string, ConstElementPtr>& values_map =
-                                                        mutable_cfg->mapValue();
-        BOOST_FOREACH(config_pair, values_map) {
-            // In principle we could have the following code structured as a series
-            // of long if else if clauses. That would give a marginal performance
-            // boost, but would make the code less readable. We had serious issues
-            // with the parser code debugability, so I decided to keep it as a
-            // series of independent ifs.
-            if (config_pair.first == "option-def") {
-                // This is converted to SimpleParser and is handled already above.
-                continue;
-            }
+        // Legacy DhcpConfigParser stuff below.
+        ConstElementPtr dhcp_ddns = mutable_cfg->get("dhcp-ddns");
+        if (dhcp_ddns) {
+            parameter_name = "dhcp-ddns";
+            // Apply defaults
+            D2ClientConfigParser::setAllDefaults(dhcp_ddns);
+            D2ClientConfigParser parser;
+            d2_client_cfg = parser.parse(dhcp_ddns);
+        }
 
-            if (config_pair.first == "option-data") {
-                OptionDataListParser parser(AF_INET);
-                CfgOptionPtr cfg_option = srv_cfg->getCfgOption();
-                parser.parse(cfg_option, config_pair.second);
-                continue;
-            }
+        ConstElementPtr client_classes = mutable_cfg->get("client-classes");
+        if (client_classes) {
+            parameter_name = "client-classes";
+            ClientClassDefListParser parser;
+            ClientClassDictionaryPtr dictionary =
+                parser.parse(client_classes, AF_INET);
+            srv_cfg->setClientClassDictionary(dictionary);
+        }
 
-            if (config_pair.first == "control-socket") {
-                ControlSocketParser parser;
-                parser.parse(*srv_cfg, config_pair.second);
-                continue;
-            }
+        // Please move at the end when migration will be finished.
+        ConstElementPtr lease_database = mutable_cfg->get("lease-database");
+        if (lease_database) {
+            parameter_name = "lease-database";
+            db::DbAccessParser parser;
+            std::string access_string;
+            parser.parse(access_string, lease_database);
+            CfgDbAccessPtr cfg_db_access = srv_cfg->getCfgDbAccess();
+            cfg_db_access->setLeaseDbAccessString(access_string);
+        }
 
-            if (config_pair.first == "dhcp-queue-control") {
-                DHCPQueueControlParser parser;
-                srv_cfg->setDHCPQueueControl(parser.parse(config_pair.second));
-                continue;
-            }
+        ConstElementPtr hosts_database = mutable_cfg->get("hosts-database");
+        if (hosts_database) {
+            parameter_name = "hosts-database";
+            db::DbAccessParser parser;
+            std::string access_string;
+            parser.parse(access_string, hosts_database);
+            CfgDbAccessPtr cfg_db_access = srv_cfg->getCfgDbAccess();
+            cfg_db_access->setHostDbAccessString(access_string);
+        }
 
-            if (config_pair.first == "host-reservation-identifiers") {
-                HostReservationIdsParser4 parser;
-                parser.parse(config_pair.second);
-                continue;
-            }
-
-            if (config_pair.first == "interfaces-config") {
-                ElementPtr ifaces_cfg =
-                    boost::const_pointer_cast<Element>(config_pair.second);
-                if (check_only) {
-                    // No re-detection in check only mode
-                    ifaces_cfg->set("re-detect", Element::create(false));
-                }
-                IfacesConfigParser parser(AF_INET);
-                CfgIfacePtr cfg_iface = srv_cfg->getCfgIface();
-                parser.parse(cfg_iface, ifaces_cfg);
-                continue;
-            }
-
-            if (config_pair.first == "sanity-checks") {
-                SanityChecksParser parser;
-                parser.parse(*srv_cfg, config_pair.second);
-                continue;
-            }
-
-            if (config_pair.first == "expired-leases-processing") {
-                ExpirationConfigParser parser;
-                parser.parse(config_pair.second);
-                continue;
-            }
-
-            if (config_pair.first == "hooks-libraries") {
-                HooksLibrariesParser hooks_parser;
-                HooksConfig& libraries = srv_cfg->getHooksConfig();
-                hooks_parser.parse(libraries, config_pair.second);
-                libraries.verifyLibraries(config_pair.second->getPosition());
-                continue;
-            }
-
-            // Legacy DhcpConfigParser stuff below
-            if (config_pair.first == "dhcp-ddns") {
-                // Apply defaults
-                D2ClientConfigParser::setAllDefaults(config_pair.second);
-                D2ClientConfigParser parser;
-                d2_client_cfg = parser.parse(config_pair.second);
-                continue;
-            }
-
-            if (config_pair.first == "client-classes") {
-                ClientClassDefListParser parser;
-                ClientClassDictionaryPtr dictionary =
-                    parser.parse(config_pair.second, AF_INET);
-                srv_cfg->setClientClassDictionary(dictionary);
-                continue;
-            }
-
-            // Please move at the end when migration will be finished.
-            if (config_pair.first == "lease-database") {
-                db::DbAccessParser parser;
+        ConstElementPtr hosts_databases = mutable_cfg->get("hosts-databases");
+        if (hosts_databases) {
+            parameter_name = "hosts-databases";
+            CfgDbAccessPtr cfg_db_access = srv_cfg->getCfgDbAccess();
+            db::DbAccessParser parser;
+            for (auto it : hosts_databases->listValue()) {
                 std::string access_string;
-                parser.parse(access_string, config_pair.second);
-                CfgDbAccessPtr cfg_db_access = srv_cfg->getCfgDbAccess();
-                cfg_db_access->setLeaseDbAccessString(access_string);
-                continue;
-            }
-
-            if (config_pair.first == "hosts-database") {
-                db::DbAccessParser parser;
-                std::string access_string;
-                parser.parse(access_string, config_pair.second);
-                CfgDbAccessPtr cfg_db_access = srv_cfg->getCfgDbAccess();
+                parser.parse(access_string, it);
                 cfg_db_access->setHostDbAccessString(access_string);
-                continue;
             }
+        }
 
-            if (config_pair.first == "hosts-databases") {
-                CfgDbAccessPtr cfg_db_access = srv_cfg->getCfgDbAccess();
-                db::DbAccessParser parser;
-                auto list = config_pair.second->listValue();
-                for (auto it : list) {
-                    std::string access_string;
-                    parser.parse(access_string, it);
-                    cfg_db_access->setHostDbAccessString(access_string);
+        // Keep relative orders of shared networks and subnets.
+        ConstElementPtr shared_networks = mutable_cfg->get("shared-networks");
+        if (shared_networks) {
+            parameter_name = "shared-networks";
+            /// We need to create instance of SharedNetworks4ListParser
+            /// and parse the list of the shared networks into the
+            /// CfgSharedNetworks4 object. One additional step is then to
+            /// add subnets from the CfgSharedNetworks4 into CfgSubnets4
+            /// as well.
+            SharedNetworks4ListParser parser;
+            CfgSharedNetworks4Ptr cfg = srv_cfg->getCfgSharedNetworks4();
+            parser.parse(cfg, shared_networks);
+
+            // We also need to put the subnets it contains into normal
+            // subnets list.
+            global_parser.copySubnets4(srv_cfg->getCfgSubnets4(), cfg);
+        }
+
+        ConstElementPtr subnet4 = mutable_cfg->get("subnet4");
+        if (subnet4) {
+            parameter_name = "subnet4";
+            Subnets4ListConfigParser subnets_parser;
+            // parse() returns number of subnets parsed. We may log it one day.
+            subnets_parser.parse(srv_cfg, subnet4);
+        }
+
+        ConstElementPtr reservations = mutable_cfg->get("reservations");
+        if (reservations) {
+            parameter_name = "reservations";
+            HostCollection hosts;
+            HostReservationsListParser<HostReservationParser4> parser;
+            parser.parse(SUBNET_ID_GLOBAL, reservations, hosts);
+            for (auto h = hosts.begin(); h != hosts.end(); ++h) {
+                srv_cfg->getCfgHosts()->add(*h);
+            }
+        }
+
+        ConstElementPtr config_control = mutable_cfg->get("config-control");
+        if (config_control) {
+            parameter_name = "config-control";
+            ConfigControlParser parser;
+            ConfigControlInfoPtr config_ctl_info = parser.parse(config_control);
+            CfgMgr::instance().getStagingCfg()->setConfigControlInfo(config_ctl_info);
+        }
+
+        ConstElementPtr compatibility = mutable_cfg->get("compatibility");
+        if (compatibility) {
+            for (auto kv : compatibility->mapValue()) {
+                if (kv.first == "lenient-option-parsing") {
+                    CfgMgr::instance().getStagingCfg()->setLenientOptionParsing(
+                        kv.second->boolValue());
                 }
-                continue;
             }
+        }
 
-            if (config_pair.first == "subnet4") {
-                Subnets4ListConfigParser subnets_parser;
-                // parse() returns number of subnets parsed. We may log it one day.
-                subnets_parser.parse(srv_cfg, config_pair.second);
-                continue;
-            }
+        // Make parsers grouping.
+        ConfigPair config_pair;
+        const std::map<std::string, ConstElementPtr>& values_map =
+            mutable_cfg->mapValue();
 
-            if (config_pair.first == "shared-networks") {
+        BOOST_FOREACH(config_pair, values_map) {
 
-                /// We need to create instance of SharedNetworks4ListParser
-                /// and parse the list of the shared networks into the
-                /// CfgSharedNetworks4 object. One additional step is then to
-                /// add subnets from the CfgSharedNetworks4 into CfgSubnets4
-                /// as well.
-                SharedNetworks4ListParser parser;
-                CfgSharedNetworks4Ptr cfg = srv_cfg->getCfgSharedNetworks4();
-                parser.parse(cfg, config_pair.second);
+            parameter_name = config_pair.first;
 
-                // We also need to put the subnets it contains into normal
-                // subnets list.
-                global_parser.copySubnets4(srv_cfg->getCfgSubnets4(), cfg);
-                continue;
-            }
-
-            if (config_pair.first == "reservations") {
-                HostCollection hosts;
-                HostReservationsListParser<HostReservationParser4> parser;
-                parser.parse(SUBNET_ID_GLOBAL, config_pair.second, hosts);
-                for (auto h = hosts.begin(); h != hosts.end(); ++h) {
-                    srv_cfg->getCfgHosts()->add(*h);
-                }
-
-                continue;
-            }
-
-            if (config_pair.first == "config-control") {
-                ConfigControlParser parser;
-                ConfigControlInfoPtr config_ctl_info = parser.parse(config_pair.second);
-                CfgMgr::instance().getStagingCfg()->setConfigControlInfo(config_ctl_info);
+            // These are converted to SimpleParser and are handled already above.
+            if ((config_pair.first == "option-def") ||
+                (config_pair.first == "option-data") ||
+                (config_pair.first == "control-socket") ||
+                (config_pair.first == "multi-threading") ||
+                (config_pair.first == "dhcp-queue-control") ||
+                (config_pair.first == "host-reservation-identifiers") ||
+                (config_pair.first == "interfaces-config") ||
+                (config_pair.first == "sanity-checks") ||
+                (config_pair.first == "expired-leases-processing") ||
+                (config_pair.first == "hooks-libraries") ||
+                (config_pair.first == "dhcp-ddns") ||
+                (config_pair.first == "client-classes") ||
+                (config_pair.first == "lease-database") ||
+                (config_pair.first == "hosts-database") ||
+                (config_pair.first == "hosts-databases") ||
+                (config_pair.first == "subnet4") ||
+                (config_pair.first == "shared-networks") ||
+                (config_pair.first == "reservations") ||
+                (config_pair.first == "config-control") ||
+                (config_pair.first == "compatibility")) {
                 continue;
             }
 
@@ -558,17 +646,33 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
                  (config_pair.first == "boot-file-name") ||
                  (config_pair.first == "server-tag") ||
                  (config_pair.first == "reservation-mode") ||
+                 (config_pair.first == "reservations-global") ||
+                 (config_pair.first == "reservations-in-subnet") ||
+                 (config_pair.first == "reservations-out-of-pool") ||
                  (config_pair.first == "calculate-tee-times") ||
                  (config_pair.first == "t1-percent") ||
                  (config_pair.first == "t2-percent") ||
+                 (config_pair.first == "cache-threshold") ||
+                 (config_pair.first == "cache-max-age") ||
                  (config_pair.first == "loggers") ||
                  (config_pair.first == "hostname-char-set") ||
-                 (config_pair.first == "hostname-char-replacement")) {
-
+                 (config_pair.first == "hostname-char-replacement") ||
+                 (config_pair.first == "ddns-send-updates") ||
+                 (config_pair.first == "ddns-override-no-update") ||
+                 (config_pair.first == "ddns-override-client-update") ||
+                 (config_pair.first == "ddns-replace-client-name") ||
+                 (config_pair.first == "ddns-generated-prefix") ||
+                 (config_pair.first == "ddns-qualifying-suffix") ||
+                 (config_pair.first == "ddns-update-on-renew") ||
+                 (config_pair.first == "ddns-use-conflict-resolution") ||
+                 (config_pair.first == "store-extended-info") ||
+                 (config_pair.first == "statistic-default-sample-count") ||
+                 (config_pair.first == "statistic-default-sample-age") ||
+                 (config_pair.first == "ip-reservations-unique") ||
+                 (config_pair.first == "parked-packet-limit")) {
                 CfgMgr::instance().getStagingCfg()->addConfiguredGlobal(config_pair.first,
                                                                         config_pair.second);
                 continue;
-
             }
 
             // Nothing to configure for the user-context.
@@ -582,6 +686,9 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
                       << " (" << config_pair.second->getPosition() << ")");
         }
 
+        // Reset parameter name.
+        parameter_name = "<post parsing>";
+
         // Apply global options in the staging config.
         global_parser.parse(srv_cfg, mutable_cfg);
 
@@ -590,27 +697,22 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
         // defined as part of shared networks.
         global_parser.sanityChecks(srv_cfg, mutable_cfg);
 
-        // Validate D2 client confuguration.
+        // Validate D2 client configuration.
         if (!d2_client_cfg) {
             d2_client_cfg.reset(new D2ClientConfig());
-            d2_client_cfg->setFetchGlobalsFn([]() -> ConstElementPtr {
-                return (CfgMgr::instance().getStagingCfg()->getConfiguredGlobals());
-             });
         }
         d2_client_cfg->validateContents();
         srv_cfg->setD2ClientConfig(d2_client_cfg);
-
     } catch (const isc::Exception& ex) {
         LOG_ERROR(dhcp4_logger, DHCP4_PARSER_FAIL)
-                  .arg(config_pair.first).arg(ex.what());
+                  .arg(parameter_name).arg(ex.what());
         answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
 
         // An error occurred, so make sure that we restore original data.
         rollback = true;
-
     } catch (...) {
         // For things like bad_cast in boost::lexical_cast
-        LOG_ERROR(dhcp4_logger, DHCP4_PARSER_EXCEPTION).arg(config_pair.first);
+        LOG_ERROR(dhcp4_logger, DHCP4_PARSER_EXCEPTION).arg(parameter_name);
         answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration"
                                            " processing error");
 
@@ -633,6 +735,7 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
     // This operation should be exception safe but let's make sure.
     if (!rollback) {
         try {
+
             // Setup the command channel.
             configureCommandChannel();
 
@@ -644,9 +747,11 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
             cfg = CfgMgr::instance().getStagingCfg()->getD2ClientConfig();
             CfgMgr::instance().setD2ClientConfig(cfg);
 
-            // This occurs last as if it succeeds, there is no easy way
+            // This occurs last as if it succeeds, there is no easy way to
             // revert it.  As a result, the failure to commit a subsequent
             // change causes problems when trying to roll back.
+            HooksManager::prepareUnloadLibraries();
+            static_cast<void>(HooksManager::unloadLibraries());
             const HooksConfig& libraries =
                 CfgMgr::instance().getStagingCfg()->getHooksConfig();
             libraries.loadLibraries();
@@ -654,12 +759,16 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
         catch (const isc::Exception& ex) {
             LOG_ERROR(dhcp4_logger, DHCP4_PARSER_COMMIT_FAIL).arg(ex.what());
             answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
+
+            // An error occurred, so make sure to restore the original data.
             rollback = true;
         } catch (...) {
             // For things like bad_cast in boost::lexical_cast
             LOG_ERROR(dhcp4_logger, DHCP4_PARSER_COMMIT_EXCEPTION);
             answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, "undefined configuration"
                                                " parsing error");
+
+            // An error occurred, so make sure to restore the original data.
             rollback = true;
         }
     }
@@ -667,6 +776,7 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
     // Moved from the commit block to add the config backend indication.
     if (!rollback) {
         try {
+
             // If there are config backends, fetch and merge into staging config
             server.getCBControl()->databaseConfigFetch(srv_cfg,
                                                        CBControlDHCPv4::FetchMode::FETCH_ALL);
@@ -676,6 +786,8 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
             err << "during update from config backend database: " << ex.what();
             LOG_ERROR(dhcp4_logger, DHCP4_PARSER_COMMIT_FAIL).arg(err.str());
             answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str());
+
+            // An error occurred, so make sure to restore the original data.
             rollback = true;
         } catch (...) {
             // For things like bad_cast in boost::lexical_cast
@@ -684,6 +796,8 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
                 << "undefined configuration parsing error";
             LOG_ERROR(dhcp4_logger, DHCP4_PARSER_COMMIT_FAIL).arg(err.str());
             answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str());
+
+            // An error occurred, so make sure to restore the original data.
             rollback = true;
         }
     }
@@ -705,5 +819,5 @@ configureDhcp4Server(Dhcpv4Srv& server, isc::data::ConstElementPtr config_set,
     return (answer);
 }
 
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+}  // namespace dhcp
+}  // namespace isc
