@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -583,7 +583,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelNegative) {
               response);
 }
 
-// Tests that the server properly responds to shtudown command sent
+// Tests that the server properly responds to shutdown command sent
 // via ControlChannel
 TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelShutdown) {
     createUnixChannelServer();
@@ -667,11 +667,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelStats) {
     EXPECT_EQ("{ \"result\": 1, \"text\": \"No 'bogus' statistic found\" }",
               response);
 
-    // Check statistic-remove-all
-    sendUnixCommand("{ \"command\" : \"statistic-remove-all\", "
-                    "  \"arguments\": {}}", response);
-    EXPECT_EQ("{ \"result\": 0, \"text\": \"All statistics removed.\" }",
-              response);
+    // Check statistic-remove-all (deprecated).
 
     // Check statistic-sample-age-set
     sendUnixCommand("{ \"command\" : \"statistic-sample-age-set\", "
@@ -1122,7 +1118,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, statusGet) {
 
     auto found_multi_threading = arguments->get("multi-threading-enabled");
     ASSERT_TRUE(found_multi_threading);
-    EXPECT_EQ(false, found_multi_threading->boolValue());
+    EXPECT_FALSE(found_multi_threading->boolValue());
 
     auto found_thread_count = arguments->get("thread-pool-size");
     ASSERT_FALSE(found_thread_count);
@@ -1130,6 +1126,9 @@ TEST_F(CtrlChannelDhcpv4SrvTest, statusGet) {
     auto found_queue_size = arguments->get("packet-queue-size");
     ASSERT_FALSE(found_queue_size);
 
+    auto found_queue_stats = arguments->get("packet-queue-statistics");
+    ASSERT_FALSE(found_queue_stats);
+
     MultiThreadingMgr::instance().setMode(true);
     MultiThreadingMgr::instance().setThreadPoolSize(4);
     MultiThreadingMgr::instance().setPacketQueueSize(64);
@@ -1165,7 +1164,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, statusGet) {
 
     found_multi_threading = arguments->get("multi-threading-enabled");
     ASSERT_TRUE(found_multi_threading);
-    EXPECT_EQ(true, found_multi_threading->boolValue());
+    EXPECT_TRUE(found_multi_threading->boolValue());
 
     found_thread_count = arguments->get("thread-pool-size");
     ASSERT_TRUE(found_thread_count);
@@ -1174,6 +1173,11 @@ TEST_F(CtrlChannelDhcpv4SrvTest, statusGet) {
     found_queue_size = arguments->get("packet-queue-size");
     ASSERT_TRUE(found_queue_size);
     EXPECT_EQ(found_queue_size->intValue(), 64);
+
+    found_queue_stats = arguments->get("packet-queue-statistics");
+    ASSERT_TRUE(found_queue_stats);
+    ASSERT_EQ(Element::list, found_queue_stats->getType());
+    EXPECT_EQ(3, found_queue_stats->size());
 }
 
 // This test verifies that the DHCP server handles config-backend-pull command
@@ -1286,7 +1290,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlLeasesReclaimRemove) {
     ASSERT_FALSE(lease1);
 }
 
-// Tests that the server properly responds to shtudown command sent
+// Tests that the server properly responds to shutdown command sent
 // via ControlChannel
 TEST_F(CtrlChannelDhcpv4SrvTest, listCommands) {
     createUnixChannelServer();
@@ -1441,6 +1445,56 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configReloadValid) {
     ::remove("test8.json");
 }
 
+// This test verifies that disable DHCP service command performs sanity check on
+// parameters.
+TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisableBadParam) {
+    createUnixChannelServer();
+    std::string response;
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"max-period\": -3"
+                    "    }"
+                    "}", response);
+    ConstElementPtr rsp;
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"'max-period' must be positive "
+              "integer\" }", response);
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"invalid value used for 'origin' "
+              "parameter: (empty string)\" }", response);
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"test\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"invalid value used for 'origin' "
+              "parameter: test\" }", response);
+}
+
 // This test verifies if it is possible to disable DHCP service via command.
 TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisable) {
     createUnixChannelServer();
@@ -1458,6 +1512,50 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisable) {
     EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
 
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->enableService(NetworkState::Origin::USER_COMMAND);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"user\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->enableService(NetworkState::Origin::USER_COMMAND);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"ha-partner\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->enableService(NetworkState::Origin::HA_COMMAND);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 }
 
 // This test verifies that it is possible to disable DHCP service for a short
@@ -1490,6 +1588,42 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisableTemporarily) {
     EXPECT_TRUE(server_->network_state_->isDelayedEnableAll());
 }
 
+// This test verifies that enable DHCP service command performs sanity check on
+// parameters.
+TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnableBadParam) {
+    createUnixChannelServer();
+    std::string response;
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"\""
+                    "    }"
+                    "}", response);
+    ConstElementPtr rsp;
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"invalid value used for 'origin' "
+              "parameter: (empty string)\" }", response);
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"test\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"invalid value used for 'origin' "
+              "parameter: test\" }", response);
+}
+
 // This test verifies if it is possible to enable DHCP service via command.
 TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnable) {
     createUnixChannelServer();
@@ -1507,6 +1641,46 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnable) {
     EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->disableService(NetworkState::Origin::USER_COMMAND);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"user\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->disableService(NetworkState::Origin::HA_COMMAND);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": \"ha-partner\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 }
 
 /// Verify that concurrent connections over the control channel can be
