@@ -110,7 +110,7 @@ const char* INFORM_CONFIGS[] = {
         "\"valid-lifetime\": 600,"
         "\"next-server\": \"10.0.0.1\","
         "\"server-hostname\": \"nohost\","
-        "\"server-hostname\": \"nofile\","
+        "\"boot-file-name\": \"nofile\","
         "\"subnet4\": [ { "
         "    \"subnet\": \"192.0.2.0/24\", "
         "    \"reservations\": [ "
@@ -135,8 +135,6 @@ public:
     InformTest()
         : Dhcpv4SrvTest(),
           iface_mgr_test_config_(true) {
-        IfaceMgr::instance().openSockets4();
-
         // Let's wipe all existing statistics.
         isc::stats::StatsMgr::instance().removeAll();
     }
