@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -71,7 +71,8 @@ public:
 
     /// @brief Constructor.
     TestCBControlDHCPv4()
-        : CBControlDHCPv4(), db_config_fetch_calls_(0),
+        : CBControlDHCPv4(), db_total_config_fetch_calls_(0),
+          db_current_config_fetch_calls_(0), db_staging_config_fetch_calls_(0),
           enable_check_fetch_mode_(false), enable_throw_(false) {
     }
 
@@ -84,33 +85,55 @@ public:
     /// It also throws an exception when desired by a test, to
     /// verify that the server gracefully handles such exception.
     ///
+    /// @param config either the staging or the current configuration.
     /// @param fetch_mode value indicating if the method is called upon the
     /// server start up or it is called to fetch configuration updates.
     ///
     /// @throw Unexpected when configured to do so.
-    virtual void databaseConfigFetch(const process::ConfigPtr&,
+    virtual void databaseConfigFetch(const process::ConfigPtr& config,
                                      const FetchMode& fetch_mode) {
-        ++db_config_fetch_calls_;
+        ++db_total_config_fetch_calls_;
+
+        if (config == CfgMgr::instance().getCurrentCfg()) {
+            ++db_current_config_fetch_calls_;
+        } else if (config == CfgMgr::instance().getStagingCfg()) {
+            ++db_staging_config_fetch_calls_;
+        }
 
         if (enable_check_fetch_mode_) {
-            if ((db_config_fetch_calls_ <= 1) && (fetch_mode == FetchMode::FETCH_UPDATE)) {
+            if ((db_total_config_fetch_calls_ <= 1) &&
+                (fetch_mode == FetchMode::FETCH_UPDATE)) {
                 ADD_FAILURE() << "databaseConfigFetch was called with the value "
                     "of fetch_mode=FetchMode::FETCH_UPDATE upon the server configuration";
 
-            } else if ((db_config_fetch_calls_ > 1) && (fetch_mode == FetchMode::FETCH_ALL)) {
+            } else if ((db_total_config_fetch_calls_ > 1) &&
+                       (fetch_mode == FetchMode::FETCH_ALL)) {
                 ADD_FAILURE() << "databaseConfigFetch was called with the value "
                     "of fetch_mode=FetchMode::FETCH_ALL during fetching the updates";
             }
         }
 
         if (enable_throw_) {
-            isc_throw(Unexpected, "testing if exceptions are corectly handled");
+            isc_throw(Unexpected, "testing if exceptions are correctly handled");
         }
     }
 
-    /// @brief Returns number of invocations of the @c databaseConfigFetch.
-    size_t getDatabaseConfigFetchCalls() const {
-        return (db_config_fetch_calls_);
+    /// @brief Returns number of invocations of the @c databaseConfigFetch
+    /// (total).
+    size_t getDatabaseTotalConfigFetchCalls() const {
+        return (db_total_config_fetch_calls_);
+    }
+
+    /// @brief Returns number of invocations of the @c databaseConfigFetch
+    /// (current configuration).
+    size_t getDatabaseCurrentConfigFetchCalls() const {
+        return (db_current_config_fetch_calls_);
+    }
+
+    /// @brief Returns number of invocations of the @c databaseConfigFetch
+    /// (staging configuration).
+    size_t getDatabaseStagingConfigFetchCalls() const {
+        return (db_staging_config_fetch_calls_);
     }
 
     /// @brief Enables checking of the @c fetch_mode value.
@@ -125,8 +148,17 @@ public:
 
 private:
 
-    /// @brief Counter holding number of invocations of the @c databaseConfigFetch.
-    size_t db_config_fetch_calls_;
+    /// @brief Counter holding number of invocations of the
+    /// @c databaseConfigFetch (total).
+    size_t db_total_config_fetch_calls_;
+
+    /// @brief Counter holding number of invocations of the
+    /// @c databaseConfigFetch (current configuration).
+    size_t db_current_config_fetch_calls_;
+
+    /// @brief Counter holding number of invocations of the
+    /// @c databaseConfigFetch (staging configuration).
+    size_t db_staging_config_fetch_calls_;
 
     /// @brief Boolean flag indicated if the value of the @c fetch_mode
     /// should be verified.
@@ -154,8 +186,6 @@ public:
         // stub implementation used in tests.
         cb_control_.reset(new TestCBControlDHCPv4());
     }
-
-    using ControlledDhcpv4Srv::signal_handler_;
 };
 
 /// @brief test class for Kea configuration backend
@@ -217,7 +247,8 @@ public:
     /// @brief This test verifies that the timer used to fetch the configuration
     /// updates from the database works as expected.
     void testConfigBackendTimer(const int config_wait_fetch_time,
-                                const bool throw_during_fetch = false) {
+                                const bool throw_during_fetch = false,
+                                const bool call_command = false) {
         std::ostringstream config;
         config <<
             "{ \"Dhcp4\": {"
@@ -257,11 +288,58 @@ public:
         }
 
         // So far there should be exactly one attempt to fetch the configuration
-        // from the backend. That's the attempt made upon startup.
-        EXPECT_EQ(1, cb_control->getDatabaseConfigFetchCalls());
+        // from the backend. That's the attempt made upon startup on
+        // the staging configuration.
+        // All other fetches will be on the current configuration:
+        //  - the timer makes a closure with the staging one but it is
+        //    committed so becomes the current one.
+        //  - the command is called outside configuration so it must
+        //    be the current configuration. The test explicitly checks this.
+        EXPECT_EQ(1, cb_control->getDatabaseTotalConfigFetchCalls());
+        EXPECT_EQ(0, cb_control->getDatabaseCurrentConfigFetchCalls());
+        EXPECT_EQ(1, cb_control->getDatabaseStagingConfigFetchCalls());
 
 
-        if ((config_wait_fetch_time > 0) && (!throw_during_fetch)) {
+        if (call_command) {
+            // The case where there is no backend is tested in the
+            // controlled server tests so we have only to verify
+            // that the command calls the database config fetch.
+
+            // Count the startup.
+            EXPECT_EQ(cb_control->getDatabaseTotalConfigFetchCalls(), 1);
+            EXPECT_EQ(cb_control->getDatabaseCurrentConfigFetchCalls(), 0);
+            EXPECT_EQ(cb_control->getDatabaseStagingConfigFetchCalls(), 1);
+
+            ConstElementPtr result =
+                ControlledDhcpv4Srv::processCommand("config-backend-pull",
+                                                    ConstElementPtr());
+            EXPECT_EQ(cb_control->getDatabaseTotalConfigFetchCalls(), 2);
+            std::string expected;
+
+            if (throw_during_fetch) {
+                expected = "{ \"result\": 1, \"text\": ";
+                expected += "\"On demand configuration update failed: ";
+                expected += "testing if exceptions are correctly handled\" }";
+            } else {
+                expected = "{ \"result\": 0, \"text\": ";
+                expected += "\"On demand configuration update successful.\" }";
+            }
+            EXPECT_EQ(expected, result->str());
+
+            // No good way to check the rescheduling...
+            ASSERT_NO_THROW(runTimersWithTimeout(srv->getIOService(), 20));
+
+            if (config_wait_fetch_time > 0) {
+                EXPECT_GE(cb_control->getDatabaseTotalConfigFetchCalls(), 5);
+                EXPECT_GE(cb_control->getDatabaseCurrentConfigFetchCalls(), 4);
+                EXPECT_EQ(cb_control->getDatabaseStagingConfigFetchCalls(), 1);
+            } else {
+                EXPECT_EQ(cb_control->getDatabaseTotalConfigFetchCalls(), 2);
+                EXPECT_EQ(cb_control->getDatabaseCurrentConfigFetchCalls(), 1);
+                EXPECT_EQ(cb_control->getDatabaseStagingConfigFetchCalls(), 1);
+            }
+
+        } else if ((config_wait_fetch_time > 0) && (!throw_during_fetch)) {
             // If we're configured to run the timer, we expect that it was
             // invoked at least 3 times. This is sufficient to verify that
             // the timer was scheduled and that the timer continued to run
@@ -271,9 +349,11 @@ public:
                 [cb_control]() {
                     // Interrupt the timers poll if we have recorded at
                     // least 3 attempts to fetch the updates.
-                    return (cb_control->getDatabaseConfigFetchCalls() >= 3);
+                    return (cb_control->getDatabaseTotalConfigFetchCalls() >= 3);
                 }));
-            EXPECT_GE(cb_control->getDatabaseConfigFetchCalls(), 3);
+            EXPECT_GE(cb_control->getDatabaseTotalConfigFetchCalls(), 3);
+            EXPECT_GE(cb_control->getDatabaseCurrentConfigFetchCalls(), 2);
+            EXPECT_EQ(cb_control->getDatabaseStagingConfigFetchCalls(), 1);
 
         } else {
             ASSERT_NO_THROW(runTimersWithTimeout(srv->getIOService(), 500));
@@ -284,12 +364,16 @@ public:
                 // the number of recorded fetches should be 12. One at
                 // startup, 10 failures and one that causes the timer
                 // to stop.
-                EXPECT_EQ(12, cb_control->getDatabaseConfigFetchCalls());
+                EXPECT_EQ(12, cb_control->getDatabaseTotalConfigFetchCalls());
+                EXPECT_EQ(11, cb_control->getDatabaseCurrentConfigFetchCalls());
+                EXPECT_EQ(1, cb_control->getDatabaseStagingConfigFetchCalls());
 
             } else {
                 // If the server is not configured to schedule the timer,
                 // we should still have one fetch attempt recorded.
-                EXPECT_EQ(1, cb_control->getDatabaseConfigFetchCalls());
+                EXPECT_EQ(1, cb_control->getDatabaseTotalConfigFetchCalls());
+                EXPECT_EQ(0, cb_control->getDatabaseCurrentConfigFetchCalls());
+                EXPECT_EQ(1, cb_control->getDatabaseStagingConfigFetchCalls());
             }
         }
     }
@@ -345,65 +429,45 @@ TEST_F(JSONFileBackendTest, jsonFile) {
     ASSERT_TRUE(subnets);
     ASSERT_EQ(3, subnets->size()); // We expect 3 subnets.
 
-
     // Check subnet 1.
-    EXPECT_EQ("192.0.2.0", subnets->at(0)->get().first.toText());
-    EXPECT_EQ(24, subnets->at(0)->get().second);
+    auto subnet = subnets->begin();
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.2.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(24, (*subnet)->get().second);
 
     // Check pools in the first subnet.
-    const PoolCollection& pools1 = subnets->at(0)->getPools(Lease::TYPE_V4);
+    const PoolCollection& pools1 = (*subnet)->getPools(Lease::TYPE_V4);
     ASSERT_EQ(1, pools1.size());
     EXPECT_EQ("192.0.2.1", pools1.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.2.100", pools1.at(0)->getLastAddress().toText());
     EXPECT_EQ(Lease::TYPE_V4, pools1.at(0)->getType());
 
     // Check subnet 2.
-    EXPECT_EQ("192.0.3.0", subnets->at(1)->get().first.toText());
-    EXPECT_EQ(24, subnets->at(1)->get().second);
+    ++subnet;
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.3.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(24, (*subnet)->get().second);
 
     // Check pools in the second subnet.
-    const PoolCollection& pools2 = subnets->at(1)->getPools(Lease::TYPE_V4);
+    const PoolCollection& pools2 = (*subnet)->getPools(Lease::TYPE_V4);
     ASSERT_EQ(1, pools2.size());
     EXPECT_EQ("192.0.3.101", pools2.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.3.150", pools2.at(0)->getLastAddress().toText());
     EXPECT_EQ(Lease::TYPE_V4, pools2.at(0)->getType());
 
     // And finally check subnet 3.
-    EXPECT_EQ("192.0.4.0", subnets->at(2)->get().first.toText());
-    EXPECT_EQ(24, subnets->at(2)->get().second);
+    ++subnet;
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.4.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(24, (*subnet)->get().second);
 
-    // ... and it's only pool.
-    const PoolCollection& pools3 = subnets->at(2)->getPools(Lease::TYPE_V4);
+    // ... and its only pool.
+    const PoolCollection& pools3 = (*subnet)->getPools(Lease::TYPE_V4);
     EXPECT_EQ("192.0.4.101", pools3.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.4.150", pools3.at(0)->getLastAddress().toText());
     EXPECT_EQ(Lease::TYPE_V4, pools3.at(0)->getType());
 }
 
-// This test verifies that the configurations for various servers
-// can coexist and that the DHCPv4 configuration parsers will simply
-// ignore them.
-TEST_F(JSONFileBackendTest, serverConfigurationsCoexistence) {
-    std::string config = "{ \"Dhcp4\": {"
-        "\"rebind-timer\": 2000, "
-        "\"renew-timer\": 1000, \n"
-        "\"valid-lifetime\": 4000 }, "
-        "\"Dhcp6\": { },"
-        "\"DhcpDdns\": { },"
-        "\"Control-agent\": { }"
-        "}";
-
-    writeFile(TEST_FILE, config);
-
-    // Now initialize the server
-    boost::scoped_ptr<ControlledDhcpv4Srv> srv;
-    ASSERT_NO_THROW(
-        srv.reset(new ControlledDhcpv4Srv(0))
-    );
-
-    // And configure it using the config file.
-    EXPECT_NO_THROW(srv->init(TEST_FILE));
-}
-
 // This test checks if configuration can be read from a JSON file
 // using hash (#) line comments
 TEST_F(JSONFileBackendTest, hashComments) {
@@ -442,11 +506,13 @@ TEST_F(JSONFileBackendTest, hashComments) {
     ASSERT_EQ(1, subnets->size());
 
     // Check subnet 1.
-    EXPECT_EQ("192.0.2.0", subnets->at(0)->get().first.toText());
-    EXPECT_EQ(22, subnets->at(0)->get().second);
+    auto subnet = subnets->begin();
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.2.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(22, (*subnet)->get().second);
 
     // Check pools in the first subnet.
-    const PoolCollection& pools1 = subnets->at(0)->getPools(Lease::TYPE_V4);
+    const PoolCollection& pools1 = (*subnet)->getPools(Lease::TYPE_V4);
     ASSERT_EQ(1, pools1.size());
     EXPECT_EQ("192.0.2.0", pools1.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.2.255", pools1.at(0)->getLastAddress().toText());
@@ -491,11 +557,13 @@ TEST_F(JSONFileBackendTest, cppLineComments) {
     ASSERT_EQ(1, subnets->size());
 
     // Check subnet 1.
-    EXPECT_EQ("192.0.2.0", subnets->at(0)->get().first.toText());
-    EXPECT_EQ(22, subnets->at(0)->get().second);
+    auto subnet = subnets->begin();
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.2.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(22, (*subnet)->get().second);
 
     // Check pools in the first subnet.
-    const PoolCollection& pools1 = subnets->at(0)->getPools(Lease::TYPE_V4);
+    const PoolCollection& pools1 = (*subnet)->getPools(Lease::TYPE_V4);
     ASSERT_EQ(1, pools1.size());
     EXPECT_EQ("192.0.2.0", pools1.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.2.255", pools1.at(0)->getLastAddress().toText());
@@ -507,7 +575,7 @@ TEST_F(JSONFileBackendTest, cppLineComments) {
 TEST_F(JSONFileBackendTest, cBlockComments) {
 
     string config_c_block_comments = "/* This is a comment. It should be \n"
-      "ignored. Real config starts in line below*/\n"
+        "ignored. Real config starts in line below*/\n"
         "{ \"Dhcp4\": {"
         "\"interfaces-config\": {"
         "    \"interfaces\": [ \"*\" ]"
@@ -540,11 +608,13 @@ TEST_F(JSONFileBackendTest, cBlockComments) {
     ASSERT_EQ(1, subnets->size());
 
     // Check subnet 1.
-    EXPECT_EQ("192.0.2.0", subnets->at(0)->get().first.toText());
-    EXPECT_EQ(22, subnets->at(0)->get().second);
+    auto subnet = subnets->begin();
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.2.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(22, (*subnet)->get().second);
 
     // Check pools in the first subnet.
-    const PoolCollection& pools1 = subnets->at(0)->getPools(Lease::TYPE_V4);
+    const PoolCollection& pools1 = (*subnet)->getPools(Lease::TYPE_V4);
     ASSERT_EQ(1, pools1.size());
     EXPECT_EQ("192.0.2.0", pools1.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.2.255", pools1.at(0)->getLastAddress().toText());
@@ -589,11 +659,13 @@ TEST_F(JSONFileBackendTest, include) {
     ASSERT_EQ(1, subnets->size());
 
     // Check subnet 1.
-    EXPECT_EQ("192.0.2.0", subnets->at(0)->get().first.toText());
-    EXPECT_EQ(22, subnets->at(0)->get().second);
+    auto subnet = subnets->begin();
+    ASSERT_TRUE(subnet != subnets->end());
+    EXPECT_EQ("192.0.2.0", (*subnet)->get().first.toText());
+    EXPECT_EQ(22, (*subnet)->get().second);
 
     // Check pools in the first subnet.
-    const PoolCollection& pools1 = subnets->at(0)->getPools(Lease::TYPE_V4);
+    const PoolCollection& pools1 = (*subnet)->getPools(Lease::TYPE_V4);
     ASSERT_EQ(1, pools1.size());
     EXPECT_EQ("192.0.2.0", pools1.at(0)->getFirstAddress().toText());
     EXPECT_EQ("192.0.2.255", pools1.at(0)->getLastAddress().toText());
@@ -857,6 +929,24 @@ TEST_F(JSONFileBackendTest, configBackendTimerWithThrow) {
     testConfigBackendTimer(1, true);
 }
 
+// This test verifies that the server will be updated by the
+// config-backend-pull command.
+TEST_F(JSONFileBackendTest, configBackendPullCommand) {
+    testConfigBackendTimer(0, false, true);
+}
+
+// This test verifies that the server will be updated by the
+// config-backend-pull command even when updates fail.
+TEST_F(JSONFileBackendTest, configBackendPullCommandWithThrow) {
+    testConfigBackendTimer(0, true, true);
+}
+
+// This test verifies that the server will be updated by the
+// config-backend-pull command and the timer rescheduled.
+TEST_F(JSONFileBackendTest, configBackendPullCommandWithTimer) {
+    testConfigBackendTimer(1, false, true);
+}
+
 // Starting tests which require MySQL backend availability. Those tests
 // will not be executed if Kea has been compiled without the
 // --with-mysql.
@@ -968,7 +1058,10 @@ testBackendReconfiguration(const std::string& backend_first,
 
     // Explicitly calling signal handler for SIGHUP to trigger server
     // reconfiguration.
-    srv->signal_handler_(SIGHUP);
+    raise(SIGHUP);
+
+    // Polling once to be sure that the signal handle has been called.
+    srv->getIOService()->poll();
 
     // The backend should have been created and its type should be
     // correct.
