@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -74,6 +74,26 @@ namespace {
 ///   - 2 pool: 2001:db8:1:1::/64
 ///   - the following class defined: option 1234 'foo', DROP
 ///
+/// - Configuration 4:
+///   - Used for the DROP class and reservation existence
+///   - 1 subnet: 2001:db8:1::/48
+///   - 2 pool: 2001:db8:1:1::/64
+///   - the following class defined: not member('KNOWN'), DROP
+/// @note the reservation includes a hostname because raw reservations are
+/// not yet allowed
+///
+/// - Configuration 5:
+///   - Used for the DROP class and reservation class
+///   - 1 subnet: 2001:db8:1::/48
+///   - 2 pool: 2001:db8:1:1::/64
+///   - the following class defined:
+///     - allowed
+///     - member('KNOWN') or member('UNKNOWN'), t
+///     - not member('allowed') and member('t'), DROP
+///     The function of the always true 't' class is to move the DROP
+///     evaluation to the classification point after the host reservation
+///     lookup, i.e. indirect KNOWN / UNKNOWN dependency
+///
 const char* CONFIGS[] = {
     // Configuration 0
     "{ \"interfaces-config\": {"
@@ -287,6 +307,63 @@ const char* CONFIGS[] = {
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"interface\": \"eth1\""
         " } ],"
+        "\"valid-lifetime\": 4000 }",
+
+    // Configuration 4
+    "{ \"interfaces-config\": {"
+        "  \"interfaces\": [ \"*\" ]"
+        "},"
+        "\"preferred-lifetime\": 3000,"
+        "\"rebind-timer\": 2000, "
+        "\"renew-timer\": 1000, "
+        "\"client-classes\": ["
+        "{"
+        "   \"name\": \"DROP\","
+        "   \"test\": \"not member('KNOWN')\""
+        "}"
+        "],"
+        "\"subnet6\": [ "
+        "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"subnet\": \"2001:db8:1::/48\", "
+        "    \"interface\": \"eth1\","
+        "    \"reservations\": ["
+        "    {"
+        "        \"duid\": \"01:02:03:04\","
+        "        \"hostname\": \"allowed\""
+        "    } ]"
+        " } ],"
+        "\"valid-lifetime\": 4000 }",
+
+    // Configuration 5
+    "{ \"interfaces-config\": {"
+        "  \"interfaces\": [ \"*\" ]"
+        "},"
+        "\"preferred-lifetime\": 3000,"
+        "\"rebind-timer\": 2000, "
+        "\"renew-timer\": 1000, "
+        "\"client-classes\": ["
+        "{"
+        "   \"name\": \"allowed\""
+        "},"
+        "{"
+        "   \"name\": \"t\","
+        "   \"test\": \"member('KNOWN') or member('UNKNOWN')\""
+        "},"
+        "{"
+        "   \"name\": \"DROP\","
+        "   \"test\": \"not member('allowed') and member('t')\""
+        "}"
+        "],"
+        "\"subnet6\": [ "
+        "{   \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ], "
+        "    \"subnet\": \"2001:db8:1::/48\", "
+        "    \"interface\": \"eth1\","
+        "    \"reservations\": ["
+        "    {"
+        "        \"duid\": \"01:02:03:04\","
+        "        \"client-classes\": [ \"allowed\" ]"
+        "    } ]"
+        " } ],"
         "\"valid-lifetime\": 4000 }"
 };
 
@@ -366,6 +443,7 @@ public:
             query->setRemoteAddr(IOAddress(remote_addr));
             query->addOption(clientid);
             query->setIface("eth1");
+            query->setIndex(ETH1_INDEX);
             query->addOption(generateIA(D6O_IA_NA, 123, 1500, 3000));
             return (query);
     }
@@ -533,16 +611,19 @@ TEST_F(ClassifyTest, required) {
     query1->setRemoteAddr(IOAddress("fe80::abcd"));
     query1->addOption(clientid);
     query1->setIface("eth1");
+    query1->setIndex(ETH1_INDEX);
     query1->addOption(generateIA(D6O_IA_NA, 123, 1500, 3000));
     Pkt6Ptr query2(new Pkt6(DHCPV6_SOLICIT, 1234));
     query2->setRemoteAddr(IOAddress("fe80::abcd"));
     query2->addOption(clientid);
     query2->setIface("eth1");
+    query2->setIndex(ETH1_INDEX);
     query2->addOption(generateIA(D6O_IA_NA, 234, 1500, 3000));
     Pkt6Ptr query3(new Pkt6(DHCPV6_SOLICIT, 1234));
     query3->setRemoteAddr(IOAddress("fe80::abcd"));
     query3->addOption(clientid);
     query3->setIface("eth1");
+    query3->setIndex(ETH1_INDEX);
     query3->addOption(generateIA(D6O_IA_NA, 345, 1500, 3000));
 
     // Create and add an ORO option to the first 2 queries
@@ -633,16 +714,19 @@ TEST_F(ClassifyTest, requiredClassification) {
     query1->setRemoteAddr(IOAddress("fe80::abcd"));
     query1->addOption(clientid);
     query1->setIface("eth1");
+    query1->setIndex(ETH1_INDEX);
     query1->addOption(generateIA(D6O_IA_NA, 123, 1500, 3000));
     Pkt6Ptr query2(new Pkt6(DHCPV6_SOLICIT, 1234));
     query2->setRemoteAddr(IOAddress("fe80::abcd"));
     query2->addOption(clientid);
     query2->setIface("eth1");
+    query2->setIndex(ETH1_INDEX);
     query2->addOption(generateIA(D6O_IA_NA, 234, 1500, 3000));
     Pkt6Ptr query3(new Pkt6(DHCPV6_SOLICIT, 1234));
     query3->setRemoteAddr(IOAddress("fe80::abcd"));
     query3->addOption(clientid);
     query3->setIface("eth1");
+    query3->setIndex(ETH1_INDEX);
     query3->addOption(generateIA(D6O_IA_NA, 345, 1500, 3000));
 
     // Create and add an ORO option to the first 2 queries
@@ -1162,6 +1246,7 @@ TEST_F(ClassifyTest, clientClassifyPoolKnown) {
     query1->addOption(generateIA(D6O_IA_NA, 234, 1500, 3000));
     query1->addOption(clientid1);
     query1->setIface("eth1");
+    query1->setIndex(ETH1_INDEX);
 
     // First pool requires reservation so the second will be used
     srv.classifyPacket(query1);
@@ -1190,6 +1275,7 @@ TEST_F(ClassifyTest, clientClassifyPoolKnown) {
     query2->addOption(generateIA(D6O_IA_NA, 234, 1500, 3000));
     query2->addOption(clientid2);
     query2->setIface("eth1");
+    query2->setIndex(ETH1_INDEX);
 
     // Now the first pool will be used
     srv.classifyPacket(query2);
@@ -1277,8 +1363,8 @@ TEST_F(ClassifyTest, relayOverrideAndClientClass) {
     ASSERT_EQ(2, subnets->size());
 
     // Let's get them for easy reference
-    Subnet6Ptr subnet1 = (*subnets)[0];
-    Subnet6Ptr subnet2 = (*subnets)[1];
+    Subnet6Ptr subnet1 = *subnets->begin();
+    Subnet6Ptr subnet2 = *std::next(subnets->begin());
     ASSERT_TRUE(subnet1);
     ASSERT_TRUE(subnet2);
 
@@ -1433,16 +1519,19 @@ TEST_F(ClassifyTest, member) {
     query1->setRemoteAddr(IOAddress("fe80::abcd"));
     query1->addOption(clientid);
     query1->setIface("eth1");
+    query1->setIndex(ETH1_INDEX);
     query1->addOption(generateIA(D6O_IA_NA, 123, 1500, 3000));
     Pkt6Ptr query2(new Pkt6(DHCPV6_SOLICIT, 1234));
     query2->setRemoteAddr(IOAddress("fe80::abcd"));
     query2->addOption(clientid);
     query2->setIface("eth1");
+    query2->setIndex(ETH1_INDEX);
     query2->addOption(generateIA(D6O_IA_NA, 234, 1500, 3000));
     Pkt6Ptr query3(new Pkt6(DHCPV6_SOLICIT, 1234));
     query3->setRemoteAddr(IOAddress("fe80::abcd"));
     query3->addOption(clientid);
     query3->setIface("eth1");
+    query3->setIndex(ETH1_INDEX);
     query3->addOption(generateIA(D6O_IA_NA, 345, 1500, 3000));
 
     // Create and add an ORO option to queries
@@ -1607,7 +1696,6 @@ TEST_F(ClassifyTest, precedencePool) {
         "\"interfaces-config\": {"
         "   \"interfaces\": [ \"*\" ]"
         "},"
-        "\"valid-lifetime\": 600,"
         "\"client-classes\": ["
         "    {"
         "       \"name\": \"all\","
@@ -1690,7 +1778,6 @@ TEST_F(ClassifyTest, precedenceSubnet) {
         "\"interfaces-config\": {"
         "   \"interfaces\": [ \"*\" ]"
         "},"
-        "\"valid-lifetime\": 600,"
         "\"client-classes\": ["
         "    {"
         "       \"name\": \"all\","
@@ -1774,7 +1861,6 @@ TEST_F(ClassifyTest, precedenceNetwork) {
         "\"interfaces-config\": {"
         "   \"interfaces\": [ \"*\" ]"
         "},"
-        "\"valid-lifetime\": 600,"
         "\"client-classes\": ["
         "    {"
         "       \"name\": \"all\","
@@ -2118,4 +2204,80 @@ TEST_F(ClassifyTest, dropClass) {
     EXPECT_EQ(1, drop_stat->getInteger().first);
 }
 
+// This test checks the handling for the DROP special class at the host
+// reservation classification point with KNOWN / UNKNOWN.
+TEST_F(ClassifyTest, dropClassUnknown) {
+    Dhcp6Client client;
+    client.setDUID("01:02:03:04");
+    client.setInterface("eth1");
+    client.requestAddress();
+
+    // Configure DHCP server.
+    ASSERT_NO_THROW(configure(CONFIGS[4], *client.getServer()));
+
+    // Send a message to the server.
+    ASSERT_NO_THROW(client.doSolicit(true));
+
+    // Reservation match: no drop.
+    EXPECT_TRUE(client.getContext().response_);
+
+    // Retry with an option matching the DROP class.
+    Dhcp6Client client2;
+
+    // Retry with another DUID.
+    client2.setDUID("01:02:03:05");
+
+    // Send a message to the server.
+    ASSERT_NO_THROW(client2.doSolicit(true));
+
+    // No reservation, dropped.
+    EXPECT_FALSE(client2.getContext().response_);
+
+    // There should also be pkt6-receive-drop stat bumped up.
+    stats::StatsMgr& mgr = stats::StatsMgr::instance();
+    stats::ObservationPtr drop_stat = mgr.getObservation("pkt6-receive-drop");
+
+    // This statistic must be present and must be set to 1.
+    ASSERT_TRUE(drop_stat);
+    EXPECT_EQ(1, drop_stat->getInteger().first);
+}
+
+// This test checks the handling for the DROP special class at the host
+// reservation classification point with a reserved class.
+TEST_F(ClassifyTest, dropClassReservedClass) {
+    Dhcp6Client client;
+    client.setDUID("01:02:03:04");
+    client.setInterface("eth1");
+    client.requestAddress();
+
+    // Configure DHCP server.
+    ASSERT_NO_THROW(configure(CONFIGS[5], *client.getServer()));
+
+    // Send a message to the server.
+    ASSERT_NO_THROW(client.doSolicit(true));
+
+    // Reservation match: no drop.
+    EXPECT_TRUE(client.getContext().response_);
+
+    // Retry with an option matching the DROP class.
+    Dhcp6Client client2;
+
+    // Retry with another DUID.
+    client2.setDUID("01:02:03:05");
+
+    // Send a message to the server.
+    ASSERT_NO_THROW(client2.doSolicit(true));
+
+    // No reservation, dropped.
+    EXPECT_FALSE(client2.getContext().response_);
+
+    // There should also be pkt6-receive-drop stat bumped up.
+    stats::StatsMgr& mgr = stats::StatsMgr::instance();
+    stats::ObservationPtr drop_stat = mgr.getObservation("pkt6-receive-drop");
+
+    // This statistic must be present and must be set to 1.
+    ASSERT_TRUE(drop_stat);
+    EXPECT_EQ(1, drop_stat->getInteger().first);
+}
+
 } // end of anonymous namespace
