@@ -68,11 +68,16 @@ ParserContext::parseCommon() {
     }
 }
 
-
 void
-ParserContext::error(const isc::netconf::location& loc, const std::string& what)
+ParserContext::error(const isc::netconf::location& loc,
+                     const std::string& what,
+                     size_t pos)
 {
-    isc_throw(ParseError, loc << ": " << what);
+    if (pos == 0) {
+        isc_throw(ParseError, loc << ": " << what);
+    } else {
+        isc_throw(ParseError, loc << " (near " << pos << "): " << what);
+    }
 }
 
 void
@@ -117,9 +122,15 @@ ParserContext::unique(const std::string& name,
 {
     ConstElementPtr value = stack_.back()->get(name);
     if (value) {
-        isc_throw(ParseError, loc << ": duplicate " << name
-                  << " entries in " << contextName()
-                  << " map (previous at " << value->getPosition() << ")");
+        if (ctx_ != NO_KEYWORDS) {
+            isc_throw(ParseError, loc << ": duplicate " << name
+                      << " entries in " << contextName()
+                      << " map (previous at " << value->getPosition() << ")");
+        } else {
+            isc_throw(ParseError, loc << ": duplicate " << name
+                      << " entries in JSON"
+                      << " map (previous at " << value->getPosition() << ")");
+        }
     }
 }
 
@@ -153,7 +164,7 @@ ParserContext::contextName()
     case MANAGED_SERVERS:
         return ("managed-servers");
     case SERVER:
-        return ("managed-servers/*");
+        return ("managed-servers entry");
     case CONTROL_SOCKET:
         return ("control-socket");
     case SOCKET_TYPE:
@@ -169,5 +180,5 @@ ParserContext::contextName()
     }
 }
 
-};
-};
+}
+}
