@@ -1,10 +1,11 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
 #include <netconf/netconf_config.h>
 #include <netconf/http_control_socket.h>
 #include <netconf/stdout_control_socket.h>
@@ -20,10 +21,12 @@
 #include <http/tests/response_test.h>
 #include <testutils/threaded_test.h>
 #include <testutils/sandbox.h>
-#include <util/threads/thread.h>
-#include <util/threads/sync.h>
+#include <yang/tests/sysrepo_setup.h>
+
 #include <gtest/gtest.h>
+
 #include <sstream>
+#include <thread>
 
 using namespace std;
 using namespace isc;
@@ -33,12 +36,13 @@ using namespace isc::data;
 using namespace isc::http;
 using namespace isc::http::test;
 using namespace isc::test;
-using namespace isc::util::thread;
+
+using isc::yang::test::SysrepoSetup;
 
 namespace {
 
 /// @brief Type definition for the pointer to Thread objects.
-typedef boost::shared_ptr<Thread> ThreadPtr;
+typedef boost::shared_ptr<thread> ThreadPtr;
 
 //////////////////////////////// STDOUT ////////////////////////////////
 
@@ -100,7 +104,7 @@ TEST(StdoutControlSocketTest, configTest) {
     StdoutControlSocketPtr scs(new StdoutControlSocket(cfg));
     ASSERT_TRUE(scs);
     ConstElementPtr answer;
-    ASSERT_NO_THROW(answer = scs->configTest(ConstElementPtr(), "foo"));
+    ASSERT_NO_THROW_LOG(answer = scs->configTest(ConstElementPtr(), "foo"));
 
     // Check answer.
     ASSERT_TRUE(answer);
@@ -119,7 +123,7 @@ TEST(StdoutControlSocketTest, configSet) {
     ASSERT_TRUE(tscs);
     ConstElementPtr json = Element::fromJSON("{ \"bar\": 1 }");
     ConstElementPtr answer;
-    ASSERT_NO_THROW(answer = tscs->configSet(json, "foo"));
+    ASSERT_NO_THROW_LOG(answer = tscs->configSet(json, "foo"));
 
     // Check answer.
     ASSERT_TRUE(answer);
@@ -143,13 +147,17 @@ public:
     /// @brief Constructor.
     UnixControlSocketTest()
         : ThreadedTest(), io_service_() {
+    }
+
+
+    void SetUp() override {
+        SysrepoSetup::cleanSharedMemory();
         removeUnixSocketFile();
     }
 
-    /// @brief Destructor.
-    virtual ~UnixControlSocketTest() {
+    void TearDown() override {
         if (thread_) {
-            thread_->wait();
+            thread_->join();
             thread_.reset();
         }
         // io_service must be stopped after the thread returns,
@@ -308,7 +316,7 @@ TEST_F(UnixControlSocketTest, configGet) {
     ASSERT_TRUE(ucs);
 
     // Run a reflecting server in a thread.
-    thread_.reset(new Thread([this]() { reflectServer(); }));
+    thread_.reset(new thread([this]() { reflectServer(); }));
 
     waitReady();
 
@@ -332,7 +340,7 @@ TEST_F(UnixControlSocketTest, configTest) {
     ASSERT_TRUE(ucs);
 
     // Run a reflecting server in a thread.
-    thread_.reset(new Thread([this]() { reflectServer(); }));
+    thread_.reset(new thread([this]() { reflectServer(); }));
 
     waitReady();
 
@@ -359,7 +367,7 @@ TEST_F(UnixControlSocketTest, configSet) {
     ASSERT_TRUE(ucs);
 
     // Run a reflecting server in a thread.
-    thread_.reset(new Thread([this]() { reflectServer(); }));
+    thread_.reset(new thread([this]() { reflectServer(); }));
 
     waitReady();
 
@@ -386,7 +394,7 @@ TEST_F(UnixControlSocketTest, timeout) {
     ASSERT_TRUE(ucs);
 
     // Run a timeout server in a thread.
-    thread_.reset(new Thread([this]() { waitReady(); }));
+    thread_.reset(new thread([this]() { waitReady(); }));
 
     // Try configGet: it should get a communication error,
     EXPECT_THROW(ucs->configGet("foo"), ControlSocketError);
@@ -432,7 +440,7 @@ protected:
     /// @param request Pointer to the HTTP request.
     /// @return Pointer to the generated HTTP response.
     virtual HttpResponsePtr
-    createStockHttpResponse(const ConstHttpRequestPtr& request,
+    createStockHttpResponse(const HttpRequestPtr& request,
                             const HttpStatusCode& status_code) const {
         // Data is in the request context.
         HttpVersion http_version(request->context()->http_version_major_,
@@ -450,10 +458,10 @@ protected:
     /// @param request Pointer to the HTTP request.
     /// @return Pointer to an object representing HTTP response.
     virtual HttpResponsePtr
-    createDynamicHttpResponse(const ConstHttpRequestPtr& request) {
+    createDynamicHttpResponse(HttpRequestPtr request) {
         // Request must always be JSON.
-        ConstPostHttpRequestJsonPtr request_json =
-            boost::dynamic_pointer_cast<const PostHttpRequestJson>(request);
+        PostHttpRequestJsonPtr request_json =
+            boost::dynamic_pointer_cast<PostHttpRequestJson>(request);
         if (!request_json) {
             isc_throw(Unexpected, "request is not JSON");
         }
@@ -506,14 +514,14 @@ public:
 /// @brief Test fixture class for http control sockets.
 class HttpControlSocketTest : public ThreadedTest {
 public:
-    HttpControlSocketTest()
-        : ThreadedTest(), io_service_() {
+    void SetUp() override {
+        SysrepoSetup::cleanSharedMemory();
     }
 
-    /// @brief Destructor.
-    virtual ~HttpControlSocketTest() {
+    void TearDown() override {
+        SysrepoSetup::cleanSharedMemory();
         if (thread_) {
-            thread_->wait();
+            thread_->join();
             thread_.reset();
         }
         // io_service must be stopped after the thread returns,
@@ -546,7 +554,7 @@ public:
     ///
     /// Run IO in a thread.
     void start() {
-        thread_.reset(new Thread([this]() {
+        thread_.reset(new thread([this]() {
             // The thread is ready to go. Signal it to the main
             // thread so it can start the actual test.
             signalReady();
@@ -569,7 +577,7 @@ public:
 
         // If the thread is ready to go, start the listener.
         if (listener_) {
-            ASSERT_NO_THROW(listener_->start());
+            ASSERT_NO_THROW_LOG(listener_->start());
         }
     }
 
@@ -590,7 +598,7 @@ public:
         // Thread has terminated. We can stop the HTTP
         // listener safely.
         if (listener_) {
-            ASSERT_NO_THROW(listener_->stop());
+            ASSERT_NO_THROW_LOG(listener_->stop());
         }
     }
 
@@ -599,12 +607,6 @@ public:
 
     /// @brief Pointer to listener.
     HttpListenerPtr listener_;
-
-    /// @brief Done flag (stopping thread).
-    bool done_;
-
-    /// @brief Finished flag (stopped thread).
-    bool finished_;
 };
 
 /// @brief Create the reflecting listener.
@@ -615,7 +617,7 @@ HttpControlSocketTest::createReflectListener() {
     listener_.reset(new
                 HttpListener(io_service_,
                              IOAddress(SERVER_ADDRESS), SERVER_PORT,
-                             factory,
+                             TlsContextPtr(), factory,
                              HttpListener::RequestTimeout(2000),
                              HttpListener::IdleTimeout(2000)));
 }
@@ -861,4 +863,4 @@ TEST_F(HttpControlSocketTest, partial) {
     stop();
 }
 
-}
+}  // namespace
