@@ -1,4 +1,4 @@
-// Copyright (C) 2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,13 +6,18 @@
 
 #include <config.h>
 
-#include <gtest/gtest.h>
 #include <cc/data.h>
 #include <netconf/parser_context.h>
 #include <cc/dhcp_config_error.h>
+#include <testutils/gtest_utils.h>
 #include <testutils/io_utils.h>
 #include <testutils/user_context_utils.h>
 
+#include <gtest/gtest.h>
+
+#include <fstream>
+#include <set>
+
 using namespace isc::data;
 using namespace isc::test;
 using namespace std;
@@ -48,7 +53,7 @@ void testParser(const std::string& txt, ParserContext::ParserType parser_type,
     bool compare = true) {
     ConstElementPtr test_json;
 
-    ASSERT_NO_THROW({
+    ASSERT_NO_THROW_LOG({
             try {
                 ParserContext ctx;
                 test_json = ctx.parseString(txt, parser_type);
@@ -65,7 +70,7 @@ void testParser(const std::string& txt, ParserContext::ParserType parser_type,
 
     // Now compare if both representations are the same.
     ElementPtr reference_json;
-    ASSERT_NO_THROW(reference_json = Element::fromJSON(txt, true));
+    ASSERT_NO_THROW_LOG(reference_json = Element::fromJSON(txt, true));
     compareJSON(reference_json, test_json);
 }
 
@@ -135,7 +140,7 @@ TEST(ParserTest, keywordNetconf) {
         "            \"validate-changes\": false,"
         "            \"control-socket\": {"
         "                \"socket-type\": \"unix\","
-        "                \"socket-name\": \"/path/to/the/unix/socket-v4\""
+        "                \"socket-name\": \"/tmp/kea4-ctrl-socket\""
         "            }"
         "        },"
         "        \"dhcp6\": {"
@@ -200,7 +205,7 @@ TEST(ParserTest, keywordSubNetconf) {
         "            \"validate-changes\": false,"
         "            \"control-socket\": {"
         "                \"socket-type\": \"unix\","
-        "                \"socket-name\": \"/path/to/the/unix/socket-v4\""
+        "                \"socket-name\": \"/tmp/kea4-ctrl-socket\""
         "            }"
         "        },"
         "        \"dhcp6\": {"
@@ -529,23 +534,43 @@ TEST(ParserTest, errors) {
               "<string>:1.3: Invalid character: e");
     testError("\"a\n\tb\"",
               ParserContext::PARSER_JSON,
-              "<string>:1.1-6: Invalid control in \"a\n\tb\"");
+              "<string>:1.1-6 (near 2): Invalid control in \"a\n\tb\"");
+    testError("\"a\n\\u12\"",
+              ParserContext::PARSER_JSON,
+              "<string>:1.1-8 (near 2): Invalid control in \"a\n\\u12\"");
     testError("\"a\\n\\tb\"",
               ParserContext::PARSER_NETCONF,
               "<string>:1.1-8: syntax error, unexpected constant string, "
               "expecting {");
     testError("\"a\\x01b\"",
               ParserContext::PARSER_JSON,
-              "<string>:1.1-8: Bad escape in \"a\\x01b\"");
+              "<string>:1.1-8 (near 3): Bad escape in \"a\\x01b\"");
     testError("\"a\\u0162\"",
               ParserContext::PARSER_JSON,
-              "<string>:1.1-9: Unsupported unicode escape in \"a\\u0162\"");
+              "<string>:1.1-9 (near 4): Unsupported unicode escape "
+              "in \"a\\u0162\"");
     testError("\"a\\u062z\"",
               ParserContext::PARSER_JSON,
-              "<string>:1.1-9: Bad escape in \"a\\u062z\"");
+              "<string>:1.1-9 (near 3): Bad escape in \"a\\u062z\"");
     testError("\"abc\\\"",
               ParserContext::PARSER_JSON,
-              "<string>:1.1-6: Overflow escape in \"abc\\\"");
+              "<string>:1.1-6 (near 6): Overflow escape in \"abc\\\"");
+    testError("\"a\\u006\"",
+              ParserContext::PARSER_JSON,
+              "<string>:1.1-8 (near 3): Overflow unicode escape "
+              "in \"a\\u006\"");
+    testError("\"\\u\"",
+              ParserContext::PARSER_JSON,
+              "<string>:1.1-4 (near 2): Overflow unicode escape in \"\\u\"");
+    testError("\"\\u\x02\"",
+              ParserContext::PARSER_JSON,
+              "<string>:1.1-5 (near 2): Bad escape in \"\\u\x02\"");
+    testError("\"\\u\\\"foo\"",
+              ParserContext::PARSER_JSON,
+              "<string>:1.1-5 (near 2): Bad escape in \"\\u\\\"...");
+    testError("\"\x02\\u\"",
+              ParserContext::PARSER_JSON,
+              "<string>:1.1-5 (near 1): Invalid control in \"\x02\\u\"");
 
     // from data_unittest.c
     testError("\\a",
@@ -574,7 +599,7 @@ TEST(ParserTest, errors) {
     testError("{ 123 }\n",
               ParserContext::PARSER_NETCONF,
               "<string>:1.3-5: syntax error, unexpected integer, "
-              "expecting Netconf or Logging");
+              "expecting Netconf");
     testError("{ \"foo\" }\n",
               ParserContext::PARSER_JSON,
               "<string>:1.9: syntax error, unexpected }, "
@@ -582,11 +607,15 @@ TEST(ParserTest, errors) {
     testError("{ \"foo\" }\n",
               ParserContext::PARSER_NETCONF,
               "<string>:1.3-7: syntax error, unexpected constant string, "
-              "expecting Netconf or Logging");
+              "expecting Netconf");
     testError("{ \"foo\":null }\n",
               ParserContext::PARSER_NETCONF,
               "<string>:1.3-7: syntax error, unexpected constant string, "
-              "expecting Netconf or Logging");
+              "expecting Netconf");
+    testError("{ \"Logging\":null }\n",
+              ParserContext::PARSER_NETCONF,
+              "<string>:1.3-11: syntax error, unexpected constant string, "
+              "expecting Netconf");
     testError("{ \"Netconf\" }\n",
               ParserContext::PARSER_NETCONF,
               "<string>:1.13: syntax error, unexpected }, "
@@ -600,6 +629,12 @@ TEST(ParserTest, errors) {
               "<string>:1.3: syntax error, unexpected {, "
               "expecting end of file");
 
+    // duplicate in map
+    testError("{ \"foo\": 1, \"foo\": true }\n",
+              ParserContext::PARSER_JSON,
+              "<string>:1:13: duplicate foo entries in "
+              "JSON map (previous at <string>:1:10)");
+
     // bad commas
     testError("{ , }\n",
               ParserContext::PARSER_JSON,
@@ -623,7 +658,7 @@ TEST(ParserTest, errors) {
 
     // unknown keyword
     testError("{ \"Netconf\":{\n"
-              " \"topping\": \"Mozarella\" }}\n",
+              " \"topping\": \"Mozzarella\" }}\n",
               ParserContext::PARSER_NETCONF,
               "<string>:2.2-10: got unexpected keyword "
               "\"topping\" in Netconf map.");
@@ -661,6 +696,32 @@ TEST(ParserTest, errors) {
               ParserContext::PARSER_NETCONF,
               "<string>:3.3-11: duplicate user-context/comment entries "
               "(previous at <string>:2:19)");
+
+    // duplicate Netconf entries
+    testError("{ \"Netconf\":{\n"
+              "  \"comment\": \"first\" },\n"
+              "  \"Netconf\":{\n"
+              "  \"comment\": \"second\" }}\n",
+              ParserContext::PARSER_NETCONF,
+              "<string>:2.23: syntax error, unexpected \",\", expecting }");
+
+    // duplicate of not string entries
+    testError("{ \"Netconf\":{\n"
+              " \"boot-update\": true,\n"
+              " \"boot-update\": false }}\n",
+              ParserContext::PARSER_NETCONF,
+              "<string>:3:2: duplicate boot-update entries in "
+              "Netconf map (previous at <string>:2:17)");
+
+    // duplicate of string entries
+    testError("{ \"Netconf\":{\n"
+              "  \"managed-servers\": {\n"
+              "    \"d2\": {\n"
+              "      \"model\": \"foo\",\n"
+              "      \"model\": \"bar\" }}}}\n",
+              ParserContext::PARSER_NETCONF,
+              "<string>:5:7: duplicate model entries in "
+              "managed-servers entry map (previous at <string>:4:16)");
 }
 
 // Check unicode escapes
@@ -674,7 +735,7 @@ TEST(ParserTest, unicodeEscapes) {
         ins[1] = c;
         ConstElementPtr e(new StringElement(ins));
         json = e->str();
-        ASSERT_NO_THROW(
+        ASSERT_NO_THROW_LOG(
         try {
             ParserContext ctx;
             result = ctx.parseString(json, ParserContext::PARSER_JSON);
@@ -692,7 +753,7 @@ TEST(ParserTest, unicodeSlash) {
     // check the 4 possible encodings of solidus '/'
     ConstElementPtr result;
     string json = "\"/\\/\\u002f\\u002F\"";
-    ASSERT_NO_THROW(
+    ASSERT_NO_THROW_LOG(
     try {
         ParserContext ctx;
         result = ctx.parseString(json, ParserContext::PARSER_JSON);
@@ -704,6 +765,158 @@ TEST(ParserTest, unicodeSlash) {
     EXPECT_EQ("////", result->stringValue());
 }
 
-};
-};
-};
+/// @brief Load a file into a JSON element.
+///
+/// @param fname The name of the file to load.
+/// @param list The JSON element list to add the parsing result to.
+void loadFile(const string& fname, ElementPtr list) {
+    ParserContext ctx;
+    ElementPtr json;
+    EXPECT_NO_THROW(json = ctx.parseFile(fname, ParserContext::PARSER_NETCONF));
+    ASSERT_TRUE(json);
+    list->add(json);
+}
+
+// This test checks that all map entries are in the sample file.
+TEST(ParserTest, mapEntries) {
+    // Type of keyword set.
+    typedef set<string> KeywordSet;
+
+    // Get keywords from the syntax file (netconf_parser.yy).
+    ifstream syntax_file(SYNTAX_FILE);
+    EXPECT_TRUE(syntax_file.is_open());
+    string line;
+    KeywordSet syntax_keys = { "user-context" };
+    // Code setting the map entry.
+    const string pattern = "ctx.stack_.back()->set(\"";
+    while (getline(syntax_file, line)) {
+        // Skip comments.
+        size_t comment = line.find("//");
+        if (comment <= pattern.size()) {
+            continue;
+        }
+        if (comment != string::npos) {
+            line.resize(comment);
+        }
+        // Search for the code pattern.
+        size_t key_begin = line.find(pattern);
+        if (key_begin == string::npos) {
+            continue;
+        }
+        // Extract keywords.
+        line = line.substr(key_begin + pattern.size());
+        size_t key_end = line.find_first_of('"');
+        EXPECT_NE(string::npos, key_end);
+        string keyword = line.substr(0, key_end);
+        // Ignore result when adding the keyword to the syntax keyword set.
+        static_cast<void>(syntax_keys.insert(keyword));
+    }
+    syntax_file.close();
+
+    // Get keywords from the sample file
+    string sample_dir(CFG_EXAMPLES);
+    sample_dir += "/";
+    ElementPtr sample_json = Element::createList();
+    loadFile(sample_dir + "simple-dhcp4.json", sample_json);
+    loadFile(sample_dir + "simple-dhcp6.json", sample_json);
+    KeywordSet sample_keys = {
+        "ca", "d2",
+        "hooks-libraries", "library", "parameters",
+        "socket-url"
+    };
+    // Recursively extract keywords.
+    static void (*extract)(ConstElementPtr, KeywordSet&) =
+        [] (ConstElementPtr json, KeywordSet& set) {
+            if (json->getType() == Element::list) {
+                // Handle lists.
+                for (auto elem : json->listValue()) {
+                    extract(elem, set);
+                }
+            } else if (json->getType() == Element::map) {
+                // Handle maps.
+                for (auto elem : json->mapValue()) {
+                    static_cast<void>(set.insert(elem.first));
+                    // Skip entries with free content.
+                    if ((elem.first != "user-context") &&
+                        (elem.first != "parameters")) {
+                        extract(elem.second, set);
+                    }
+                }
+            }
+        };
+    extract(sample_json, sample_keys);
+
+    // Compare.
+    EXPECT_EQ(syntax_keys, sample_keys);
+}
+
+/// @brief Tests a duplicate entry.
+///
+/// The entry was duplicated by adding a new <name>DDDD entry.
+/// An error is expected, usually it is a duplicate but there are
+/// a few syntax errors when the syntax allows only one parameter.
+///
+/// @param json the JSON configuration with the duplicate entry.
+void testDuplicate(ConstElementPtr json) {
+    string config = json->str();
+    size_t where = config.find("DDDD");
+    ASSERT_NE(string::npos, where);
+    string before = config.substr(0, where);
+    string after = config.substr(where + 4, string::npos);
+    ParserContext ctx;
+    EXPECT_THROW(ctx.parseString(before + after,
+                                 ParserContext::PARSER_NETCONF),
+                 ParseError) << "config: " << config;
+}
+
+// This test checks that duplicate entries make parsing to fail.
+TEST(ParserTest, duplicateMapEntries) {
+    // Get the config to work with from the sample file.
+    string sample_fname(CFG_EXAMPLES);
+    sample_fname += "/simple-dhcp6.json";
+    ParserContext ctx;
+    ElementPtr sample_json;
+    EXPECT_NO_THROW(sample_json =
+        ctx.parseFile(sample_fname, ParserContext::PARSER_NETCONF));
+    ASSERT_TRUE(sample_json);
+
+    // Recursively check duplicates.
+    static void (*test)(ElementPtr, ElementPtr, size_t&) =
+        [] (ElementPtr config, ElementPtr json, size_t& cnt) {
+            if (json->getType() == Element::list) {
+                // Handle lists.
+                for (auto elem : json->listValue()) {
+                    test(config, elem, cnt);
+                }
+            } else if (json->getType() == Element::map) {
+                // Handle maps.
+                for (auto elem : json->mapValue()) {
+                    // Skip entries with free content.
+                    if ((elem.first == "user-context") ||
+                        (elem.first == "parameters")) {
+                        continue;
+                    }
+
+                    // Perform tests.
+                    string dup = elem.first + "DDDD";
+                    json->set(dup, elem.second);
+                    testDuplicate(config);
+                    json->remove(dup);
+                    ++cnt;
+
+                    // Recursive call.
+                    ElementPtr mutable_json =
+                        boost::const_pointer_cast<Element>(elem.second);
+                    ASSERT_TRUE(mutable_json);
+                    test(config, mutable_json, cnt);
+                }
+            }
+        };
+    size_t cnt = 0;
+    test(sample_json, sample_json, cnt);
+    cout << "checked " << cnt << " duplicated map entries\n";
+}
+
+}
+}
+}
