@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -104,8 +104,7 @@ public:
             }
         }
 
-        /// @brief specifies parameter types (true = query by address, false =
-        ///         query by identifier-type,identifier)
+        /// @brief specifies parameter types
         Type query_type;
 
         /// @brief Lease type (NA,TA or PD) used for v6 leases
@@ -119,8 +118,8 @@ public:
 
         /// @brief Default constructor.
         Parameters()
-            : addr("::"), query_type(TYPE_ADDR), lease_type(Lease::TYPE_NA),
-              iaid(0), updateDDNS(false) {
+            : subnet_id(0), addr("::"), query_type(TYPE_ADDR),
+              lease_type(Lease::TYPE_NA), iaid(0), updateDDNS(false) {
         }
     };
 
@@ -654,7 +653,21 @@ LeaseCmdsImpl::addOrUpdate4(Lease4Ptr lease, bool force_create) {
         LeaseCmdsImpl::updateStatsOnAdd(lease);
         return (true);
     }
-    LeaseMgrFactory::instance().updateLease4(lease);
+    if (existing) {
+        // Update lease current expiration time with value received from the
+        // database. Some database backends reject operations on the lease if
+        // the current expiration time value does not match what is stored.
+        Lease::syncCurrentExpirationTime(*existing, *lease);
+    }
+    try {
+        LeaseMgrFactory::instance().updateLease4(lease);
+    } catch (const NoSuchLease&) {
+        isc_throw(InvalidOperation, "failed to update the lease with address "
+                  << lease->addr_ << " either because the lease has been "
+                  "deleted or it has changed in the database, in both cases a "
+                  "retry might succeed");
+    }
+
     LeaseCmdsImpl::updateStatsOnUpdate(existing, lease);
     return (false);
 }
@@ -672,7 +685,21 @@ LeaseCmdsImpl::addOrUpdate6(Lease6Ptr lease, bool force_create) {
         LeaseCmdsImpl::updateStatsOnAdd(lease);
         return (true);
     }
-    LeaseMgrFactory::instance().updateLease6(lease);
+    if (existing) {
+        // Update lease current expiration time with value received from the
+        // database. Some database backends reject operations on the lease if
+        // the current expiration time value does not match what is stored.
+        Lease::syncCurrentExpirationTime(*existing, *lease);
+    }
+    try {
+        LeaseMgrFactory::instance().updateLease6(lease);
+    } catch (const NoSuchLease&) {
+        isc_throw(InvalidOperation, "failed to update the lease with address "
+                  << lease->addr_ << " either because the lease has been "
+                  "deleted or it has changed in the database, in both cases a "
+                  "retry might succeed");
+    }
+
     LeaseCmdsImpl::updateStatsOnUpdate(existing, lease);
     return (false);
 }
@@ -705,67 +732,55 @@ LeaseCmdsImpl::leaseAddHandler(CalloutHandle& handle) {
         if (v4) {
             Lease4Parser parser;
             lease4 = parser.parse(config, cmd_args_, force_create);
-
             if (lease4) {
                 bool success;
-                if (MultiThreadingMgr::instance().getMode() &&
-                    !MultiThreadingMgr::instance().isInCriticalSection()) {
-                    bool use_cs = false;
-                    {
-                        // Try to avoid a race.
-                        ResourceHandler4 resource_handler;
-                        use_cs = !resource_handler.tryLock4(lease4->addr_);
-                        if (!use_cs) {
-                            success = LeaseMgrFactory::instance().addLease(lease4);
-                        }
-                    }
-                    if (use_cs) {
-                        // Failed to avoid the race.
-                        MultiThreadingCriticalSection cs;
-                        success = LeaseMgrFactory::instance().addLease(lease4);
-                    }
-                } else {
-                    // No multi-threading.
+                if (!MultiThreadingMgr::instance().getMode()) {
+                    // Not multi-threading.
                     success = LeaseMgrFactory::instance().addLease(lease4);
+                } else {
+                    // Multi-threading, try to lock first to avoid a race.
+                    ResourceHandler4 resource_handler;
+                    if (resource_handler.tryLock4(lease4->addr_)) {
+                        success = LeaseMgrFactory::instance().addLease(lease4);
+                    } else {
+                        isc_throw(ResourceBusy,
+                                  "ResourceBusy: IP address:" << lease4->addr_
+                                  << " could not be added.");
+                    }
                 }
+
                 if (!success) {
                     isc_throw(db::DuplicateEntry, "IPv4 lease already exists.");
                 }
+
                 LeaseCmdsImpl::updateStatsOnAdd(lease4);
                 resp << "Lease for address " << lease4->addr_.toText()
                      << ", subnet-id " << lease4->subnet_id_ << " added.";
             }
-
         } else {
             Lease6Parser parser;
             lease6 = parser.parse(config, cmd_args_, force_create);
-
             if (lease6) {
                 bool success;
-                if (MultiThreadingMgr::instance().getMode() &&
-                    !MultiThreadingMgr::instance().isInCriticalSection()) {
-                    bool use_cs = false;
-                    {
-                        // Try to avoid a race.
-                        ResourceHandler resource_handler;
-                        use_cs = !resource_handler.tryLock(lease6->type_,
-                                                           lease6->addr_);
-                        if (!use_cs) {
-                            success = LeaseMgrFactory::instance().addLease(lease6);
-                        }
-                    }
-                    if (use_cs) {
-                        // Failed to avoid the race.
-                        MultiThreadingCriticalSection cs;
-                        success = LeaseMgrFactory::instance().addLease(lease6);
-                    }
-                } else {
-                    // No multi-threading.
+                if (!MultiThreadingMgr::instance().getMode()) {
+                    // Not multi-threading.
                     success = LeaseMgrFactory::instance().addLease(lease6);
+                } else {
+                    // Multi-threading, try to lock first to avoid a race.
+                    ResourceHandler resource_handler;
+                    if (resource_handler.tryLock(lease6->type_, lease6->addr_)) {
+                        success = LeaseMgrFactory::instance().addLease(lease6);
+                    } else {
+                        isc_throw(ResourceBusy,
+                                  "ResourceBusy: IP address:" << lease6->addr_
+                                  << " could not be added.");
+                    }
                 }
+
                 if (!success) {
                     isc_throw(db::DuplicateEntry, "IPv6 lease already exists.");
                 }
+
                 LeaseCmdsImpl::updateStatsOnAdd(lease6);
                 if (lease6->type_ == Lease::TYPE_NA) {
                     resp << "Lease for address " << lease6->addr_.toText()
@@ -1220,7 +1235,7 @@ LeaseCmdsImpl::leaseGetByHwAddressHandler(CalloutHandle& handle) {
 
         // The 'hw-address' argument is a string.
         if (hw_address->getType() != Element::string) {
-            isc_throw(BadValue, "'hw-address'parameter must be a string");
+            isc_throw(BadValue, "'hw-address' parameter must be a string");
         }
 
         HWAddr hwaddr = HWAddr::fromText(hw_address->stringValue());
@@ -1270,7 +1285,7 @@ LeaseCmdsImpl::leaseGetByClientIdHandler(CalloutHandle& handle) {
 
         // The 'client-id' argument is a string.
         if (client_id->getType() != Element::string) {
-            isc_throw(BadValue, "'client-id'parameter must be a string");
+            isc_throw(BadValue, "'client-id' parameter must be a string");
         }
 
         ClientIdPtr clientid = ClientId::fromText(client_id->stringValue());
@@ -1320,7 +1335,7 @@ LeaseCmdsImpl::leaseGetByDuidHandler(CalloutHandle& handle) {
 
         // The 'duid' argument is a string.
         if (duid->getType() != Element::string) {
-            isc_throw(BadValue, "'duid'parameter must be a string");
+            isc_throw(BadValue, "'duid' parameter must be a string");
         }
 
         DUID duid_ = DUID::fromText(duid->stringValue());
@@ -1372,7 +1387,7 @@ LeaseCmdsImpl::leaseGetByHostnameHandler(CalloutHandle& handle) {
 
         // The 'hostname' argument is a string.
         if (hostname->getType() != Element::string) {
-            isc_throw(BadValue, "'hostname'parameter must be a string");
+            isc_throw(BadValue, "'hostname' parameter must be a string");
         }
 
         std::string hostname_ = hostname->stringValue();
@@ -1626,30 +1641,22 @@ LeaseCmdsImpl::lease6BulkApplyHandler(CalloutHandle& handle) {
             for (auto lease : parsed_leases_list) {
 
                 try {
-                    if (MultiThreadingMgr::instance().getMode() &&
-                        !MultiThreadingMgr::instance().isInCriticalSection()) {
-                        bool use_cs = false;
-                        {
-                            // Try to avoid a race.
-                            ResourceHandler resource_handler;
-                            use_cs = !resource_handler.tryLock(lease->type_,
-                                                               lease->addr_);
-                            if (!use_cs) {
-                                addOrUpdate6(lease, true);
-                            }
-                        }
-                        if (use_cs) {
-                            // Failed to avoid the race.
-                            MultiThreadingCriticalSection cs;
-                            addOrUpdate6(lease, true);
-                        }
-                    } else {
-                        // No multi-threading.
+                    if (!MultiThreadingMgr::instance().getMode()) {
+                        // Not multi-threading.
                         addOrUpdate6(lease, true);
+                    } else {
+                        // Multi-threading, try to lock first to avoid a race.
+                        ResourceHandler resource_handler;
+                        if (resource_handler.tryLock(lease->type_, lease->addr_)) {
+                            addOrUpdate6(lease, true);
+                        } else {
+                            isc_throw(ResourceBusy,
+                                      "ResourceBusy: IP address:" << lease->addr_
+                                      << " could not be updated.");
+                        }
                     }
 
                     ++success_count;
-
                 } catch (const std::exception& ex) {
                     // Lazy creation of the list of leases which failed to add/update.
                     if (!failed_leases_list) {
@@ -1678,7 +1685,7 @@ LeaseCmdsImpl::lease6BulkApplyHandler(CalloutHandle& handle) {
 
             // failed-leases
             if (failed_leases_list) {
-                args->set("failed-deleted-leases", failed_leases_list);
+                args->set("failed-leases", failed_leases_list);
             }
         }
 
@@ -1784,26 +1791,21 @@ LeaseCmdsImpl::lease4UpdateHandler(CalloutHandle& handle) {
         // subnet-id is valid, etc)
         lease4 = parser.parse(config, cmd_args_, force_create);
         bool added = false;
-        if (MultiThreadingMgr::instance().getMode() &&
-            !MultiThreadingMgr::instance().isInCriticalSection()) {
-            bool use_cs = false;
-            {
-                // Try to avoid a race.
-                ResourceHandler4 resource_handler;
-                use_cs = !resource_handler.tryLock4(lease4->addr_);
-                if (use_cs) {
-                    added = addOrUpdate4(lease4, force_create);
-                }
-            }
-            if (use_cs) {
-                // Failed to avoid the race.
-                MultiThreadingCriticalSection cs;
-                added = addOrUpdate4(lease4, force_create);
-            }
-        } else {
-            // No multi-threading.
+        if (!MultiThreadingMgr::instance().getMode()) {
+            // Not multi-threading.
             added = addOrUpdate4(lease4, force_create);
+        } else {
+            // Multi-threading, try to lock first to avoid a race.
+            ResourceHandler4 resource_handler;
+            if (resource_handler.tryLock4(lease4->addr_)) {
+                added = addOrUpdate4(lease4, force_create);
+            } else {
+                isc_throw(ResourceBusy,
+                          "ResourceBusy: IP address:" << lease4->addr_
+                          << " could not be updated.");
+            }
         }
+
         if (added) {
             setSuccessResponse(handle, "IPv4 lease added.");
         } else {
@@ -1836,28 +1838,22 @@ LeaseCmdsImpl::lease6UpdateHandler(CalloutHandle& handle) {
         // The parser does sanity checks (if the address is in scope, if
         // subnet-id is valid, etc)
         lease6 = parser.parse(config, cmd_args_, force_create);
-        bool added;
-        if (MultiThreadingMgr::instance().getMode() &&
-            !MultiThreadingMgr::instance().isInCriticalSection()) {
-            bool use_cs = false;
-            {
-                // Try to avoid a race.
-                ResourceHandler resource_handler;
-                use_cs = !resource_handler.tryLock(lease6->type_,
-                                                   lease6->addr_);
-                if (!use_cs) {
-                    added = addOrUpdate6(lease6, force_create);
-                }
-            }
-            if (use_cs) {
-                // Failed to avoid the race.
-                MultiThreadingCriticalSection cs;
-                added = addOrUpdate6(lease6, force_create);
-            }
-        } else {
-            // No multi-threading.
+        bool added = false;
+        if (!MultiThreadingMgr::instance().getMode()) {
+            // Not multi-threading.
             added = addOrUpdate6(lease6, force_create);
+        } else {
+            // Multi-threading, try to lock first to avoid a race.
+            ResourceHandler resource_handler;
+            if (resource_handler.tryLock(lease6->type_, lease6->addr_)) {
+                added = addOrUpdate6(lease6, force_create);
+            } else {
+                isc_throw(ResourceBusy,
+                          "ResourceBusy: IP address:" << lease6->addr_
+                          << " could not be updated.");
+            }
         }
+
         if (added) {
             setSuccessResponse(handle, "IPv6 lease added.");
         } else {
@@ -2217,8 +2213,6 @@ LeaseCmdsImpl::createFailedLeaseMap(const Lease::Type& lease_type,
     return (failed_lease_map);
 }
 
-// Do lease changes in a critical section.
-
 int
 LeaseCmds::leaseAddHandler(CalloutHandle& handle) {
     return (impl_->leaseAddHandler(handle));
