@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,14 +18,14 @@
 #include <asiodns/logger.h>
 #include <dns/messagerenderer.h>
 #include <dns/opcode.h>
+#include <dns/qid_gen.h>
 #include <dns/rcode.h>
 #include <util/buffer.h>
-#include <util/random/qid_gen.h>
 
-#include <boost/bind.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <boost/date_time/posix_time/posix_time_types.hpp>
 
+#include <functional>
 #include <unistd.h>             // for some IPC/network system calls
 #include <netinet/in.h>
 #include <stdint.h>
@@ -35,7 +35,6 @@ using namespace boost::asio;
 using namespace isc::asiolink;
 using namespace isc::dns;
 using namespace isc::util;
-using namespace isc::util::random;
 using namespace isc::log;
 using namespace std;
 
@@ -252,7 +251,7 @@ IOFetch::operator()(boost::system::error_code ec, size_t length) {
                 // first two bytes of the packet).
                 data_->msgbuf->writeUint16At(data_->qid, 0);
 
-            } 
+            }
         }
 
         // If we timeout, we stop, which will can cancel outstanding I/Os and
@@ -260,7 +259,7 @@ IOFetch::operator()(boost::system::error_code ec, size_t length) {
         if (data_->timeout != -1) {
             data_->timer.expires_from_now(boost::posix_time::milliseconds(
                 data_->timeout));
-            data_->timer.async_wait(boost::bind(&IOFetch::stop, *this,
+            data_->timer.async_wait(std::bind(&IOFetch::stop, *this,
                 TIME_OUT));
         }
 
@@ -279,7 +278,7 @@ IOFetch::operator()(boost::system::error_code ec, size_t length) {
             data_->origin = ASIODNS_SEND_DATA;
             BOOST_ASIO_CORO_YIELD data_->socket->asyncSend(data_->msgbuf->getData(),
                 data_->msgbuf->getLength(), data_->remote_snd.get(), *this);
-    
+
             // Now receive the response.  Since TCP may not receive the entire
             // message in one operation, we need to loop until we have received
             // it. (This can't be done within the asyncReceive() method because
@@ -298,7 +297,7 @@ IOFetch::operator()(boost::system::error_code ec, size_t length) {
             // the expected amount of data.  Then we need to loop until we have
             // received all the data before copying it back to the user's buffer.
             // And we want to minimize the amount of copying...
-    
+
             data_->origin = ASIODNS_READ_DATA;
             data_->cumulative = 0;          // No data yet received
             data_->offset = 0;              // First data into start of buffer
@@ -394,10 +393,12 @@ IOFetch::stop(Result result) {
 void IOFetch::logIOFailure(boost::system::error_code ec) {
 
     // Should only get here with a known error code.
-    assert((data_->origin == ASIODNS_OPEN_SOCKET) ||
-           (data_->origin == ASIODNS_SEND_DATA) ||
-           (data_->origin == ASIODNS_READ_DATA) ||
-           (data_->origin == ASIODNS_UNKNOWN_ORIGIN));
+    if ((data_->origin != ASIODNS_OPEN_SOCKET) &&
+        (data_->origin != ASIODNS_SEND_DATA) &&
+        (data_->origin != ASIODNS_READ_DATA) &&
+        (data_->origin != ASIODNS_UNKNOWN_ORIGIN)) {
+        isc_throw(isc::Unexpected, "impossible error code " << data_->origin);
+    }
 
     LOG_ERROR(logger, data_->origin).arg(ec.value()).
         arg((data_->remote_snd->getProtocol() == IPPROTO_TCP) ?
