@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
 // Copyright (C) 2016-2017 Deutsche Telekom AG.
 //
 // Authors: Razvan Becheriu <razvan.becheriu@qualitance.com>
@@ -275,8 +275,9 @@ CqlBindUdt(const boost::any& value,
         } catch (const boost::bad_any_cast& exception) {
             isc_throw(DbOperationError,
                       "CqlCommon::udtSetData(): "
-                          << exception.what() << " when binding parameter "
-                          << " of type " << element.type().name()
+                          << exception.what()
+                          << " when binding parameter of type "
+                          << element.type().name()
                           << "in UDT with function CQL_FUNCTIONS["
                           << exchangeType(element) << "].cqlUdtSetFunction_");
         }
@@ -742,17 +743,10 @@ void
 CqlExchange::convertToDatabaseTime(const time_t& cltt,
                                    const uint32_t& valid_lifetime,
                                    cass_int64_t& expire) {
-    // Calculate expire time. Store it in the 64-bit value so as we can
-    // detect overflows.
+    // Calculate expire time.
     cass_int64_t expire_time = static_cast<cass_int64_t>(cltt) +
             static_cast<cass_int64_t>(valid_lifetime);
 
-    if (expire_time > DatabaseConnection::MAX_DB_TIME) {
-        isc_throw(BadValue,
-                  "CqlExchange(): convertToDatabaseTime(): time value "
-                      << expire_time << " is too large");
-    }
-
     expire = expire_time;
 }
 
@@ -760,11 +754,7 @@ void
 CqlExchange::convertFromDatabaseTime(const cass_int64_t& expire,
                                      const cass_int64_t& valid_lifetime,
                                      time_t& cltt) {
-    /// @todo: Although 2037 is still far away, there are people who use infinite
-    /// lifetimes. Cassandra doesn't have to support it right now, but at least
-    /// we should be able to detect a problem.
-
-    // Convert to local time
+    // Convert to local time.
     cltt = static_cast<time_t>(expire - valid_lifetime);
 }
 
