@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,6 +12,7 @@
 #include <database/db_log.h>
 #include <database/db_messages.h>
 #include <exceptions/exceptions.h>
+#include <util/strutil.h>
 
 #include <boost/algorithm/string.hpp>
 #include <boost/foreach.hpp>
@@ -36,26 +37,62 @@ DatabaseConnection::getParameter(const std::string& name) const {
 DatabaseConnection::ParameterMap
 DatabaseConnection::parse(const std::string& dbaccess) {
     DatabaseConnection::ParameterMap mapped_tokens;
+    std::string dba = dbaccess;
 
-    if (!dbaccess.empty()) {
-        vector<string> tokens;
+    if (!dba.empty()) {
+        try {
+            vector<string> tokens;
 
-        // We need to pass a string to is_any_of, not just char*. Otherwise
-        // there are cryptic warnings on Debian6 running g++ 4.4 in
-        // /usr/include/c++/4.4/bits/stl_algo.h:2178 "array subscript is above
-        // array bounds"
-        boost::split(tokens, dbaccess, boost::is_any_of(string("\t ")));
-        BOOST_FOREACH(std::string token, tokens) {
-            size_t pos = token.find("=");
-            if (pos != string::npos) {
-                string name = token.substr(0, pos);
-                string value = token.substr(pos + 1);
-                mapped_tokens.insert(make_pair(name, value));
-            } else {
-                DB_LOG_ERROR(DB_INVALID_ACCESS).arg(dbaccess);
-                isc_throw(InvalidParameter, "Cannot parse " << token
-                          << ", expected format is name=value");
+            // Handle the special case of a password which is enclosed in apostrophes.
+            // Such password may include whitespace.
+            std::string password_prefix = "password='";
+            auto password_pos = dba.find(password_prefix);
+            if (password_pos != string::npos) {
+                // Password starts with apostrophe, so let's find ending apostrophe.
+                auto password_end_pos = dba.find('\'', password_pos + password_prefix.length());
+                if (password_end_pos == string::npos) {
+                    // No ending apostrophe. This is wrong.
+                    isc_throw(InvalidParameter, "Apostrophe (') expected at the end of password");
+                }
+                // Extract the password value. It starts after the password=' prefix and ends
+                // at the position of ending apostrophe.
+                auto password = dba.substr(password_pos + password_prefix.length(),
+                                           password_end_pos - password_pos - password_prefix.length());
+                mapped_tokens.insert(make_pair("password", password));
+
+                // We need to erase the password from the access string because the generic
+                // algorithm parsing other parameters requires that there are no whitespaces
+                // within the parameter values.
+                dba.erase(password_pos, password_prefix.length() + password.length() + 2);
+                // Leading or trailing whitespace may remain after the password removal.
+                dba = util::str::trim(dba);
+                // If the password was the only parameter in the access string, there is
+                // nothing more to do.
+                if (dba.empty()) {
+                    return (mapped_tokens);
+                }
             }
+
+            // We need to pass a string to is_any_of, not just char*. Otherwise
+            // there are cryptic warnings on Debian6 running g++ 4.4 in
+            // /usr/include/c++/4.4/bits/stl_algo.h:2178 "array subscript is above
+            // array bounds"
+            boost::split(tokens, dba, boost::is_any_of(string("\t ")));
+            BOOST_FOREACH(std::string token, tokens) {
+                size_t pos = token.find("=");
+                if (pos != string::npos) {
+                    string name = token.substr(0, pos);
+                    string value = token.substr(pos + 1);
+                    mapped_tokens.insert(make_pair(name, value));
+                } else {
+                    isc_throw(InvalidParameter, "Cannot parse " << token
+                              << ", expected format is name=value");
+                }
+            }
+        } catch (const std::exception& ex) {
+            // We'd obscure the password if we could parse the access string.
+            DB_LOG_ERROR(DB_INVALID_ACCESS).arg(dbaccess);
+            throw;
         }
     }
 
@@ -110,14 +147,13 @@ DatabaseConnection::configuredReadOnly() const {
     return (readonly_value == "true");
 }
 
-ReconnectCtlPtr
-DatabaseConnection::makeReconnectCtl() const {
-    ReconnectCtlPtr retry;
+void
+DatabaseConnection::makeReconnectCtl(const std::string& timer_name) {
     string type = "unknown";
     unsigned int retries = 0;
     unsigned int interval = 0;
 
-    // Assumes that parsing ensurse only valid values are present
+    // Assumes that parsing ensures only valid values are present
     try {
         type = getParameter("type");
     } catch (...) {
@@ -139,15 +175,40 @@ DatabaseConnection::makeReconnectCtl() const {
         // Wasn't specified so we'll use default of 0;
     }
 
-    retry.reset(new ReconnectCtl(type, retries, interval));
-    return (retry);
+    OnFailAction action = OnFailAction::STOP_RETRY_EXIT;
+    try {
+        parm_str = getParameter("on-fail");
+        action = ReconnectCtl::onFailActionFromText(parm_str);
+    } catch (...) {
+        // Wasn't specified so we'll use default of "stop-retry-exit";
+    }
+
+    reconnect_ctl_ = boost::make_shared<ReconnectCtl>(type, timer_name, retries,
+                                                      interval, action);
 }
 
 bool
-DatabaseConnection::invokeDbLostCallback() const {
-    if (DatabaseConnection::db_lost_callback) {
-        // Invoke the callback, passing in a new instance of ReconnectCtl
-        return (DatabaseConnection::db_lost_callback)(makeReconnectCtl());
+DatabaseConnection::invokeDbLostCallback(const ReconnectCtlPtr& db_reconnect_ctl) {
+    if (DatabaseConnection::db_lost_callback_) {
+        return (DatabaseConnection::db_lost_callback_(db_reconnect_ctl));
+    }
+
+    return (false);
+}
+
+bool
+DatabaseConnection::invokeDbRecoveredCallback(const ReconnectCtlPtr& db_reconnect_ctl) {
+    if (DatabaseConnection::db_recovered_callback_) {
+        return (DatabaseConnection::db_recovered_callback_(db_reconnect_ctl));
+    }
+
+    return (false);
+}
+
+bool
+DatabaseConnection::invokeDbFailedCallback(const ReconnectCtlPtr& db_reconnect_ctl) {
+    if (DatabaseConnection::db_failed_callback_) {
+        return (DatabaseConnection::db_failed_callback_(db_reconnect_ctl));
     }
 
     return (false);
@@ -197,7 +258,8 @@ DatabaseConnection::toElement(const ParameterMap& params) {
                    (keyword == "contact-points") ||
                    (keyword == "consistency") ||
                    (keyword == "serial-consistency") ||
-                   (keyword == "keyspace")) {
+                   (keyword == "keyspace") ||
+                   (keyword == "on-fail")) {
             result->set(keyword, isc::data::Element::create(value));
         } else {
             LOG_ERROR(database_logger, DATABASE_TO_JSON_ERROR)
@@ -214,8 +276,35 @@ DatabaseConnection::toElementDbAccessString(const std::string& dbaccess) {
     return (toElement(params));
 }
 
-DatabaseConnection::DbLostCallback
-DatabaseConnection::db_lost_callback = 0;
+std::string
+ReconnectCtl::onFailActionToText(OnFailAction action) {
+    switch (action) {
+    case OnFailAction::STOP_RETRY_EXIT:
+        return ("stop-retry-exit");
+    case OnFailAction::SERVE_RETRY_EXIT:
+        return ("serve-retry-exit");
+    case OnFailAction::SERVE_RETRY_CONTINUE:
+        return ("serve-retry-continue");
+    }
+    return ("invalid-action-type");
+}
 
-};
-};
+OnFailAction
+ReconnectCtl::onFailActionFromText(const std::string& text) {
+    if (text == "stop-retry-exit") {
+        return (OnFailAction::STOP_RETRY_EXIT);
+    } else if (text == "serve-retry-exit") {
+        return (OnFailAction::SERVE_RETRY_EXIT);
+    } else if (text == "serve-retry-continue") {
+        return (OnFailAction::SERVE_RETRY_CONTINUE);
+    } else {
+        isc_throw(BadValue, "Invalid action on connection loss: " << text);
+    }
+}
+
+DbCallback DatabaseConnection::db_lost_callback_ = 0;
+DbCallback DatabaseConnection::db_recovered_callback_ = 0;
+DbCallback DatabaseConnection::db_failed_callback_ = 0;
+
+}  // namespace db
+}  // namespace isc
