@@ -1,9 +1,11 @@
-// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <dhcp/duid.h>
 #include <exceptions/exceptions.h>
 #include <util/io_utilities.h>
@@ -20,7 +22,8 @@ namespace dhcp {
 
 DUID::DUID(const std::vector<uint8_t>& duid) {
     if (duid.size() > MAX_DUID_LEN) {
-        isc_throw(isc::BadValue, "DUID too large");
+        isc_throw(isc::BadValue, "DUID size is " << duid.size()
+                  << " bytes, exceeds maximum of " << MAX_DUID_LEN);
     }
     if (duid.empty()) {
         isc_throw(isc::BadValue, "Empty DUIDs are not allowed");
@@ -30,7 +33,8 @@ DUID::DUID(const std::vector<uint8_t>& duid) {
 
 DUID::DUID(const uint8_t* data, size_t len) {
     if (len > MAX_DUID_LEN) {
-        isc_throw(isc::BadValue, "DUID too large");
+        isc_throw(isc::BadValue, "DUID size is " << len
+                  << " bytes, exceeds maximum of " << MAX_DUID_LEN);
     }
     if (len == 0) {
         isc_throw(isc::BadValue, "Empty DUIDs/Client-ids not allowed");
@@ -42,7 +46,6 @@ DUID::DUID(const uint8_t* data, size_t len) {
 const std::vector<uint8_t>& DUID::getDuid() const {
     return (duid_);
 }
-
 DUID::DUIDType DUID::getType() const {
     if (duid_.size() < 2) {
         return (DUID_UNKNOWN);
@@ -62,16 +65,11 @@ DUID::fromText(const std::string& text) {
     return (DUID(binary));
 }
 
-DuidPtr
-DUID::generateEmpty() {
-
-    // Technically this is a one byte DUID with value of 0. However, we do have
-    // a number of safety checks against invalid duids (too long or empty) and
-    // we should keep them. Therefore "empty" means a single byte with value of 0.
-    std::vector<uint8_t> empty_duid(1,0);
-
-    DuidPtr duid(new DUID(empty_duid));
-    return (duid);
+const DUID&
+DUID::EMPTY() {
+    static std::vector<uint8_t> empty_duid(1,0);
+    static DUID empty(empty_duid);
+        return (empty);
 }
 
 std::string DUID::toText() const {
