@@ -1,9 +1,11 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <dhcp/dhcp4.h>
 #include <dhcp/option4_client_fqdn.h>
 #include <dns/labelsequence.h>
@@ -52,9 +54,9 @@ public:
 
     /// @brief Constructor, from wire data.
     ///
-    /// @param first An iterator pointing to the begining of the option data
+    /// @param first An iterator pointing to the beginning of the option data
     /// in the wire format.
-    /// @param last An iterator poiting to the end of the option data in the
+    /// @param last An iterator pointing to the end of the option data in the
     /// wire format.
     Option4ClientFqdnImpl(OptionBufferConstIter first,
                           OptionBufferConstIter last);
@@ -92,9 +94,9 @@ public:
 
     /// @brief Parse the Option provided in the wire format.
     ///
-    /// @param first An iterator pointing to the begining of the option data
+    /// @param first An iterator pointing to the beginning of the option data
     /// in the wire format.
-    /// @param last An iterator poiting to the end of the option data in the
+    /// @param last An iterator pointing to the end of the option data in the
     /// wire format.
     void parseWireData(OptionBufferConstIter first,
                        OptionBufferConstIter last);
@@ -104,11 +106,11 @@ public:
     void parseCanonicalDomainName(OptionBufferConstIter first,
                                   OptionBufferConstIter last);
 
-    /// @brief Parse domain-name emcoded in the deprecated ASCII format.
+    /// @brief Parse domain-name encoded in the deprecated ASCII format.
     ///
-    /// @param first An iterator pointing to the begining of the option data
+    /// @param first An iterator pointing to the beginning of the option data
     /// where domain-name is stored.
-    /// @param last An iterator poiting to the end of the option data where
+    /// @param last An iterator pointing to the end of the option data where
     /// domain-name is stored.
     void parseASCIIDomainName(OptionBufferConstIter first,
                               OptionBufferConstIter last);
@@ -121,7 +123,7 @@ Option4ClientFqdnImpl(const uint8_t flags,
                       const std::string& domain_name,
                       // cppcheck 1.57 complains that const enum value is not passed
                       // by reference. Note that, it accepts the non-const enum value
-                      // to be passed by value. In both cases it is unneccessary to
+                      // to be passed by value. In both cases it is unnecessary to
                       // pass the enum by reference.
                       // cppcheck-suppress passedByValue
                       const Option4ClientFqdn::DomainNameType name_type)
@@ -188,7 +190,7 @@ Option4ClientFqdnImpl::
 setDomainName(const std::string& domain_name,
               // cppcheck 1.57 complains that const enum value is not passed
               // by reference. Note that, it accepts the non-const enum
-              // to be passed by value. In both cases it is unneccessary to
+              // to be passed by value. In both cases it is unnecessary to
               // pass the enum by reference.
               // cppcheck-suppress passedByValue
               const Option4ClientFqdn::DomainNameType name_type) {
@@ -208,7 +210,9 @@ setDomainName(const std::string& domain_name,
 
     } else {
         try {
-            domain_name_.reset(new isc::dns::Name(name));
+            // The second argument indicates that the name should be converted
+            // to lower case.
+            domain_name_.reset(new isc::dns::Name(name, true));
 
         } catch (const Exception&) {
             isc_throw(InvalidOption4FqdnDomainName,
@@ -289,7 +293,9 @@ Option4ClientFqdnImpl::parseCanonicalDomainName(OptionBufferConstIter first,
             buf.push_back(0);
             // Reset domain name.
             isc::util::InputBuffer name_buf(&buf[0], buf.size());
-            domain_name_.reset(new isc::dns::Name(name_buf));
+            // The second argument indicates that the name should be converted
+            // to lower case.
+            domain_name_.reset(new isc::dns::Name(name_buf, true));
             // Terminating zero was missing, so set the domain-name type
             // to partial.
             domain_name_type_ = Option4ClientFqdn::PARTIAL;
@@ -299,7 +305,9 @@ Option4ClientFqdnImpl::parseCanonicalDomainName(OptionBufferConstIter first,
             // Name object constructor.
             isc::util::InputBuffer name_buf(&(*first),
                                             std::distance(first, last));
-            domain_name_.reset(new isc::dns::Name(name_buf));
+            // The second argument indicates that the name should be converted
+            // to lower case.
+            domain_name_.reset(new isc::dns::Name(name_buf, true));
             // Set the domain-type to fully qualified domain name.
             domain_name_type_ = Option4ClientFqdn::FULL;
         }
@@ -311,7 +319,9 @@ Option4ClientFqdnImpl::parseASCIIDomainName(OptionBufferConstIter first,
                                             OptionBufferConstIter last) {
     if (std::distance(first, last) > 0) {
         std::string domain_name(first, last);
-        domain_name_.reset(new isc::dns::Name(domain_name));
+        // The second argument indicates that the name should be converted
+        // to lower case.
+        domain_name_.reset(new isc::dns::Name(domain_name, true));
         domain_name_type_ = domain_name[domain_name.length() - 1] == '.' ?
             Option4ClientFqdn::FULL : Option4ClientFqdn::PARTIAL;
     }
