@@ -184,18 +184,18 @@ Pkt4::unpack() {
     if (buffer_in.getLength() == buffer_in.getPosition()) {
         // this is *NOT* DHCP packet. It does not have any DHCPv4 options. In
         // particular, it does not have magic cookie, a 4 byte sequence that
-        // differentiates between DHCP and BOOTP packets.
-        isc_throw(InvalidOperation, "Received BOOTP packet. BOOTP is not supported.");
+        // differentiates between DHCP and RFC 951 BOOTP packets.
+        isc_throw(InvalidOperation, "Received BOOTP packet without vendor information extensions.");
     }
 
     if (buffer_in.getLength() - buffer_in.getPosition() < 4) {
-      // there is not enough data to hold magic DHCP cookie
-      isc_throw(Unexpected, "Truncated or no DHCP packet.");
+        // there is not enough data to hold magic DHCP cookie
+        isc_throw(Unexpected, "Truncated or no DHCP packet.");
     }
 
     uint32_t magic = buffer_in.readUint32();
     if (magic != DHCP_OPTIONS_COOKIE) {
-      isc_throw(Unexpected, "Invalid or missing DHCP magic cookie");
+        isc_throw(Unexpected, "Invalid or missing DHCP magic cookie");
     }
 
     size_t opts_len = buffer_in.getLength() - buffer_in.getPosition();
@@ -286,6 +286,14 @@ Pkt4::getName(const uint8_t type) {
     static const char* DHCPNAK_NAME = "DHCPNAK";
     static const char* DHCPRELEASE_NAME = "DHCPRELEASE";
     static const char* DHCPINFORM_NAME = "DHCPINFORM";
+    static const char* DHCPLEASEQUERY_NAME = "DHCPLEASEQUERY";
+    static const char* DHCPLEASEUNASSIGNED_NAME = "DHCPLEASEUNASSIGNED";
+    static const char* DHCPLEASEUNKNOWN_NAME = "DHCPLEASEUNKNOWN";
+    static const char* DHCPLEASEACTIVE_NAME = "DHCPLEASEACTIVE";
+    static const char* DHCPBULKLEASEQUERY_NAME = "DHCPBULKLEASEQUERY";
+    static const char* DHCPLEASEQUERYDONE_NAME = "DHCPLEASEQUERYDONE";
+    static const char* DHCPLEASEQUERYSTATUS_NAME = "DHCPLEASEQUERYSTATUS";
+    static const char* DHCPTLS_NAME = "DHCPTLS";
     static const char* UNKNOWN_NAME = "UNKNOWN";
 
     switch (type) {
@@ -313,6 +321,30 @@ Pkt4::getName(const uint8_t type) {
         case DHCPINFORM:
             return (DHCPINFORM_NAME);
 
+        case DHCPLEASEQUERY:
+            return (DHCPLEASEQUERY_NAME);
+
+        case DHCPLEASEUNASSIGNED:
+            return (DHCPLEASEUNASSIGNED_NAME);
+
+        case DHCPLEASEUNKNOWN:
+            return (DHCPLEASEUNKNOWN_NAME);
+
+        case DHCPLEASEACTIVE:
+            return (DHCPLEASEACTIVE_NAME);
+
+        case DHCPBULKLEASEQUERY:
+            return (DHCPBULKLEASEQUERY_NAME);
+
+        case DHCPLEASEQUERYDONE:
+            return (DHCPLEASEQUERYDONE_NAME);
+
+        case DHCPLEASEQUERYSTATUS:
+            return (DHCPLEASEQUERYSTATUS_NAME);
+
+        case DHCPTLS:
+            return (DHCPTLS_NAME);
+
         default:
             ;
     }
