@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -23,7 +23,6 @@
 namespace isc {
 namespace dhcp {
 
-
 /// @brief Tag for indexes by address.
 struct AddressIndexTag { };
 
@@ -39,8 +38,14 @@ struct HWAddressSubnetIdIndexTag { };
 /// @brief Tag for indexes by client and subnet identifiers.
 struct ClientIdSubnetIdIndexTag { };
 
-/// @brief Tag for indexes by client id, HW address and subnet id.
-struct ClientIdHWAddressSubnetIdIndexTag { };
+/// @brief Tag for indexes by subnet-id.
+struct SubnetIdIndexTag { };
+
+/// @brief Tag for index using DUID.
+struct DuidIndexTag { };
+
+/// @brief Tag for index using hostname.
+struct HostnameIndexTag { };
 
 /// @name Multi index containers holding DHCPv4 and DHCPv6 leases.
 ///
@@ -54,7 +59,7 @@ struct ClientIdHWAddressSubnetIdIndexTag { };
 /// - using a composite index: boolean flag indicating if the state is
 ///   "expired-reclaimed" and expiration time.
 ///
-/// Indexes can be accessed using the index number (from 0 to 2) or a
+/// Indexes can be accessed using the index number (from 0 to 5) or a
 /// name tag. It is recommended to use the tags to access indexes as
 /// they do not depend on the order of indexes in the container.
 typedef boost::multi_index_container<
@@ -103,8 +108,32 @@ typedef boost::multi_index_container<
                 boost::multi_index::const_mem_fun<Lease, int64_t,
                                                   &Lease::getExpirationTime>
             >
+        >,
+
+        // Specification of the fourth index starts here.
+        // This index sorts leases by SubnetID.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<SubnetIdIndexTag>,
+            boost::multi_index::member<Lease, isc::dhcp::SubnetID,
+            &Lease::subnet_id_>
+        >,
+
+        // Specification of the fifth index starts here
+        // This index is used to retrieve leases for matching duid.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<DuidIndexTag>,
+            boost::multi_index::const_mem_fun<Lease6,
+                                              const std::vector<uint8_t>&,
+                                              &Lease6::getDuidVector>
+        >,
+
+        // Specification of the sixth index starts here
+        // This index is used to retrieve leases for matching hostname.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<HostnameIndexTag>,
+            boost::multi_index::member<Lease, std::string, &Lease::hostname_>
         >
-     >
+    >
 > Lease6Storage; // Specify the type name of this container.
 
 /// @brief A multi index container holding DHCPv4 leases.
@@ -113,11 +142,10 @@ typedef boost::multi_index_container<
 /// - IPv6 address,
 /// - composite index: HW address and subnet id,
 /// - composite index: client id and subnet id,
-/// - composite index: HW address, client id and subnet id
 /// - using a composite index: boolean flag indicating if the state is
 ///   "expired-reclaimed" and expiration time.
 ///
-/// Indexes can be accessed using the index number (from 0 to 4) or a
+/// Indexes can be accessed using the index number (from 0 to 5) or a
 /// name tag. It is recommended to use the tags to access indexes as
 /// they do not depend on the order of indexes in the container.
 typedef boost::multi_index_container<
@@ -173,28 +201,6 @@ typedef boost::multi_index_container<
         >,
 
         // Specification of the fourth index starts here.
-        boost::multi_index::ordered_non_unique<
-            boost::multi_index::tag<ClientIdHWAddressSubnetIdIndexTag>,
-            // This is a composite index that uses three values to search for a
-            // lease: client id, HW address and subnet id.
-            boost::multi_index::composite_key<
-                Lease4,
-                // The client id can be retrieved from the Lease4 object by
-                // calling getClientIdVector const function.
-                boost::multi_index::const_mem_fun<Lease4, const std::vector<uint8_t>&,
-                                                  &Lease4::getClientIdVector>,
-                // The hardware address is held in the hwaddr_ object. We can
-                // access the raw data using lease->hwaddr_->hwaddr_, but Boost
-                // doesn't seem to provide a way to use member of a member for this,
-                // so we need a simple key extractor method (getHWAddrVector).
-                boost::multi_index::const_mem_fun<Lease, const std::vector<uint8_t>&,
-                                                  &Lease::getHWAddrVector>,
-                // The subnet id is accessed through the subnet_id_ member.
-                boost::multi_index::member<Lease, SubnetID, &Lease::subnet_id_>
-            >
-        >,
-
-        // Specification of the fifth index starts here.
         boost::multi_index::ordered_non_unique<
             boost::multi_index::tag<ExpirationIndexTag>,
             // This is a composite index that will be used to search for
@@ -210,6 +216,20 @@ typedef boost::multi_index_container<
                 boost::multi_index::const_mem_fun<Lease, int64_t,
                                                   &Lease::getExpirationTime>
             >
+        >,
+
+        // Specification of the fifth index starts here.
+        // This index sorts leases by SubnetID.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<SubnetIdIndexTag>,
+            boost::multi_index::member<Lease, isc::dhcp::SubnetID, &Lease::subnet_id_>
+        >,
+
+        // Specification of the seventh index starts here
+        // This index is used to retrieve leases for matching hostname.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<HostnameIndexTag>,
+            boost::multi_index::member<Lease, std::string, &Lease::hostname_>
         >
     >
 > Lease4Storage; // Specify the type name for this container.
@@ -229,10 +249,19 @@ typedef Lease6Storage::index<DuidIaidTypeIndexTag>::type Lease6StorageDuidIaidTy
 /// @brief DHCPv6 lease storage index by expiration time.
 typedef Lease6Storage::index<ExpirationIndexTag>::type Lease6StorageExpirationIndex;
 
+/// @brief DHCPv6 lease storage index by Subnet-id.
+typedef Lease6Storage::index<SubnetIdIndexTag>::type Lease6StorageSubnetIdIndex;
+
+/// @brief DHCPv6 lease storage index by DUID.
+typedef Lease6Storage::index<DuidIndexTag>::type Lease6StorageDuidIndex;
+
+/// @brief DHCPv6 lease storage index by hostname.
+typedef Lease6Storage::index<HostnameIndexTag>::type Lease6StorageHostnameIndex;
+
 /// @brief DHCPv4 lease storage index by address.
 typedef Lease4Storage::index<AddressIndexTag>::type Lease4StorageAddressIndex;
 
-/// @brief DHCPv4 lease storage index by exiration time.
+/// @brief DHCPv4 lease storage index by expiration time.
 typedef Lease4Storage::index<ExpirationIndexTag>::type Lease4StorageExpirationIndex;
 
 /// @brief DHCPv4 lease storage index by HW address and subnet identifier.
@@ -243,9 +272,11 @@ Lease4StorageHWAddressSubnetIdIndex;
 typedef Lease4Storage::index<ClientIdSubnetIdIndexTag>::type
 Lease4StorageClientIdSubnetIdIndex;
 
-/// @brief DHCPv4 lease storage index by client id, HW address and subnet id.
-typedef Lease4Storage::index<ClientIdHWAddressSubnetIdIndexTag>::type
-Lease4StorageClientIdHWAddressSubnetIdIndex;
+/// @brief DHCPv4 lease storage index subnet identifier.
+typedef Lease4Storage::index<SubnetIdIndexTag>::type Lease4StorageSubnetIdIndex;
+
+/// @brief DHCPv4 lease storage index by hostname.
+typedef Lease4Storage::index<HostnameIndexTag>::type Lease4StorageHostnameIndex;
 
 //@}
 } // end of isc::dhcp namespace
