@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,12 +10,12 @@
 #include <cc/data.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/cfg_option.h>
-#include <dhcpsrv/parsers/dhcp_parsers.h>
 #include <dhcpsrv/parsers/option_data_parser.h>
 #include <dhcpsrv/parsers/shared_network_parser.h>
 #include <dhcpsrv/parsers/simple_parser4.h>
 #include <dhcpsrv/parsers/simple_parser6.h>
 #include <dhcpsrv/shared_network.h>
+#include <boost/make_shared.hpp>
 #include <boost/pointer_cast.hpp>
 #include <string>
 
@@ -26,6 +26,10 @@ using namespace isc::util;
 namespace isc {
 namespace dhcp {
 
+SharedNetwork4Parser::SharedNetwork4Parser(bool check_iface)
+    : check_iface_(check_iface) {
+}
+
 SharedNetwork4Ptr
 SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
     SharedNetwork4Ptr shared_network;
@@ -40,30 +44,47 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
         std::string name = getString(shared_network_data, "name");
         shared_network.reset(new SharedNetwork4(name));
 
-        // Parse timers.
+        // Move from reservation mode to new reservations flags.
+        ElementPtr mutable_params;
+        mutable_params = boost::const_pointer_cast<Element>(shared_network_data);
+        BaseNetworkParser::moveReservationMode(mutable_params);
+
+        // Parse parameters common to all Network derivations.
         NetworkPtr network = boost::dynamic_pointer_cast<Network>(shared_network);
-        parseCommonTimers(shared_network_data, network);
+        parseCommon(mutable_params, network);
 
         // interface is an optional parameter
         if (shared_network_data->contains("interface")) {
-            shared_network->setIface(getString(shared_network_data, "interface"));
+            std::string iface = getString(shared_network_data, "interface");
+            if (!iface.empty()) {
+                if (check_iface_ && !IfaceMgr::instance().getIface(iface)) {
+                    ConstElementPtr error =
+                        shared_network_data->get("interface");
+                    isc_throw(DhcpConfigError,
+                              "Specified network interface name " << iface
+                              << " for shared network " << name
+                              << " is not present in the system ("
+                              << error->getPosition() << ")");
+                }
+                shared_network->setIface(iface);
+            }
         }
 
         if (shared_network_data->contains("option-data")) {
             auto json = shared_network_data->get("option-data");
             // Create parser instance for option-data.
             CfgOptionPtr cfg_option = shared_network->getCfgOption();
-            OptionDataListParser parser(AF_INET);
-            parser.parse(cfg_option, json);
+            auto parser = createOptionDataListParser();
+            parser->parse(cfg_option, json);
         }
 
         if (shared_network_data->contains("subnet4")) {
             auto json = shared_network_data->get("subnet4");
 
             // Create parser instance of subnet4.
-            Subnets4ListConfigParser parser;
+            auto parser = createSubnetsListParser();
             Subnet4Collection subnets;
-            parser.parse(subnets, json);
+            parser->parse(subnets, json);
 
             // Add all returned subnets into shared network.
             for (auto subnet = subnets.cbegin(); subnet != subnets.cend();
@@ -169,11 +190,13 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
             }
         }
 
-        // reservation-mode
-        parseHostReservationMode(shared_network_data, network);
-
         parseTeePercents(shared_network_data, network);
 
+        // Parse DDNS parameters
+        parseDdnsParams(shared_network_data, network);
+
+        // Parse lease cache parameters
+        parseCacheParams(shared_network_data, network);
     } catch (const DhcpConfigError&) {
         // Position was already added
         throw;
@@ -192,6 +215,22 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data) {
     return (shared_network);
 }
 
+boost::shared_ptr<OptionDataListParser>
+SharedNetwork4Parser::createOptionDataListParser() const {
+    auto parser = boost::make_shared<OptionDataListParser>(AF_INET);
+    return (parser);
+}
+
+boost::shared_ptr<Subnets4ListConfigParser>
+SharedNetwork4Parser::createSubnetsListParser() const {
+    auto parser = boost::make_shared<Subnets4ListConfigParser>(check_iface_);
+    return (parser);
+}
+
+SharedNetwork6Parser::SharedNetwork6Parser(bool check_iface)
+    : check_iface_(check_iface) {
+}
+
 SharedNetwork6Ptr
 SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
     SharedNetwork6Ptr shared_network;
@@ -206,12 +245,18 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
         std::string name = getString(shared_network_data, "name");
         shared_network.reset(new SharedNetwork6(name));
 
+        // Move from reservation mode to new reservations flags.
+        ElementPtr mutable_params;
+        mutable_params = boost::const_pointer_cast<Element>(shared_network_data);
+        BaseNetworkParser::moveReservationMode(mutable_params);
+
+        // Parse parameters common to all Network derivations.
         NetworkPtr network = boost::dynamic_pointer_cast<Network>(shared_network);
-        parseCommonTimers(shared_network_data, network);
+        parseCommon(mutable_params, network);
 
         // preferred-lifetime
-        shared_network->setPreferred(parseLifetime(shared_network_data,
-                                                   "preferred-lifetime"));
+        shared_network->setPreferred(parseIntTriplet(shared_network_data,
+                                                     "preferred-lifetime"));
 
         // Get interface-id option content. For now we support string
         // representation only
@@ -220,6 +265,7 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             ifaceid = getString(shared_network_data, "interface-id");
         }
 
+        // Interface is an optional parameter
         Optional<std::string> iface;
         if (shared_network_data->contains("interface")) {
             iface = getString(shared_network_data, "interface");
@@ -246,17 +292,20 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             shared_network->setInterfaceId(opt);
         }
 
-        // Get interface name. If it is defined, then the subnet is available
+        // Set interface name. If it is defined, then subnets are available
         // directly over specified network interface.
         if (!iface.unspecified() && !iface.empty()) {
+            if (check_iface_ && !IfaceMgr::instance().getIface(iface)) {
+                ConstElementPtr error = shared_network_data->get("interface");
+                isc_throw(DhcpConfigError,
+                          "Specified network interface name " << iface
+                          << " for shared network " << name
+                          << " is not present in the system ("
+                          << error->getPosition() << ")");
+            }
             shared_network->setIface(iface);
         }
 
-        // Interface is an optional parameter
-        if (shared_network_data->contains("interface")) {
-            shared_network->setIface(getString(shared_network_data, "interface"));
-        }
-
         if (shared_network_data->contains("rapid-commit")) {
             shared_network->setRapidCommit(getBoolean(shared_network_data,
                                                       "rapid-commit"));
@@ -266,8 +315,8 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             auto json = shared_network_data->get("option-data");
             // Create parser instance for option-data.
             CfgOptionPtr cfg_option = shared_network->getCfgOption();
-            OptionDataListParser parser(AF_INET6);
-            parser.parse(cfg_option, json);
+            auto parser = createOptionDataListParser();
+            parser->parse(cfg_option, json);
         }
 
         if (shared_network_data->contains("client-class")) {
@@ -300,9 +349,9 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             auto json = shared_network_data->get("subnet6");
 
             // Create parser instance of subnet6.
-            Subnets6ListConfigParser parser;
+            auto parser = createSubnetsListParser();
             Subnet6Collection subnets;
-            parser.parse(subnets, json);
+            parser->parse(subnets, json);
 
             // Add all returned subnets into shared network.
             for (auto subnet = subnets.cbegin(); subnet != subnets.cend();
@@ -321,11 +370,13 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
             }
         }
 
-        // reservation-mode
-        parseHostReservationMode(shared_network_data, network);
-
         parseTeePercents(shared_network_data, network);
 
+        // Parse DDNS parameters
+        parseDdnsParams(shared_network_data, network);
+
+        // Parse lease cache parameters
+        parseCacheParams(shared_network_data, network);
     } catch (const std::exception& ex) {
         isc_throw(DhcpConfigError, ex.what() << " ("
                   << shared_network_data->getPosition() << ")");
@@ -341,5 +392,17 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data) {
     return (shared_network);
 }
 
+boost::shared_ptr<OptionDataListParser>
+SharedNetwork6Parser::createOptionDataListParser() const {
+    auto parser = boost::make_shared<OptionDataListParser>(AF_INET6);
+    return (parser);
+}
+
+boost::shared_ptr<Subnets6ListConfigParser>
+SharedNetwork6Parser::createSubnetsListParser() const {
+    auto parser = boost::make_shared<Subnets6ListConfigParser>(check_iface_);
+    return (parser);
+}
+
 } // end of namespace isc::dhcp
 } // end of namespace isc
