@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -71,6 +71,9 @@ const SimpleKeywords SimpleParser4::GLOBAL4_PARAMETERS = {
     { "config-control",                 Element::map },
     { "server-tag",                     Element::string },
     { "reservation-mode",               Element::string },
+    { "reservations-global",            Element::boolean },
+    { "reservations-in-subnet",         Element::boolean },
+    { "reservations-out-of-pool",       Element::boolean },
     { "calculate-tee-times",            Element::boolean },
     { "t1-percent",                     Element::real },
     { "t2-percent",                     Element::real },
@@ -86,7 +89,14 @@ const SimpleKeywords SimpleParser4::GLOBAL4_PARAMETERS = {
     { "store-extended-info",            Element::boolean },
     { "statistic-default-sample-count", Element::integer },
     { "statistic-default-sample-age",   Element::integer },
-    { "multi-threading",                Element::map }
+    { "multi-threading",                Element::map },
+    { "cache-threshold",                Element::real },
+    { "cache-max-age",                  Element::integer },
+    { "ip-reservations-unique",         Element::boolean },
+    { "ddns-update-on-renew",           Element::boolean },
+    { "ddns-use-conflict-resolution",   Element::boolean },
+    { "compatibility",                  Element::map },
+    { "parked-packet-limit",            Element::integer },
 };
 
 /// @brief This table defines default global values for DHCPv4
@@ -105,7 +115,9 @@ const SimpleDefaults SimpleParser4::GLOBAL4_DEFAULTS = {
     { "server-hostname",                Element::string,  "" },
     { "boot-file-name",                 Element::string,  "" },
     { "server-tag",                     Element::string,  "" },
-    { "reservation-mode",               Element::string,  "all" },
+    { "reservations-global",            Element::boolean, "false" },
+    { "reservations-in-subnet",         Element::boolean, "true" },
+    { "reservations-out-of-pool",       Element::boolean, "false" },
     { "calculate-tee-times",            Element::boolean, "false" },
     { "t1-percent",                     Element::real,    ".50" },
     { "t2-percent",                     Element::real,    ".875" },
@@ -119,7 +131,11 @@ const SimpleDefaults SimpleParser4::GLOBAL4_DEFAULTS = {
     { "hostname-char-replacement",      Element::string,  "" },
     { "store-extended-info",            Element::boolean, "false" },
     { "statistic-default-sample-count", Element::integer, "20" },
-    { "statistic-default-sample-age",   Element::integer, "0" }
+    { "statistic-default-sample-age",   Element::integer, "0" },
+    { "ip-reservations-unique",         Element::boolean, "true" },
+    { "ddns-update-on-renew",           Element::boolean, "false" },
+    { "ddns-use-conflict-resolution",   Element::boolean, "true" },
+    { "parked-packet-limit",            Element::integer, "256" },
 };
 
 /// @brief This table defines all option definition parameters.
@@ -199,6 +215,9 @@ const SimpleKeywords SimpleParser4::SUBNET4_PARAMETERS = {
     { "require-client-classes",         Element::list },
     { "reservations",                   Element::list },
     { "reservation-mode",               Element::string },
+    { "reservations-global",            Element::boolean },
+    { "reservations-in-subnet",         Element::boolean },
+    { "reservations-out-of-pool",       Element::boolean },
     { "relay",                          Element::map },
     { "match-client-id",                Element::boolean },
     { "authoritative",                  Element::boolean },
@@ -222,7 +241,11 @@ const SimpleKeywords SimpleParser4::SUBNET4_PARAMETERS = {
     { "hostname-char-set",              Element::string },
     { "hostname-char-replacement",      Element::string },
     { "store-extended-info",            Element::boolean },
-    { "metadata",                       Element::map }
+    { "metadata",                       Element::map },
+    { "cache-threshold",                Element::real },
+    { "cache-max-age",                  Element::integer },
+    { "ddns-update-on-renew",           Element::boolean },
+    { "ddns-use-conflict-resolution",   Element::boolean }
 };
 
 /// @brief This table defines default values for each IPv4 subnet.
@@ -273,7 +296,9 @@ const ParamsList SimpleParser4::INHERIT_TO_SUBNET4 = {
     "calculate-tee-times",
     "t1-percent",
     "t2-percent",
-    "store-extended-info"
+    "store-extended-info",
+    "cache-threshold",
+    "cache-max-age"
 };
 
 /// @brief This table defines all pool parameters.
@@ -310,6 +335,9 @@ const SimpleKeywords SimpleParser4::SHARED_NETWORK4_PARAMETERS = {
     { "boot-file-name",                 Element::string },
     { "relay",                          Element::map },
     { "reservation-mode",               Element::string },
+    { "reservations-global",            Element::boolean },
+    { "reservations-in-subnet",         Element::boolean },
+    { "reservations-out-of-pool",       Element::boolean },
     { "client-class",                   Element::string },
     { "require-client-classes",         Element::list },
     { "valid-lifetime",                 Element::integer },
@@ -329,7 +357,11 @@ const SimpleKeywords SimpleParser4::SHARED_NETWORK4_PARAMETERS = {
     { "hostname-char-set",              Element::string },
     { "hostname-char-replacement",      Element::string },
     { "store-extended-info",            Element::boolean },
-    { "metadata",                       Element::map }
+    { "metadata",                       Element::map },
+    { "cache-threshold",                Element::real },
+    { "cache-max-age",                  Element::integer },
+    { "ddns-update-on-renew",           Element::boolean },
+    { "ddns-use-conflict-resolution",   Element::boolean }
 };
 
 /// @brief This table defines default values for each IPv4 shared network.
