@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -93,17 +93,19 @@ public:
     /// @brief Wrapper method for invoking AllocEngine4::updateLease4ExtendedInfo().
     /// @param lease lease to update
     /// @param ctx current packet processing context
-    void callUpdateLease4ExtendedInfo(const Lease4Ptr& lease,
+    /// @return true if extended information was changed
+    bool callUpdateLease4ExtendedInfo(const Lease4Ptr& lease,
                                       AllocEngine::ClientContext4& ctx) const {
-        updateLease4ExtendedInfo(lease,ctx);
+        return (updateLease4ExtendedInfo(lease, ctx));
     }
 
     /// @brief Wrapper method for invoking AllocEngine6::updateLease6ExtendedInfo().
     /// @param lease lease to update
     /// @param ctx current packet processing context
-    void callUpdateLease6ExtendedInfo(const Lease6Ptr& lease,
+    /// @return true if extended information was changed
+    bool callUpdateLease6ExtendedInfo(const Lease6Ptr& lease,
                                       AllocEngine::ClientContext6& ctx) const {
-        updateLease6ExtendedInfo(lease,ctx);
+        return (updateLease6ExtendedInfo(lease, ctx));
     }
 
 };
@@ -240,6 +242,7 @@ public:
         EXPECT_EQ(fqdn_rev_, lease->fqdn_rev_);
         EXPECT_EQ(hostname_, lease->hostname_);
         EXPECT_TRUE(*lease->duid_ == *duid);
+        EXPECT_EQ(0, lease->reuseable_valid_lft_);
         /// @todo: check cltt
     }
 
@@ -413,7 +416,7 @@ public:
     /// can succeed. The invalid hint should be ignored completely.
     ///
     /// @param type Lease type
-    /// @param hint hint (as send by a client)
+    /// @param hint hint (as sent by a client)
     /// @param expected_pd_len (used in validation)
     void allocBogusHint6(Lease::Type type, asiolink::IOAddress hint,
                          uint8_t expected_pd_len);
@@ -559,6 +562,7 @@ public:
             EXPECT_TRUE(*lease->client_id_ == *clientid_);
         }
         EXPECT_TRUE(*lease->hwaddr_ == *hwaddr_);
+        EXPECT_EQ(0, lease->reuseable_valid_lft_);
         /// @todo: check cltt
     }
 
@@ -620,8 +624,8 @@ public:
                                      ///< allocation engine functions.
 };
 
-}; // namespace test
-}; // namespace dhcp
-}; // namespace isc
+} // namespace test
+} // namespace dhcp
+} // namespace isc
 
 #endif
