@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -15,6 +15,7 @@
 #include <dhcpsrv/parsers/dhcp_parsers.h>
 #include <asiolink/io_address.h>
 #include <eval/evaluate.h>
+#include <testutils/gtest_utils.h>
 #include <gtest/gtest.h>
 #include <sstream>
 #include <stdint.h>
@@ -102,7 +103,7 @@ protected:
     /// occur.
     /// @return Returns a pointer to class instance created, or NULL if
     /// for some unforeseen reason it wasn't created in the local dictionary
-    /// @throw indirectly, exceptions convertring the JSON text to elements,
+    /// @throw indirectly, exceptions converting the JSON text to elements,
     /// or by the parsing itself are not caught
     ClientClassDefPtr parseClientClassDef(const std::string& config,
                                           uint16_t family) {
@@ -128,7 +129,7 @@ protected:
     }
 
     /// @brief Test that client class parser throws when unspported parameter
-    /// is specfied.
+    /// is specified.
     ///
     /// @param config JSON string containing the client class configuration.
     /// @param family The address family indicating whether the DHCPv4 or
@@ -156,18 +157,21 @@ protected:
     /// @param config - JSON string containing the list of definitions to parse.
     /// @param family - the address family in which the parsing should
     /// occur.
+    /// @param check_dependencies - indicates if the parser should check whether
+    /// referenced classes exist.
     /// @return Returns a pointer to class dictionary created
-    /// @throw indirectly, execptions convertring the JSON text to elements,
+    /// @throw indirectly, exceptions converting the JSON text to elements,
     /// or by the parsing itself are not caught
     ClientClassDictionaryPtr parseClientClassDefList(const std::string& config,
-                                                     uint16_t family)
+                                                     uint16_t family,
+                                                     bool check_dependencies = true)
     {
         // Turn config into elements.  This may emit exceptions.
         ElementPtr config_element = Element::fromJSON(config);
 
         // Parse the configuration. This may emit exceptions.
         ClientClassDefListParser parser;
-        return (parser.parse(config_element, family));
+        return (parser.parse(config_element, family, check_dependencies));
     }
 };
 
@@ -1086,6 +1090,24 @@ TEST_F(ClientClassDefListParserTest, dependentNotDefined) {
     EXPECT_THROW(parseClientClassDefList(cfg_text, AF_INET6), DhcpConfigError);
 }
 
+// Verifies that error is not reported when a class references another
+// not defined class, but dependency checking is disabled.
+TEST_F(ClientClassDefListParserTest, dependencyCheckingDisabled) {
+    std::string cfg_text =
+        "[ \n"
+        "   { \n"
+        "       \"name\": \"one\", \n"
+        "       \"test\": \"member('foo')\" \n"
+        "   } \n"
+        "] \n";
+    try {
+        parseClientClassDefList(cfg_text, AF_INET6, false);
+    } catch ( const std::exception& ex) {
+        std::cout << ex.what() << std::endl;
+    }
+    EXPECT_NO_THROW(parseClientClassDefList(cfg_text, AF_INET6, false));
+}
+
 // Verifies that forward dependencies will not parse.
 TEST_F(ClientClassDefListParserTest, dependentForwardError) {
     std::string cfg_text =
@@ -1257,8 +1279,7 @@ TEST_F(ClientClassDefListParserTest, builtinCheckError) {
     EXPECT_THROW(parseClientClassDefList(cfg_text, AF_INET6), DhcpConfigError);
 }
 
-// Verifies that the special DROP class can't be required or
-// dependent on KNOWN/UNKNOWN
+// Verifies that the special DROP class can't be required.
 TEST_F(ClientClassDefListParserTest, dropCheckError) {
     std::string cfg_text =
         "[ \n"
@@ -1280,6 +1301,7 @@ TEST_F(ClientClassDefListParserTest, dropCheckError) {
 
     EXPECT_THROW(parseClientClassDefList(cfg_text, AF_INET), DhcpConfigError);
 
+    // This constraint was relaxed in #1815.
     cfg_text =
         "[ \n"
         "   { \n"
@@ -1288,7 +1310,132 @@ TEST_F(ClientClassDefListParserTest, dropCheckError) {
         "   } \n"
         "] \n";
 
-    EXPECT_THROW(parseClientClassDefList(cfg_text, AF_INET6), DhcpConfigError);
+    EXPECT_NO_THROW(parseClientClassDefList(cfg_text, AF_INET6));
 }
 
+// Verify the ability to configure valid lifetime triplet.
+TEST_F(ClientClassDefParserTest, validLifetimeTests) {
+
+    struct Scenario {
+        std::string desc_;
+        std::string cfg_txt_;
+        Triplet<uint32_t> exp_triplet_;
+    };
+
+    std::vector<Scenario> scenarios = {
+        {
+            "unspecified",
+            "",
+            Triplet<uint32_t>()
+        },
+        {
+            "valid only",
+            "\"valid-lifetime\": 100",
+            Triplet<uint32_t>(100)
+        },
+        {
+            "min only",
+            "\"min-valid-lifetime\": 50",
+            Triplet<uint32_t>(50, 50, 50)
+        },
+        {
+            "max only",
+            "\"max-valid-lifetime\": 75",
+            Triplet<uint32_t>(75, 75, 75)
+        },
+        {
+            "all three",
+            "\"min-valid-lifetime\": 25, \"valid-lifetime\": 50, \"max-valid-lifetime\": 75",
+            Triplet<uint32_t>(25, 50, 75)
+        }
+    };
+
+    for (auto scenario : scenarios) {
+        SCOPED_TRACE(scenario.desc_); {
+            std::stringstream oss;
+            oss << "{ \"name\": \"foo\"";
+            if (!scenario.cfg_txt_.empty()) {
+                oss << ",\n" << scenario.cfg_txt_;
+            }
+            oss << "\n}\n";
+
+            ClientClassDefPtr class_def;
+            ASSERT_NO_THROW_LOG(class_def = parseClientClassDef(oss.str(), AF_INET));
+            ASSERT_TRUE(class_def);
+            if (scenario.exp_triplet_.unspecified()) {
+                EXPECT_TRUE(class_def->getValid().unspecified());
+            } else {
+                EXPECT_EQ(class_def->getValid(), scenario.exp_triplet_);
+                EXPECT_EQ(class_def->getValid().getMin(), scenario.exp_triplet_.getMin());
+                EXPECT_EQ(class_def->getValid().get(), scenario.exp_triplet_.get());
+                EXPECT_EQ(class_def->getValid().getMax(), scenario.exp_triplet_.getMax());
+            }
+        }
+    }
+}
+
+// Verify the ability to configure lease preferred lifetime triplet.
+TEST_F(ClientClassDefParserTest, preferredLifetimeTests) {
+
+    struct Scenario {
+        std::string desc_;
+        std::string cfg_txt_;
+        Triplet<uint32_t> exp_triplet_;
+    };
+
+    std::vector<Scenario> scenarios = {
+        {
+            "unspecified",
+            "",
+            Triplet<uint32_t>()
+        },
+        {
+            "preferred only",
+            "\"preferred-lifetime\": 100",
+            Triplet<uint32_t>(100)
+        },
+        {
+            "min only",
+            "\"min-preferred-lifetime\": 50",
+            Triplet<uint32_t>(50, 50, 50)
+        },
+        {
+            "max only",
+            "\"max-preferred-lifetime\": 75",
+            Triplet<uint32_t>(75, 75, 75)
+        },
+        {
+            "all three",
+            "\"min-preferred-lifetime\": 25,"
+            "\"preferred-lifetime\": 50,"
+            "\"max-preferred-lifetime\": 75",
+            Triplet<uint32_t>(25, 50, 75)
+        }
+    };
+
+    for (auto scenario : scenarios) {
+        SCOPED_TRACE(scenario.desc_); {
+            std::stringstream oss;
+            oss << "{ \"name\": \"foo\"";
+            if (!scenario.cfg_txt_.empty()) {
+                oss << ",\n" << scenario.cfg_txt_;
+            }
+            oss << "\n}\n";
+
+            ClientClassDefPtr class_def;
+            ASSERT_NO_THROW_LOG(class_def = parseClientClassDef(oss.str(), AF_INET6));
+            ASSERT_TRUE(class_def);
+            if (scenario.exp_triplet_.unspecified()) {
+                EXPECT_TRUE(class_def->getPreferred().unspecified());
+            } else {
+                EXPECT_EQ(class_def->getPreferred(), scenario.exp_triplet_);
+                EXPECT_EQ(class_def->getPreferred().getMin(), scenario.exp_triplet_.getMin());
+                EXPECT_EQ(class_def->getPreferred().get(), scenario.exp_triplet_.get());
+                EXPECT_EQ(class_def->getPreferred().getMax(), scenario.exp_triplet_.getMax());
+            }
+        }
+    }
+}
+
+
 } // end of anonymous namespace
