@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,7 +9,9 @@
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/cfg_duid.h>
 #include <dhcpsrv/parsers/duid_config_parser.h>
+#include <dhcpsrv/parsers/dhcp_parsers.h>
 #include <dhcpsrv/testutils/config_result_check.h>
+#include <testutils/test_to_element.h>
 #include <util/encode/hex.h>
 #include <gtest/gtest.h>
 #include <limits>
@@ -26,6 +28,13 @@ namespace {
 class DUIDConfigParserTest : public ::testing::Test {
 public:
 
+    /// @brief constructor
+    ///
+    /// Initializes cfg_duid_ to a new empty object
+    DUIDConfigParserTest()
+        :cfg_duid_(new CfgDUID()){
+    }
+
     /// @brief Creates simple configuration with DUID type only.
     ///
     /// @param duid_type DUID type in the textual format.
@@ -82,9 +91,12 @@ public:
     /// @param vec Input vector.
     /// @return String of hexadecimal digits converted from vector.
     std::string toString(const std::vector<uint8_t>& vec) const;
+
+    /// Config DUID pointer
+    CfgDUIDPtr cfg_duid_;
 };
 
-std::string 
+std::string
 DUIDConfigParserTest::createConfigWithType(const std::string& duid_type) const {
     std::ostringstream s;
     s << "{ \"type\": \"" << duid_type << "\" }";
@@ -103,10 +115,9 @@ void
 DUIDConfigParserTest::build(const std::string& config) const {
     ElementPtr config_element = Element::fromJSON(config);
     DUIDConfigParser parser;
-    parser.build(config_element);
+    parser.parse(cfg_duid_, config_element);
 }
 
-
 void
 DUIDConfigParserTest::testTypeOnly(const DUID::DUIDType& duid_type,
                                    const std::string& duid_type_text) const {
@@ -115,12 +126,12 @@ DUIDConfigParserTest::testTypeOnly(const DUID::DUIDType& duid_type,
 
     // Make sure that the type is correct and that other parameters are set
     // to their defaults.
-    CfgDUIDPtr cfg_duid = CfgMgr::instance().getStagingCfg()->getCfgDUID();
-    EXPECT_EQ(duid_type, cfg_duid->getType());
-    EXPECT_TRUE(cfg_duid->getIdentifier().empty());
-    EXPECT_EQ(0, cfg_duid->getHType());
-    EXPECT_EQ(0, cfg_duid->getTime());
-    EXPECT_EQ(0, cfg_duid->getEnterpriseId());
+    ASSERT_TRUE(cfg_duid_);
+    EXPECT_EQ(duid_type, cfg_duid_->getType());
+    EXPECT_TRUE(cfg_duid_->getIdentifier().empty());
+    EXPECT_EQ(0, cfg_duid_->getHType());
+    EXPECT_EQ(0, cfg_duid_->getTime());
+    EXPECT_EQ(0, cfg_duid_->getEnterpriseId());
 }
 
 void
@@ -172,22 +183,27 @@ TEST_F(DUIDConfigParserTest, noType) {
 // This test verifies that all parameters can be set.
 TEST_F(DUIDConfigParserTest, allParameters) {
     // Set all parameters.
-    ASSERT_NO_THROW(build("{ \"type\": \"EN\","
-                          "  \"identifier\": \"ABCDEF\","
-                          "  \"time\": 100,"
-                          "  \"htype\": 8,"
-                          "  \"enterprise-id\": 2024,"
-                          "  \"persist\": false"
-                          "}"));
+    std::string config = "{"
+        " \"type\": \"EN\","
+        " \"identifier\": \"ABCDEF\","
+        " \"time\": 100,"
+        " \"htype\": 8,"
+        " \"enterprise-id\": 2024,"
+        " \"persist\": false"
+        "}";
+    ASSERT_NO_THROW(build(config));
 
     // Verify that parameters have been set correctly.
-    CfgDUIDPtr cfg_duid = CfgMgr::instance().getStagingCfg()->getCfgDUID();
-    EXPECT_EQ(DUID::DUID_EN, cfg_duid->getType());
-    EXPECT_EQ("ABCDEF", toString(cfg_duid->getIdentifier()));
-    EXPECT_EQ(8, cfg_duid->getHType());
-    EXPECT_EQ(100, cfg_duid->getTime());
-    EXPECT_EQ(2024, cfg_duid->getEnterpriseId());
-    EXPECT_FALSE(cfg_duid->persist());
+    ASSERT_TRUE(cfg_duid_);
+    EXPECT_EQ(DUID::DUID_EN, cfg_duid_->getType());
+    EXPECT_EQ("ABCDEF", toString(cfg_duid_->getIdentifier()));
+    EXPECT_EQ(8, cfg_duid_->getHType());
+    EXPECT_EQ(100, cfg_duid_->getTime());
+    EXPECT_EQ(2024, cfg_duid_->getEnterpriseId());
+    EXPECT_FALSE(cfg_duid_->persist());
+
+    // Check the config can be got back.
+    isc::test::runToElementTest<CfgDUID>(config, *cfg_duid_);
 }
 
 // Test out of range values for time.
