@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,8 +7,13 @@
 #ifndef GENERIC_LEASE_MGR_UNITTEST_H
 #define GENERIC_LEASE_MGR_UNITTEST_H
 
+#include <asiolink/io_service.h>
 #include <dhcpsrv/lease_mgr.h>
+#include <dhcpsrv/timer_mgr.h>
+
 #include <gtest/gtest.h>
+
+#include <boost/make_shared.hpp>
 #include <vector>
 #include <set>
 
@@ -68,8 +73,7 @@ public:
     /// @param address Address to use for the initialization
     ///
     /// @return Lease6Ptr.  This will not point to anything if the
-    /// initialization
-    ///         failed (e.g. unknown address).
+    ///         initialization failed (e.g. unknown address).
     Lease6Ptr initializeLease6(std::string address);
 
     /// @brief Check Leases present and different
@@ -100,7 +104,7 @@ public:
 
     /// @brief Compares a StatsMgr statistic to an expected value
     ///
-    /// Attempt to fetch the named statistic from the StatsMg and if
+    /// Attempt to fetch the named statistic from the StatsMgr and if
     /// found, compare its observed value to the given value.
     /// Fails if the stat is not found or if the values do not match.
     ///
@@ -123,8 +127,11 @@ public:
     /// @param address - IPv4 address for the lease
     /// @param subnet_id - subnet ID to which the lease belongs
     /// @param state - the state of the lease
-    void makeLease4(const std::string& address, const SubnetID& subnet_id,
-                    const uint32_t state = Lease::STATE_DEFAULT);
+    ///
+    /// @return pointer to created Lease4
+    Lease4Ptr makeLease4(const std::string& address,
+                         const SubnetID& subnet_id,
+                         const uint32_t state = Lease::STATE_DEFAULT);
 
     /// @brief Constructs a minimal IPv6 lease and adds it to the lease storage
     ///
@@ -135,9 +142,13 @@ public:
     /// @param prefix_len = length of the prefix (should be 0 for TYPE_NA)
     /// @param subnet_id - subnet ID to which the lease belongs
     /// @param state - the state of the lease
-    void makeLease6(const Lease::Type& type, const std::string& address,
-                    uint8_t prefix_len, const SubnetID& subnet_id,
-                    const uint32_t state = Lease::STATE_DEFAULT);
+    ///
+    /// @return pointer to created Lease6
+    Lease6Ptr makeLease6(const Lease::Type& type,
+                         const std::string& address,
+                         uint8_t prefix_len,
+                         const SubnetID& subnet_id,
+                         const uint32_t state = Lease::STATE_DEFAULT);
 
     /// @brief checks that addLease, getLease4(addr) and deleteLease() works
     void testBasicLease4();
@@ -145,6 +156,9 @@ public:
     /// @brief checks that invalid dates are safely handled.
     void testMaxDate4();
 
+    /// @brief checks that infinite lifetimes do not overflow.
+    void testInfiniteLifeTime4();
+
     /// @brief Test lease retrieval using client id.
     void testGetLease4ClientId();
 
@@ -160,9 +174,6 @@ public:
     /// HWAddr information.
     void testGetLease4HWAddr2();
 
-    /// @brief Test lease retrieval using client id, HW address and subnet id.
-    void testGetLease4ClientIdHWAddrSubnetId();
-
     /// @brief Get lease4 by hardware address (2)
     ///
     /// Check that the system can cope with getting a hardware address of
@@ -201,6 +212,9 @@ public:
     /// @brief Test method which returns all IPv4 leases for Subnet ID.
     void testGetLeases4SubnetId();
 
+    /// @brief Test method which returns all IPv4 leases for Hostname.
+    void testGetLeases4Hostname();
+
     /// @brief Test method which returns all IPv4 leases.
     void testGetLeases4();
 
@@ -210,6 +224,9 @@ public:
     /// @brief Test method which returns all IPv6 leases for Subnet ID.
     void testGetLeases6SubnetId();
 
+    /// @brief Test method which returns all IPv6 leases for Hostname.
+    void testGetLeases6Hostname();
+
     /// @brief Test making/fetching leases with IAIDs > signed 32-bit max.
     void testLease6LargeIaidCheck();
 
@@ -244,6 +261,9 @@ public:
     /// @brief Checks that invalid dates are safely handled.
     void testMaxDate6();
 
+    /// @brief checks that infinite lifetimes do not overflow.
+    void testInfiniteLifeTime6();
+
     /// @brief Checks that Lease6 can be stored with and without a hardware address.
     void testLease6MAC();
 
@@ -279,7 +299,7 @@ public:
     /// @brief verifies getLeases6 method by DUID
     ///
     /// Adds 3 leases to backend and retrieves, verifes empty
-    /// retrival of non existent DUID.
+    /// retrieval of non existent DUID.
     void testGetLeases6Duid();
 
     /// @brief Checks that getLease6() works with different DUID sizes
@@ -302,11 +322,23 @@ public:
     /// Checks that the code is able to update an IPv4 lease in the database.
     void testUpdateLease4();
 
+    /// @brief Lease4 concurrent update test
+    ///
+    /// Checks that the code is not able to concurrently update an IPv4 lease in
+    /// the database.
+    void testConcurrentUpdateLease4();
+
     /// @brief Lease6 update test
     ///
     /// Checks that the code is able to update an IPv6 lease in the database.
     void testUpdateLease6();
 
+    /// @brief Lease6 concurrent update test
+    ///
+    /// Checks that the code is not able to concurrently update an IPv6 lease in
+    /// the database.
+    void testConcurrentUpdateLease6();
+
     /// @brief Check that the IPv6 lease can be added, removed and recreated.
     ///
     /// This test creates a lease, removes it and then recreates it with some
@@ -342,6 +374,14 @@ public:
     /// - reclaimed leases are not returned.
     void testGetExpiredLeases6();
 
+    /// @brief Checks that DHCPv4 leases with infinite valid lifetime
+    /// will never expire.
+    void testInfiniteAreNotExpired4();
+
+    /// @brief Checks that DHCPv6 leases with infinite valid lifetime
+    /// will never expire.
+    void testInfiniteAreNotExpired6();
+
     /// @brief Checks that declined IPv4 leases that have expired can be retrieved.
     ///
     /// This test checks that the following:
@@ -407,7 +447,7 @@ public:
 
     /// @brief Checks operation of v4 LeaseStatsQuery variants
     ///
-    /// It creates three subnets with leasese in various states in
+    /// It creates three subnets with leases in various states in
     /// each.  It runs and verifies the returned query contents for
     /// each of the v4 startLeaseQuery variants:
     ///
@@ -419,7 +459,7 @@ public:
 
     /// @brief Checks operation of v6 LeaseStatsQuery variants
     ///
-    /// It creates three subnets with leasese in various states in
+    /// It creates three subnets with leases in various states in
     /// each.  It runs and verifies the returned query contents for
     /// each of the v6 startLeaseQuery variants:
     ///
@@ -429,6 +469,26 @@ public:
     ///
     void testLeaseStatsQuery6();
 
+    /// @brief Checks if v4 LeaseStatsQuery can get bad attribution.
+    ///
+    /// It creates two subnets with leases and move one from the first
+    /// to the second. If counters are not updated this can lead to
+    /// bad attribution i.e. a lease is counted in a subnet when it
+    /// belongs to another one.
+    ///
+    void testLeaseStatsQueryAttribution4();
+
+    /// @brief Checks if v6 LeaseStatsQuery can get bad attribution.
+    ///
+    /// It creates two subnets with leases and move one from the first
+    /// to the second. If counters are not updated this can lead to
+    /// bad attribution i.e. a lease is counted in a subnet when it
+    /// belongs to another one.
+    ///
+    /// @note We can check the lease type change too but in the real
+    /// world this never happens.
+    void testLeaseStatsQueryAttribution6();
+
     /// @brief Compares LeaseQueryStats content to expected set of rows
     ///
     /// @param qry - a started LeaseStatsQuery
@@ -463,12 +523,23 @@ public:
 
 class LeaseMgrDbLostCallbackTest : public ::testing::Test {
 public:
-    LeaseMgrDbLostCallbackTest() {
-        db::DatabaseConnection::db_lost_callback = 0;
+    LeaseMgrDbLostCallbackTest()
+        : db_lost_callback_called_(0), db_recovered_callback_called_(0),
+          db_failed_callback_called_(0),
+          io_service_(boost::make_shared<isc::asiolink::IOService>()) {
+        db::DatabaseConnection::db_lost_callback_ = 0;
+        db::DatabaseConnection::db_recovered_callback_ = 0;
+        db::DatabaseConnection::db_failed_callback_ = 0;
+        LeaseMgr::setIOService(io_service_);
+        TimerMgr::instance()->setIOService(io_service_);
     }
 
     virtual ~LeaseMgrDbLostCallbackTest() {
-        db::DatabaseConnection::db_lost_callback = 0;
+        db::DatabaseConnection::db_lost_callback_ = 0;
+        db::DatabaseConnection::db_recovered_callback_ = 0;
+        db::DatabaseConnection::db_failed_callback_ = 0;
+        LeaseMgr::setIOService(isc::asiolink::IOServicePtr());
+        TimerMgr::instance()->unregisterTimers();
     }
 
     /// @brief Prepares the class for a test.
@@ -483,10 +554,10 @@ public:
     /// we created and toss our lease manager.
     virtual void TearDown();
 
-    /// @brief Abstract method for destroying the back end specific shcema
+    /// @brief Abstract method for destroying the back end specific schema
     virtual void destroySchema() = 0;
 
-    /// @brief Abstract method for creating the back end specific shcema
+    /// @brief Abstract method for creating the back end specific schema
     virtual void createSchema() = 0;
 
     /// @brief Abstract method which returns the back end specific connection
@@ -504,26 +575,82 @@ public:
     /// open should be handled directly by the application layer.
     void testNoCallbackOnOpenFailure();
 
-    /// @brief Verifies the host manager's behavior if DB connection is lost
+    /// @brief Verifies the lease manager's behavior if DB connection is lost
     ///
-    /// This function creates a lease manager with an back end that
-    /// supports connectivity lost callback (currently only MySQL and
-    /// PostgreSQL currently).  It verifies connectivity by issuing a known
-    /// valid query.  Next it simulates connectivity lost by identifying and
-    /// closing the socket connection to the host backend.  It then reissues
-    /// the query and verifies that:
+    /// This function creates a lease manager with a back end that supports
+    /// connectivity lost callback (currently only MySQL and PostgreSQL). It
+    /// verifies connectivity by issuing a known valid query. Next it simulates
+    /// connectivity lost by identifying and closing the socket connection to
+    /// the CB backend. It then reissues the query and verifies that:
     /// -# The Query throws  DbOperationError (rather than exiting)
     /// -# The registered DbLostCallback was invoked
-    void testDbLostCallback();
+    /// -# The registered DbRecoveredCallback was invoked
+    void testDbLostAndRecoveredCallback();
 
-    /// @brief Callback function registered with the host manager
+    /// @brief Verifies the lease manager's behavior if DB connection is lost
+    ///
+    /// This function creates a lease manager with a back end that supports
+    /// connectivity lost callback (currently only MySQL and PostgreSQL). It
+    /// verifies connectivity by issuing a known valid query. Next it simulates
+    /// connectivity lost by identifying and closing the socket connection to
+    /// the CB backend. It then reissues the query and verifies that:
+    /// -# The Query throws  DbOperationError (rather than exiting)
+    /// -# The registered DbLostCallback was invoked
+    /// -# The registered DbFailedCallback was invoked
+    void testDbLostAndFailedCallback();
+
+    /// @brief Verifies the lease manager's behavior if DB connection is lost
+    ///
+    /// This function creates a lease manager with a back end that supports
+    /// connectivity lost callback (currently only MySQL and PostgreSQL). It
+    /// verifies connectivity by issuing a known valid query. Next it simulates
+    /// connectivity lost by identifying and closing the socket connection to
+    /// the CB backend. It then reissues the query and verifies that:
+    /// -# The Query throws  DbOperationError (rather than exiting)
+    /// -# The registered DbLostCallback was invoked
+    /// -# The registered DbRecoveredCallback was invoked after two reconnect
+    /// attempts (once failing and second triggered by timer)
+    void testDbLostAndRecoveredAfterTimeoutCallback();
+
+    /// @brief Verifies the lease manager's behavior if DB connection is lost
+    ///
+    /// This function creates a lease manager with a back end that supports
+    /// connectivity lost callback (currently only MySQL and PostgreSQL). It
+    /// verifies connectivity by issuing a known valid query. Next it simulates
+    /// connectivity lost by identifying and closing the socket connection to
+    /// the CB backend. It then reissues the query and verifies that:
+    /// -# The Query throws  DbOperationError (rather than exiting)
+    /// -# The registered DbLostCallback was invoked
+    /// -# The registered DbFailedCallback was invoked after two reconnect
+    /// attempts (once failing and second triggered by timer)
+    void testDbLostAndFailedAfterTimeoutCallback();
+
+    /// @brief Callback function registered with the lease manager
     bool db_lost_callback(db::ReconnectCtlPtr /* not_used */) {
-        return (callback_called_ = true);
+        return (++db_lost_callback_called_);
     }
 
     /// @brief Flag used to detect calls to db_lost_callback function
-    bool callback_called_;
+    uint32_t db_lost_callback_called_;
 
+    /// @brief Callback function registered with the lease manager
+    bool db_recovered_callback(db::ReconnectCtlPtr /* not_used */) {
+        return (++db_recovered_callback_called_);
+    }
+
+    /// @brief Flag used to detect calls to db_recovered_callback function
+    uint32_t db_recovered_callback_called_;
+
+    /// @brief Callback function registered with the lease manager
+    bool db_failed_callback(db::ReconnectCtlPtr /* not_used */) {
+        return (++db_failed_callback_called_);
+    }
+
+    /// @brief Flag used to detect calls to db_failed_callback function
+    uint32_t db_failed_callback_called_;
+
+    /// The IOService object, used for all ASIO operations.
+    isc::asiolink::IOServicePtr io_service_;
 };
 
 }  // namespace test
