@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,10 +13,9 @@
 #include <dhcpsrv/d2_client_mgr.h>
 #include <dhcpsrv/lease.h>
 
-#include <ctime>
-#include <boost/bind.hpp>
 #include <gtest/gtest.h>
-
+#include <ctime>
+#include <functional>
 #include <stdint.h>
 #include <string>
 
@@ -24,6 +23,7 @@ using namespace isc;
 using namespace isc::asiolink;
 using namespace isc::dhcp;
 using namespace isc::dhcp_ddns;
+namespace ph = std::placeholders;
 
 namespace {
 
@@ -41,11 +41,17 @@ public:
     /// @brief Pointer to the lease object used by the tests.
     LeasePtrType lease_;
 
+    /// @brief Pointer to the lease's subnet
+    SubnetPtr subnet_;
+
     /// @brief Constructor.
     NCRGeneratorTest()
         : d2_mgr_(CfgMgr::instance().getD2ClientMgr()), lease_() {
     }
 
+    /// @brief Destructor
+    virtual ~NCRGeneratorTest() = default;
+
     /// @brief Initializes the lease pointer used by the tests and starts D2.
     ///
     /// This method initializes the pointer to the lease which will be used
@@ -70,6 +76,8 @@ public:
         disableD2();
         // Base class TearDown.
         ::testing::Test::TearDown();
+
+        CfgMgr::instance().clear();
     }
 
     /// @brief Enables DHCP-DDNS updates.
@@ -81,8 +89,8 @@ public:
         D2ClientConfigPtr cfg(new D2ClientConfig());
         ASSERT_NO_THROW(cfg->enableUpdates(true));
         ASSERT_NO_THROW(CfgMgr::instance().setD2ClientConfig(cfg));
-        d2_mgr_.startSender(boost::bind(&NCRGeneratorTest::d2ErrorHandler, this,
-                                        _1, _2));
+        d2_mgr_.startSender(std::bind(&NCRGeneratorTest::d2ErrorHandler, this,
+                                      ph::_1, ph::_2));
     }
 
     /// @brief Disables DHCP-DDNS updates.
@@ -133,7 +141,8 @@ public:
                                  const std::string& dhcid,
                                  const uint64_t expires,
                                  const uint16_t len,
-                                 const std::string& fqdn="") {
+                                 const std::string& fqdn="",
+                                 const bool use_conflict_resolution = true) {
         NameChangeRequestPtr ncr;
         ASSERT_NO_THROW(ncr = CfgMgr::instance().getD2ClientMgr().peekAt(0));
         ASSERT_TRUE(ncr);
@@ -146,6 +155,7 @@ public:
         EXPECT_EQ(expires, ncr->getLeaseExpiresOn());
         EXPECT_EQ(len, ncr->getLeaseLength());
         EXPECT_EQ(isc::dhcp_ddns::ST_NEW, ncr->getStatus());
+        EXPECT_EQ(use_conflict_resolution, ncr->useConflictResolution());
 
         if (!fqdn.empty()) {
            EXPECT_EQ(fqdn, ncr->getFqdn());
@@ -213,16 +223,20 @@ public:
     /// @param rev Perform reverse update.
     /// @param fqdn Hostname.
     /// @param exp_dhcid Expected DHCID.
+    /// @param exp_use_cr expected value of conflict resolution flag
     void testNCR(const NameChangeType chg_type, const bool fwd, const bool rev,
-                 const std::string& fqdn, const std::string exp_dhcid) {
+                 const std::string& fqdn, const std::string exp_dhcid,
+                 const bool exp_use_cr = true) {
         // Queue NCR.
         ASSERT_NO_FATAL_FAILURE(sendNCR(chg_type, fwd, rev, fqdn));
         // Expecting one NCR be generated.
         ASSERT_EQ(1, d2_mgr_.getQueueSize());
+
+        uint32_t ttl = calculateDdnsTtl(lease_->valid_lft_);
+
         // Check the details of the NCR.
         verifyNameChangeRequest(chg_type, rev, fwd, lease_->addr_.toText(), exp_dhcid,
-                                lease_->cltt_ + lease_->valid_lft_,
-                                lease_->valid_lft_);
+                                lease_->cltt_ + ttl, ttl, fqdn, exp_use_cr);
     }
 
     /// @brief Test that calling queueNCR for NULL lease doesn't cause
@@ -253,8 +267,21 @@ public:
 
     /// @brief Implementation of the method creating DHCPv6 lease instance.
     virtual void initLease() {
+        Subnet6Ptr subnet(new Subnet6(IOAddress("2001:db8:1::"), 64, 100, 200, 300, 400));
+        // Normally, this would be set via defaults
+        subnet->setDdnsUseConflictResolution(true);
+
+        Pool6Ptr pool(new Pool6(Lease::TYPE_NA, IOAddress("2001:db8:1::1"),
+                                IOAddress("2001:db8:1::200")));
+        subnet->addPool(pool);
+        subnet_ = subnet;
+
+        CfgMgr& cfg_mgr = CfgMgr::instance();
+        cfg_mgr.getStagingCfg()->getCfgSubnets6()->add(subnet);
+        cfg_mgr.commit();
+
         lease_.reset(new Lease6(Lease::TYPE_NA, IOAddress("2001:db8:1::1"),
-                                duid_, 1234, 501, 502, 1, HWAddrPtr(), 0));
+                                duid_, 1234, 501, 502, subnet_->getID(), HWAddrPtr()));
     }
 };
 
@@ -415,6 +442,24 @@ TEST_F(NCRGenerator6Test, nullLease) {
     }
 }
 
+// Verify that conflict resolution is set correctly by v6 queueNCR()
+TEST_F(NCRGenerator6Test, useConflictResolution) {
+    {
+        SCOPED_TRACE("Subnet flag is false");
+        subnet_->setDdnsUseConflictResolution(false);
+        testNCR(CHG_REMOVE, true, true, "MYHOST.example.com.",
+                "000201BE0D7A66F8AB6C4082E7F8B81E2656667A102E3D0ECCEA5E0DD71730F392119A",
+                false);
+    }
+    {
+        SCOPED_TRACE("Subnet flag is true");
+        subnet_->setDdnsUseConflictResolution(true);
+        testNCR(CHG_REMOVE, true, true, "MYHOST.example.com.",
+                "000201BE0D7A66F8AB6C4082E7F8B81E2656667A102E3D0ECCEA5E0DD71730F392119A",
+                true);
+    }
+}
+
 /// @brief Test fixture class implementation for DHCPv4.
 class NCRGenerator4Test : public NCRGeneratorTest<Lease4Ptr> {
 public:
@@ -431,9 +476,24 @@ public:
 
     /// @brief Implementation of the method creating DHCPv4 lease instance.
     virtual void initLease() {
+
+        Subnet4Ptr subnet(new Subnet4(IOAddress("192.0.2.0"), 24, 1, 2, 3));
+        // Normally, this would be set via defaults
+        subnet->setDdnsUseConflictResolution(true);
+
+        Pool4Ptr pool(new Pool4(IOAddress("192.0.2.100"),
+                                IOAddress("192.0.2.200")));
+        subnet->addPool(pool);
+
+        CfgMgr& cfg_mgr = CfgMgr::instance();
+        cfg_mgr.getStagingCfg()->getCfgSubnets4()->add(subnet);
+        cfg_mgr.commit();
+
+        subnet_ = subnet;
         lease_.reset(new Lease4(IOAddress("192.0.2.1"), hwaddr_, ClientIdPtr(),
-                                100, time(NULL), 1));
+                                100, time(NULL), subnet_->getID()));
     }
+
 };
 
 // Test creation of the NameChangeRequest for both forward and reverse
@@ -566,11 +626,12 @@ TEST_F(NCRGenerator4Test, useClientId) {
     ASSERT_NO_FATAL_FAILURE(queueRemovalNCR(true, true, "myhost.example.com."));
     ASSERT_EQ(1, d2_mgr_.getQueueSize());
 
+    uint32_t ttl = calculateDdnsTtl(lease_->valid_lft_);
     verifyNameChangeRequest(isc::dhcp_ddns::CHG_REMOVE, true, true,
                             "192.0.2.1",
                             "000101C7AA5420483BDA99C437636EA7DA2FE18"
                             "31C9679FEB031C360CA571298F3D1FA",
-                            lease_->cltt_ + lease_->valid_lft_, 100);
+                            lease_->cltt_ + ttl, ttl);
     {
         SCOPED_TRACE("case CHG_REMOVE");
         testNCR(CHG_REMOVE, true, true, "myhost.example.com.",
@@ -599,4 +660,32 @@ TEST_F(NCRGenerator4Test, nullLease) {
     }
 }
 
+// Verify that conflict resolution is set correctly by v4 queueNCR()
+TEST_F(NCRGenerator4Test, useConflictResolution) {
+    {
+        SCOPED_TRACE("Subnet flag is false");
+        subnet_->setDdnsUseConflictResolution(false);
+        testNCR(CHG_REMOVE, true, true, "MYHOST.example.com.",
+                "000001E356D43E5F0A496D65BCA24D982D646140813E3"
+                "B03AB370BFF46BFA309AE7BFD", false);
+    }
+    {
+        SCOPED_TRACE("Subnet flag is true");
+        subnet_->setDdnsUseConflictResolution(true);
+        testNCR(CHG_REMOVE, true, true, "MYHOST.example.com.",
+                "000001E356D43E5F0A496D65BCA24D982D646140813E3"
+                "B03AB370BFF46BFA309AE7BFD", true);
+    }
+}
+
+// Verify that calculateDdnsTtl() produces the expected values.
+TEST_F(NCRGenerator4Test, calculateDdnsTtl) {
+
+    // A life time less than or equal to 1800 should yield a TTL of 600 seconds.
+    EXPECT_EQ(600, calculateDdnsTtl(100));
+
+    // A life time > 1800 should be 1/3 of the value.
+    EXPECT_EQ(601, calculateDdnsTtl(1803));
+}
+
 } // end of anonymous namespace
