@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,8 +12,10 @@
 #include <dhcp/option.h>
 #include <dhcp/option4_addrlst.h>
 #include <dhcp/option6_addrlst.h>
+#include <dhcp/tests/iface_mgr_test_config.h>
 #include <dhcpsrv/cfg_option.h>
 #include <dhcpsrv/parsers/shared_network_parser.h>
+#include <testutils/gtest_utils.h>
 #include <gtest/gtest.h>
 #include <string>
 
@@ -21,6 +23,7 @@ using namespace isc;
 using namespace isc::asiolink;
 using namespace isc::data;
 using namespace isc::dhcp;
+using namespace isc::dhcp::test;
 
 namespace {
 class SharedNetworkParserTest : public ::testing::Test {
@@ -125,7 +128,9 @@ public:
                 "    \"rebind-timer\": 199,"
                 "    \"relay\": { \"ip-addresses\": [ \"10.1.1.1\" ] },"
                 "    \"renew-timer\": 99,"
-                "    \"reservation-mode\": \"out-of-pool\","
+                "    \"reservations-global\": false,"
+                "    \"reservations-in-subnet\": true,"
+                "    \"reservations-out-of-pool\": true,"
                 "    \"server-hostname\": \"example.org\","
                 "    \"require-client-classes\": [ \"runner\" ],"
                 "    \"user-context\": { \"comment\": \"example\" },"
@@ -135,6 +140,18 @@ public:
                 "    \"calculate-tee-times\": true,"
                 "    \"t1-percent\": 0.345,"
                 "    \"t2-percent\": 0.721,"
+                "    \"ddns-send-updates\": true,"
+                "    \"ddns-override-no-update\": true,"
+                "    \"ddns-override-client-update\": true,"
+                "    \"ddns-replace-client-name\": \"always\","
+                "    \"ddns-generated-prefix\": \"prefix\","
+                "    \"ddns-qualifying-suffix\": \"example.com.\","
+                "    \"hostname-char-set\": \"[^A-Z]\","
+                "    \"hostname-char-replacement\": \"x\","
+                "    \"store-extended-info\": true,"
+                "    \"cache-threshold\": 0.123,"
+                "    \"cache-max-age\": 123,"
+                "    \"ddns-update-on-renew\": true,"
                 "    \"option-data\": ["
                 "        {"
                 "            \"name\": \"domain-name-servers\","
@@ -158,14 +175,18 @@ public:
                 "            \"boot-file-name\": \"\","
                 "            \"client-class\": \"\","
                 "            \"require-client-classes\": []\n,"
-                "            \"reservation-mode\": \"all\","
+                "            \"reservations-global\": false,"
+                "            \"reservations-in-subnet\": true,"
+                "            \"reservations-out-of-pool\": false,"
                 "            \"4o6-interface\": \"\","
                 "            \"4o6-interface-id\": \"\","
                 "            \"4o6-subnet\": \"\","
-                "            \"reservation-mode\": \"all\","
                 "            \"calculate-tee-times\": true,"
                 "            \"t1-percent\": .45,"
-                "            \"t2-percent\": .65"
+                "            \"t2-percent\": .65,"
+                "            \"hostname-char-set\": \"\","
+                "            \"cache-threshold\": .20,"
+                "            \"cache-max-age\": 50"
                 "        },"
                 "        {"
                 "            \"id\": 2,"
@@ -181,14 +202,17 @@ public:
                 "            \"boot-file-name\": \"\","
                 "            \"client-class\": \"\","
                 "            \"require-client-classes\": []\n,"
-                "            \"reservation-mode\": \"all\","
+                "            \"reservations-global\": false,"
+                "            \"reservations-in-subnet\": true,"
+                "            \"reservations-out-of-pool\": false,"
                 "            \"4o6-interface\": \"\","
                 "            \"4o6-interface-id\": \"\","
                 "            \"4o6-subnet\": \"\","
-                "            \"reservation-mode\": \"all\","
                 "            \"calculate-tee-times\": false,"
                 "            \"t1-percent\": .40,"
-                "            \"t2-percent\": .80"
+                "            \"t2-percent\": .80,"
+                "            \"cache-threshold\": .15,"
+                "            \"cache-max-age\": 5"
                 "        }"
                 "    ]"
                 "}";
@@ -210,6 +234,8 @@ private:
 // This test verifies that shared network parser for IPv4 works properly
 // in a positive test scenario.
 TEST_F(SharedNetwork4ParserTest, parse) {
+    IfaceMgrTestConfig ifmgr(true);
+
     // Basic configuration for shared network. A bunch of parameters
     // have to be specified for subnets because subnet parsers expect
     // that default and global values are set.
@@ -220,13 +246,7 @@ TEST_F(SharedNetwork4ParserTest, parse) {
     SharedNetwork4Parser parser;
     SharedNetwork4Ptr network;
 
-    try {
-        network = parser.parse(config_element);
-    } catch (const std::exception& ex) {
-        std::cout << "kabook: " << ex.what() << std::endl;
-    }
-
-    ASSERT_NO_THROW(network = parser.parse(config_element));
+    ASSERT_NO_THROW_LOG(network = parser.parse(config_element));
     ASSERT_TRUE(network);
 
     // Check basic parameters.
@@ -245,7 +265,21 @@ TEST_F(SharedNetwork4ParserTest, parse) {
     EXPECT_EQ("/dev/null", network->getFilename().get());
     EXPECT_EQ("10.0.0.1", network->getSiaddr().get().toText());
     EXPECT_EQ("example.org", network->getSname().get());
-    EXPECT_EQ(Network::HR_OUT_OF_POOL, network->getHostReservationMode());
+    EXPECT_FALSE(network->getReservationsGlobal());
+    EXPECT_TRUE(network->getReservationsInSubnet());
+    EXPECT_TRUE(network->getReservationsOutOfPool());
+    EXPECT_TRUE(network->getDdnsSendUpdates().get());
+    EXPECT_TRUE(network->getDdnsOverrideNoUpdate().get());
+    EXPECT_TRUE(network->getDdnsOverrideClientUpdate().get());
+    EXPECT_EQ(D2ClientConfig::RCM_ALWAYS, network->getDdnsReplaceClientNameMode().get());
+    EXPECT_EQ("prefix", network->getDdnsGeneratedPrefix().get());
+    EXPECT_EQ("example.com.", network->getDdnsQualifyingSuffix().get());
+    EXPECT_EQ("[^A-Z]", network->getHostnameCharSet().get());
+    EXPECT_EQ("x", network->getHostnameCharReplacement().get());
+    EXPECT_TRUE(network->getStoreExtendedInfo().get());
+    EXPECT_EQ(0.123, network->getCacheThreshold());
+    EXPECT_EQ(123, network->getCacheMaxAge());
+    EXPECT_TRUE(network->getDdnsUpdateOnRenew().get());
 
     // Relay information.
     auto relay_info = network->getRelayInfo();
@@ -263,20 +297,24 @@ TEST_F(SharedNetwork4ParserTest, parse) {
     EXPECT_TRUE(context->get("comment"));
 
     // Subnet with id 1
-    Subnet4Ptr subnet1 = network->getSubnet(SubnetID(1));
-    ASSERT_TRUE(subnet1);
-    EXPECT_EQ("10.1.2.0", subnet1->get().first.toText());
-    EXPECT_EQ(300, subnet1->getValid());
-    EXPECT_EQ(200, subnet1->getValid().getMin());
-    EXPECT_EQ(400, subnet1->getValid().getMax());
+    Subnet4Ptr subnet = network->getSubnet(SubnetID(1));
+    ASSERT_TRUE(subnet);
+    EXPECT_EQ("10.1.2.0", subnet->get().first.toText());
+    EXPECT_EQ(300, subnet->getValid());
+    EXPECT_EQ(200, subnet->getValid().getMin());
+    EXPECT_EQ(400, subnet->getValid().getMax());
+    EXPECT_FALSE(subnet->getHostnameCharSet().unspecified());
+    EXPECT_EQ("", subnet->getHostnameCharSet().get());
 
     // Subnet with id 2
-    Subnet4Ptr subnet2 = network->getSubnet(SubnetID(2));
-    ASSERT_TRUE(subnet2);
-    EXPECT_EQ("192.0.2.0", subnet2->get().first.toText());
-    EXPECT_EQ(30, subnet2->getValid());
-    EXPECT_EQ(30, subnet2->getValid().getMin());
-    EXPECT_EQ(30, subnet2->getValid().getMax());
+    subnet = network->getSubnet(SubnetID(2));
+    ASSERT_TRUE(subnet);
+    EXPECT_EQ("192.0.2.0", subnet->get().first.toText());
+    EXPECT_EQ(30, subnet->getValid());
+    EXPECT_EQ(30, subnet->getValid().getMin());
+    EXPECT_EQ(30, subnet->getValid().getMax());
+    EXPECT_EQ("[^A-Z]", subnet->getHostnameCharSet().get());
+    EXPECT_EQ("x", subnet->getHostnameCharReplacement().get());
 
     // DHCP options
     ConstCfgOptionPtr cfg_option = network->getCfgOption();
@@ -309,6 +347,8 @@ TEST_F(SharedNetwork4ParserTest, missingName) {
 // This test verifies that it's possible to specify client-class,
 // match-client-id, and authoritative on shared-network level.
 TEST_F(SharedNetwork4ParserTest, clientClassMatchClientIdAuthoritative) {
+    IfaceMgrTestConfig ifmgr(true);
+
     std::string config = getWorkingConfig();
     ElementPtr config_element = Element::fromJSON(config);
 
@@ -332,6 +372,7 @@ TEST_F(SharedNetwork4ParserTest, clientClassMatchClientIdAuthoritative) {
 // This test verifies that parsing of the "relay" element.
 // It checks both valid and invalid scenarios.
 TEST_F(SharedNetwork4ParserTest, relayInfoTests) {
+    IfaceMgrTestConfig ifmgr(true);
 
     // Create the vector of test scenarios.
     std::vector<RelayTest> tests = {
@@ -410,6 +451,83 @@ TEST_F(SharedNetwork4ParserTest, relayInfoTests) {
     }
 }
 
+// This test verifies that the optional interface check works as expected.
+TEST_F(SharedNetwork4ParserTest, iface) {
+    // Basic configuration for shared network.
+    std::string config = getWorkingConfig();
+    ElementPtr config_element = Element::fromJSON(config);
+
+    // Parse configuration specified above.
+
+    // The interface check can be disabled.
+    SharedNetwork4Parser parser_no_check(false);
+    SharedNetwork4Ptr network;
+    EXPECT_NO_THROW(network = parser_no_check.parse(config_element));
+    ASSERT_TRUE(network);
+    EXPECT_FALSE(network->getIface().unspecified());
+    EXPECT_EQ("eth1", network->getIface().get());
+
+    // Retry with the interface check enabled.
+    SharedNetwork4Parser parser;
+    EXPECT_THROW(parser.parse(config_element), DhcpConfigError);
+
+    // Configure default test interfaces.
+    IfaceMgrTestConfig ifmgr(true);
+
+    EXPECT_NO_THROW(network = parser_no_check.parse(config_element));
+    ASSERT_TRUE(network);
+    EXPECT_FALSE(network->getIface().unspecified());
+    EXPECT_EQ("eth1", network->getIface().get());
+
+    EXPECT_NO_THROW(network = parser.parse(config_element));
+    ASSERT_TRUE(network);
+    EXPECT_FALSE(network->getIface().unspecified());
+    EXPECT_EQ("eth1", network->getIface().get());
+}
+
+// This test verifies that shared network parser for IPv4 works properly
+// when using invalid renew and rebind timers.
+TEST_F(SharedNetwork4ParserTest, parseWithInvalidRenewRebind) {
+    IfaceMgrTestConfig ifmgr(true);
+
+    // Basic configuration for shared network.
+    std::string config = getWorkingConfig();
+    ElementPtr config_element = Element::fromJSON(config);
+    ConstElementPtr valid_element = config_element->get("rebind-timer");
+    int64_t value = valid_element->intValue();
+    valid_element = config_element->get("renew-timer");
+    ElementPtr mutable_element = boost::const_pointer_cast<Element>(valid_element);
+    mutable_element->setValue(value + 1);
+
+    // Parse configuration specified above.
+    SharedNetwork4Parser parser;
+    SharedNetwork4Ptr network;
+
+    ASSERT_THROW(network = parser.parse(config_element), DhcpConfigError);
+    ASSERT_FALSE(network);
+}
+
+// This test verifies that shared network parser for IPv4 works properly
+// when renew and rebind timers are equal.
+TEST_F(SharedNetwork4ParserTest, parseValidWithEqualRenewRebind) {
+    IfaceMgrTestConfig ifmgr(true);
+
+    // Basic configuration for shared network.
+    std::string config = getWorkingConfig();
+    ElementPtr config_element = Element::fromJSON(config);
+    ConstElementPtr valid_element = config_element->get("rebind-timer");
+    int64_t value = valid_element->intValue();
+    valid_element = config_element->get("renew-timer");
+    ElementPtr mutable_element = boost::const_pointer_cast<Element>(valid_element);
+    mutable_element->setValue(value);
+
+    // Parse configuration specified above.
+    SharedNetwork4Parser parser;
+    SharedNetwork4Ptr network;
+
+    ASSERT_NO_THROW(network = parser.parse(config_element));
+    ASSERT_TRUE(network);
+}
 
 /// @brief Test fixture class for SharedNetwork6Parser class.
 class SharedNetwork6ParserTest : public SharedNetworkParserTest {
@@ -437,7 +555,9 @@ public:
                 "    \"relay\": { \"ip-addresses\": [ \"2001:db8:1::1\" ] },"
                 "    \"renew-timer\": 99,"
                 "    \"require-client-classes\": [ \"runner\" ],"
-                "    \"reservation-mode\": \"out-of-pool\","
+                "    \"reservations-global\": false,"
+                "    \"reservations-in-subnet\": true,"
+                "    \"reservations-out-of-pool\": true,"
                 "    \"user-context\": { },"
                 "    \"valid-lifetime\": 399,"
                 "    \"min-valid-lifetime\": 299,"
@@ -445,6 +565,18 @@ public:
                 "    \"calculate-tee-times\": true,"
                 "    \"t1-percent\": 0.345,"
                 "    \"t2-percent\": 0.721,"
+                "    \"ddns-send-updates\": true,"
+                "    \"ddns-override-no-update\": true,"
+                "    \"ddns-override-client-update\": true,"
+                "    \"ddns-replace-client-name\": \"always\","
+                "    \"ddns-generated-prefix\": \"prefix\","
+                "    \"ddns-qualifying-suffix\": \"example.com.\","
+                "    \"hostname-char-set\": \"[^A-Z]\","
+                "    \"hostname-char-replacement\": \"x\","
+                "    \"store-extended-info\": true,"
+                "    \"cache-threshold\": 0.123,"
+                "    \"cache-max-age\": 123,"
+                "    \"ddns-update-on-renew\": true,"
                 "    \"option-data\": ["
                 "        {"
                 "            \"name\": \"dns-servers\","
@@ -467,8 +599,11 @@ public:
                 "            \"max-valid-lifetime\": 500,"
                 "            \"client-class\": \"\","
                 "            \"require-client-classes\": []\n,"
-                "            \"reservation-mode\": \"all\","
-                "            \"rapid-commit\": false"
+                "            \"reservations-global\": false,"
+                "            \"reservations-in-subnet\": true,"
+                "            \"reservations-out-of-pool\": false,"
+                "            \"rapid-commit\": false,"
+                "            \"hostname-char-set\": \"\""
                 "        },"
                 "        {"
                 "            \"id\": 2,"
@@ -481,7 +616,9 @@ public:
                 "            \"valid-lifetime\": 40,"
                 "            \"client-class\": \"\","
                 "            \"require-client-classes\": []\n,"
-                "            \"reservation-mode\": \"all\","
+                "            \"reservations-global\": false,"
+                "            \"reservations-in-subnet\": true,"
+                "            \"reservations-out-of-pool\": false,"
                 "            \"rapid-commit\": false"
                 "        }"
                 "    ]"
@@ -506,9 +643,11 @@ public:
     bool use_iface_id_;
 };
 
-// This test verifies that shared network parser for IPv4 works properly
+// This test verifies that shared network parser for IPv6 works properly
 // in a positive test scenario.
 TEST_F(SharedNetwork6ParserTest, parse) {
+    IfaceMgrTestConfig ifmgr(true);
+
     // Basic configuration for shared network. A bunch of parameters
     // have to be specified for subnets because subnet parsers expect
     // that default and global values are set.
@@ -537,6 +676,18 @@ TEST_F(SharedNetwork6ParserTest, parse) {
     EXPECT_TRUE(network->getCalculateTeeTimes());
     EXPECT_EQ(0.345, network->getT1Percent());
     EXPECT_EQ(0.721, network->getT2Percent());
+    EXPECT_TRUE(network->getDdnsSendUpdates().get());
+    EXPECT_TRUE(network->getDdnsOverrideNoUpdate().get());
+    EXPECT_TRUE(network->getDdnsOverrideClientUpdate().get());
+    EXPECT_EQ(D2ClientConfig::RCM_ALWAYS, network->getDdnsReplaceClientNameMode().get());
+    EXPECT_EQ("prefix", network->getDdnsGeneratedPrefix().get());
+    EXPECT_EQ("example.com.", network->getDdnsQualifyingSuffix().get());
+    EXPECT_EQ("[^A-Z]", network->getHostnameCharSet().get());
+    EXPECT_EQ("x", network->getHostnameCharReplacement().get());
+    EXPECT_TRUE(network->getStoreExtendedInfo().get());
+    EXPECT_EQ(0.123, network->getCacheThreshold());
+    EXPECT_EQ(123, network->getCacheMaxAge());
+    EXPECT_TRUE(network->getDdnsUpdateOnRenew().get());
 
     // Relay information.
     auto relay_info = network->getRelayInfo();
@@ -554,26 +705,30 @@ TEST_F(SharedNetwork6ParserTest, parse) {
     EXPECT_EQ(0, context->size());
 
     // Subnet with id 1
-    Subnet6Ptr subnet1 = network->getSubnet(SubnetID(1));
-    ASSERT_TRUE(subnet1);
-    EXPECT_EQ("3000::", subnet1->get().first.toText());
-    EXPECT_EQ(300, subnet1->getPreferred());
-    EXPECT_EQ(200, subnet1->getPreferred().getMin());
-    EXPECT_EQ(400, subnet1->getPreferred().getMax());
-    EXPECT_EQ(400, subnet1->getValid());
-    EXPECT_EQ(300, subnet1->getValid().getMin());
-    EXPECT_EQ(500, subnet1->getValid().getMax());
+    Subnet6Ptr subnet = network->getSubnet(SubnetID(1));
+    ASSERT_TRUE(subnet);
+    EXPECT_EQ("3000::", subnet->get().first.toText());
+    EXPECT_EQ(300, subnet->getPreferred());
+    EXPECT_EQ(200, subnet->getPreferred().getMin());
+    EXPECT_EQ(400, subnet->getPreferred().getMax());
+    EXPECT_EQ(400, subnet->getValid());
+    EXPECT_EQ(300, subnet->getValid().getMin());
+    EXPECT_EQ(500, subnet->getValid().getMax());
+    EXPECT_FALSE(subnet->getHostnameCharSet().unspecified());
+    EXPECT_EQ("", subnet->getHostnameCharSet().get());
 
     // Subnet with id 2
-    Subnet6Ptr subnet2 = network->getSubnet(SubnetID(2));
-    ASSERT_TRUE(subnet2);
-    EXPECT_EQ("2001:db8:1::", subnet2->get().first.toText());
-    EXPECT_EQ(30, subnet2->getPreferred());
-    EXPECT_EQ(30, subnet2->getPreferred().getMin());
-    EXPECT_EQ(30, subnet2->getPreferred().getMax());
-    EXPECT_EQ(40, subnet2->getValid());
-    EXPECT_EQ(40, subnet2->getValid().getMin());
-    EXPECT_EQ(40, subnet2->getValid().getMax());
+    subnet = network->getSubnet(SubnetID(2));
+    ASSERT_TRUE(subnet);
+    EXPECT_EQ("2001:db8:1::", subnet->get().first.toText());
+    EXPECT_EQ(30, subnet->getPreferred());
+    EXPECT_EQ(30, subnet->getPreferred().getMin());
+    EXPECT_EQ(30, subnet->getPreferred().getMax());
+    EXPECT_EQ(40, subnet->getValid());
+    EXPECT_EQ(40, subnet->getValid().getMin());
+    EXPECT_EQ(40, subnet->getValid().getMax());
+    EXPECT_EQ("[^A-Z]", subnet->getHostnameCharSet().get());
+    EXPECT_EQ("x", subnet->getHostnameCharReplacement().get());
 
     // DHCP options
     ConstCfgOptionPtr cfg_option = network->getCfgOption();
@@ -589,9 +744,11 @@ TEST_F(SharedNetwork6ParserTest, parse) {
     EXPECT_EQ("2001:db8:1::cafe", addresses[0].toText());
 }
 
-// This test verifies that shared network parser for IPv4 works properly
+// This test verifies that shared network parser for IPv6 works properly
 // in a positive test scenario.
 TEST_F(SharedNetwork6ParserTest, parseWithInterfaceId) {
+    IfaceMgrTestConfig ifmgr(true);
+
     // Use the configuration with interface-id instead of interface parameter.
     use_iface_id_ = true;
     std::string config = getWorkingConfig();
@@ -608,9 +765,57 @@ TEST_F(SharedNetwork6ParserTest, parseWithInterfaceId) {
     ASSERT_TRUE(opt_iface_id);
 }
 
+// This test verifies that shared network parser for IPv6 works properly
+// when using invalid renew and rebind timers.
+TEST_F(SharedNetwork6ParserTest, parseWithInvalidRenewRebind) {
+    IfaceMgrTestConfig ifmgr(true);
+
+    // Use the configuration with interface-id instead of interface parameter.
+    use_iface_id_ = true;
+    std::string config = getWorkingConfig();
+    ElementPtr config_element = Element::fromJSON(config);
+    ConstElementPtr valid_element = config_element->get("rebind-timer");
+    int64_t value = valid_element->intValue();
+    valid_element = config_element->get("renew-timer");
+    ElementPtr mutable_element = boost::const_pointer_cast<Element>(valid_element);
+    mutable_element->setValue(value + 1);
+
+    // Parse configuration specified above.
+    SharedNetwork6Parser parser;
+    SharedNetwork6Ptr network;
+
+    ASSERT_THROW(network = parser.parse(config_element), DhcpConfigError);
+    ASSERT_FALSE(network);
+}
+
+// This test verifies that shared network parser for IPv6 works properly
+// when renew and rebind timers are equal.
+TEST_F(SharedNetwork6ParserTest, parseValidWithEqualRenewRebind) {
+    IfaceMgrTestConfig ifmgr(true);
+
+    // Use the configuration with interface-id instead of interface parameter.
+    use_iface_id_ = true;
+    std::string config = getWorkingConfig();
+    ElementPtr config_element = Element::fromJSON(config);
+    ConstElementPtr valid_element = config_element->get("rebind-timer");
+    int64_t value = valid_element->intValue();
+    valid_element = config_element->get("renew-timer");
+    ElementPtr mutable_element = boost::const_pointer_cast<Element>(valid_element);
+    mutable_element->setValue(value);
+
+    // Parse configuration specified above.
+    SharedNetwork6Parser parser;
+    SharedNetwork6Ptr network;
+
+    ASSERT_NO_THROW(network = parser.parse(config_element));
+    ASSERT_TRUE(network);
+}
+
 // This test verifies that error is returned when trying to configure a
 // shared network with both interface and interface id.
 TEST_F(SharedNetwork6ParserTest, mutuallyExclusiveInterfaceId) {
+    IfaceMgrTestConfig ifmgr(true);
+
     // Use the configuration with interface-id instead of interface parameter.
     use_iface_id_ = true;
     std::string config = getWorkingConfig();
@@ -627,6 +832,8 @@ TEST_F(SharedNetwork6ParserTest, mutuallyExclusiveInterfaceId) {
 // This test verifies that it's possible to specify client-class
 // on shared-network level.
 TEST_F(SharedNetwork6ParserTest, clientClass) {
+    IfaceMgrTestConfig ifmgr(true);
+
     std::string config = getWorkingConfig();
     ElementPtr config_element = Element::fromJSON(config);
 
@@ -644,6 +851,8 @@ TEST_F(SharedNetwork6ParserTest, clientClass) {
 // This test verifies that it's possible to specify require-client-classes
 // on shared-network level.
 TEST_F(SharedNetwork6ParserTest, evalClientClasses) {
+    IfaceMgrTestConfig ifmgr(true);
+
     std::string config = getWorkingConfig();
     ElementPtr config_element = Element::fromJSON(config);
 
@@ -666,6 +875,8 @@ TEST_F(SharedNetwork6ParserTest, evalClientClasses) {
 // This test verifies that bad require-client-classes configs raise
 // expected errors.
 TEST_F(SharedNetwork6ParserTest, badEvalClientClasses) {
+    IfaceMgrTestConfig ifmgr(true);
+
     std::string config = getWorkingConfig();
     ElementPtr config_element = Element::fromJSON(config);
 
@@ -695,6 +906,8 @@ TEST_F(SharedNetwork6ParserTest, badEvalClientClasses) {
 // This test verifies that v6 parsing of the "relay" element.
 // It checks both valid and invalid scenarios.
 TEST_F(SharedNetwork6ParserTest, relayInfoTests) {
+    IfaceMgrTestConfig ifmgr(true);
+
 
     // Create the vector of test scenarios.
     std::vector<RelayTest> tests = {
@@ -773,4 +986,38 @@ TEST_F(SharedNetwork6ParserTest, relayInfoTests) {
     }
 }
 
+// This test verifies that the optional interface check works as expected.
+TEST_F(SharedNetwork6ParserTest, iface) {
+    // Basic configuration for shared network.
+    std::string config = getWorkingConfig();
+    ElementPtr config_element = Element::fromJSON(config);
+
+    // Parse configuration specified above.
+
+    // The interface check can be disabled.
+    SharedNetwork6Parser parser_no_check(false);
+    SharedNetwork6Ptr network;
+    EXPECT_NO_THROW(network = parser_no_check.parse(config_element));
+    ASSERT_TRUE(network);
+    EXPECT_FALSE(network->getIface().unspecified());
+    EXPECT_EQ("eth1", network->getIface().get());
+
+    // Retry with the interface check enabled.
+    SharedNetwork6Parser parser;
+    EXPECT_THROW(parser.parse(config_element), DhcpConfigError);
+
+    // Configure default test interfaces.
+    IfaceMgrTestConfig ifmgr(true);
+
+    EXPECT_NO_THROW(network = parser_no_check.parse(config_element));
+    ASSERT_TRUE(network);
+    EXPECT_FALSE(network->getIface().unspecified());
+    EXPECT_EQ("eth1", network->getIface().get());
+
+    EXPECT_NO_THROW(network = parser.parse(config_element));
+    ASSERT_TRUE(network);
+    EXPECT_FALSE(network->getIface().unspecified());
+    EXPECT_EQ("eth1", network->getIface().get());
+}
+
 } // end of anonymous namespace
