@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,11 +13,11 @@
 #include <dhcp/option.h>
 #include <dhcp/option_custom.h>
 #include <dhcp/option_definition.h>
-#include <dhcp/dhcp6.h>
 #include <dhcp/option_space.h>
 #include <dhcpsrv/shared_network.h>
 #include <dhcpsrv/subnet.h>
 #include <exceptions/exceptions.h>
+#include <testutils/multi_threading_utils.h>
 
 #include <boost/pointer_cast.hpp>
 #include <boost/scoped_ptr.hpp>
@@ -27,9 +27,11 @@
 // don't import the entire boost namespace.  It will unexpectedly hide uint8_t
 // for some systems.
 using boost::scoped_ptr;
+
 using namespace isc;
 using namespace isc::dhcp;
 using namespace isc::asiolink;
+using namespace isc::test;
 
 namespace {
 
@@ -80,8 +82,14 @@ TEST(Subnet4Test, defaults) {
     EXPECT_TRUE(subnet.getT2().unspecified());
     EXPECT_EQ(0, subnet.getT2().get());
 
-    EXPECT_TRUE(subnet.getHostReservationMode().unspecified());
-    EXPECT_EQ(Network::HR_ALL, subnet.getHostReservationMode().get());
+    EXPECT_TRUE(subnet.getReservationsGlobal().unspecified());
+    EXPECT_FALSE(subnet.getReservationsGlobal().get());
+
+    EXPECT_TRUE(subnet.getReservationsInSubnet().unspecified());
+    EXPECT_TRUE(subnet.getReservationsInSubnet().get());
+
+    EXPECT_TRUE(subnet.getReservationsOutOfPool().unspecified());
+    EXPECT_FALSE(subnet.getReservationsOutOfPool().get());
 
     EXPECT_TRUE(subnet.getCalculateTeeTimes().unspecified());
     EXPECT_FALSE(subnet.getCalculateTeeTimes().get());
@@ -115,6 +123,33 @@ TEST(Subnet4Test, defaults) {
     EXPECT_TRUE(subnet.get4o6().getSubnet4o6().unspecified());
     EXPECT_TRUE(subnet.get4o6().getSubnet4o6().get().first.isV6Zero());
     EXPECT_EQ(128, subnet.get4o6().getSubnet4o6().get().second);
+
+    EXPECT_TRUE(subnet.getDdnsSendUpdates().unspecified());
+    EXPECT_FALSE(subnet.getDdnsSendUpdates().get());
+
+    EXPECT_TRUE(subnet.getDdnsOverrideNoUpdate().unspecified());
+    EXPECT_FALSE(subnet.getDdnsOverrideNoUpdate().get());
+
+    EXPECT_TRUE(subnet.getDdnsOverrideClientUpdate().unspecified());
+    EXPECT_FALSE(subnet.getDdnsOverrideClientUpdate().get());
+
+    EXPECT_TRUE(subnet.getDdnsReplaceClientNameMode().unspecified());
+    EXPECT_EQ(D2ClientConfig::RCM_NEVER, subnet.getDdnsReplaceClientNameMode().get());
+
+    EXPECT_TRUE(subnet.getDdnsGeneratedPrefix().unspecified());
+    EXPECT_TRUE(subnet.getDdnsGeneratedPrefix().empty());
+
+    EXPECT_TRUE(subnet.getDdnsQualifyingSuffix().unspecified());
+    EXPECT_TRUE(subnet.getDdnsQualifyingSuffix().empty());
+
+    EXPECT_TRUE(subnet.getHostnameCharSet().unspecified());
+    EXPECT_TRUE(subnet.getHostnameCharSet().empty());
+
+    EXPECT_TRUE(subnet.getHostnameCharReplacement().unspecified());
+    EXPECT_TRUE(subnet.getHostnameCharReplacement().empty());
+
+    EXPECT_TRUE(subnet.getDdnsUpdateOnRenew().unspecified());
+    EXPECT_FALSE(subnet.getDdnsUpdateOnRenew().get());
 }
 
 // Checks that the subnet id can be either autogenerated or set to an
@@ -659,7 +694,6 @@ TEST(Subnet4Test, get) {
     EXPECT_EQ(28, subnet->get().second);
 }
 
-
 // Checks if last allocated address/prefix is stored/retrieved properly
 TEST(Subnet4Test, lastAllocated) {
     IOAddress addr("192.0.2.17");
@@ -681,6 +715,28 @@ TEST(Subnet4Test, lastAllocated) {
     EXPECT_THROW(subnet->setLastAllocated(Lease::TYPE_PD, addr), BadValue);
 }
 
+// Checks if last allocated address/prefix is stored/retrieved properly
+TEST(Subnet4Test, lastAllocatedMultiThreading) {
+    MultiThreadingTest mt(true);
+    IOAddress addr("192.0.2.17");
+
+    IOAddress last("192.0.2.255");
+
+    Subnet4Ptr subnet(new Subnet4(IOAddress("192.0.2.0"), 24, 1, 2, 3));
+
+    // Check initial conditions (all should be set to the last address in range)
+    EXPECT_EQ(last.toText(), subnet->getLastAllocated(Lease::TYPE_V4).toText());
+
+    // Now set last allocated for IA
+    EXPECT_NO_THROW(subnet->setLastAllocated(Lease::TYPE_V4, addr));
+    EXPECT_EQ(addr.toText(), subnet->getLastAllocated(Lease::TYPE_V4).toText());
+
+    // No, you can't set the last allocated IPv6 address in IPv4 subnet
+    EXPECT_THROW(subnet->setLastAllocated(Lease::TYPE_TA, addr), BadValue);
+    EXPECT_THROW(subnet->setLastAllocated(Lease::TYPE_TA, addr), BadValue);
+    EXPECT_THROW(subnet->setLastAllocated(Lease::TYPE_PD, addr), BadValue);
+}
+
 // Checks if the V4 is the only allowed type for Pool4 and if getPool()
 // is working properly.
 TEST(Subnet4Test, PoolType) {
@@ -736,7 +792,7 @@ TEST(Subnet4Test, getServerId) {
 
     // Add server identifier.
     OptionDefinitionPtr option_def = LibDHCP::getOptionDef(DHCP4_OPTION_SPACE,
-                                                         DHO_DHCP_SERVER_IDENTIFIER);
+                                                           DHO_DHCP_SERVER_IDENTIFIER);
     OptionCustomPtr option_server_id(new OptionCustom(*option_def, Option::V4));
     option_server_id->writeAddress(IOAddress("1.2.3.4"));
 
@@ -802,8 +858,14 @@ TEST(SharedNetwork6Test, defaults) {
     EXPECT_TRUE(subnet.getT2().unspecified());
     EXPECT_EQ(0, subnet.getT2().get());
 
-    EXPECT_TRUE(subnet.getHostReservationMode().unspecified());
-    EXPECT_EQ(Network::HR_ALL, subnet.getHostReservationMode().get());
+    EXPECT_TRUE(subnet.getReservationsGlobal().unspecified());
+    EXPECT_FALSE(subnet.getReservationsGlobal().get());
+
+    EXPECT_TRUE(subnet.getReservationsInSubnet().unspecified());
+    EXPECT_TRUE(subnet.getReservationsInSubnet().get());
+
+    EXPECT_TRUE(subnet.getReservationsOutOfPool().unspecified());
+    EXPECT_FALSE(subnet.getReservationsOutOfPool().get());
 
     EXPECT_TRUE(subnet.getCalculateTeeTimes().unspecified());
     EXPECT_FALSE(subnet.getCalculateTeeTimes().get());
@@ -819,6 +881,33 @@ TEST(SharedNetwork6Test, defaults) {
 
     EXPECT_TRUE(subnet.getRapidCommit().unspecified());
     EXPECT_FALSE(subnet.getRapidCommit().get());
+
+    EXPECT_TRUE(subnet.getDdnsSendUpdates().unspecified());
+    EXPECT_FALSE(subnet.getDdnsSendUpdates().get());
+
+    EXPECT_TRUE(subnet.getDdnsOverrideNoUpdate().unspecified());
+    EXPECT_FALSE(subnet.getDdnsOverrideNoUpdate().get());
+
+    EXPECT_TRUE(subnet.getDdnsOverrideClientUpdate().unspecified());
+    EXPECT_FALSE(subnet.getDdnsOverrideClientUpdate().get());
+
+    EXPECT_TRUE(subnet.getDdnsReplaceClientNameMode().unspecified());
+    EXPECT_EQ(D2ClientConfig::RCM_NEVER, subnet.getDdnsReplaceClientNameMode().get());
+
+    EXPECT_TRUE(subnet.getDdnsGeneratedPrefix().unspecified());
+    EXPECT_TRUE(subnet.getDdnsGeneratedPrefix().empty());
+
+    EXPECT_TRUE(subnet.getDdnsQualifyingSuffix().unspecified());
+    EXPECT_TRUE(subnet.getDdnsQualifyingSuffix().empty());
+
+    EXPECT_TRUE(subnet.getHostnameCharSet().unspecified());
+    EXPECT_TRUE(subnet.getHostnameCharSet().empty());
+
+    EXPECT_TRUE(subnet.getHostnameCharReplacement().unspecified());
+    EXPECT_TRUE(subnet.getHostnameCharReplacement().empty());
+
+    EXPECT_TRUE(subnet.getDdnsUpdateOnRenew().unspecified());
+    EXPECT_FALSE(subnet.getDdnsUpdateOnRenew().get());
 }
 
 // Checks that the subnet id can be either autogenerated or set to an
@@ -1414,7 +1503,6 @@ TEST(Subnet6Test, getOptions) {
     }
 }
 
-
 TEST(Subnet6Test, addVendorOption) {
 
     // Create as subnet to add options to it.
@@ -1466,8 +1554,6 @@ TEST(Subnet6Test, addVendorOption) {
     EXPECT_TRUE(options->empty());
 }
 
-
-
 // This test verifies that inRange() and inPool() methods work properly.
 TEST(Subnet6Test, inRangeinPool) {
     Subnet6Ptr subnet(new Subnet6(IOAddress("2001:db8::"), 32, 1, 2, 3, 4));
@@ -1684,6 +1770,42 @@ TEST(Subnet6Test, lastAllocated) {
     EXPECT_THROW(subnet->setLastAllocated(Lease::TYPE_V4, ia), BadValue);
 }
 
+// Checks if last allocated address/prefix is stored/retrieved properly
+TEST(Subnet6Test, lastAllocatedMultiThreading) {
+    MultiThreadingTest mt(true);
+    IOAddress ia("2001:db8:1::1");
+    IOAddress ta("2001:db8:1::abcd");
+    IOAddress pd("2001:db8:1::1234:5678");
+
+    IOAddress last("2001:db8:1::ffff:ffff:ffff:ffff");
+
+    Subnet6Ptr subnet(new Subnet6(IOAddress("2001:db8:1::"), 64, 1, 2, 3, 4));
+
+    // Check initial conditions (all should be set to the last address in range)
+    EXPECT_EQ(last.toText(), subnet->getLastAllocated(Lease::TYPE_NA).toText());
+    EXPECT_EQ(last.toText(), subnet->getLastAllocated(Lease::TYPE_TA).toText());
+    EXPECT_EQ(last.toText(), subnet->getLastAllocated(Lease::TYPE_PD).toText());
+
+    // Now set last allocated for IA
+    EXPECT_NO_THROW(subnet->setLastAllocated(Lease::TYPE_NA, ia));
+    EXPECT_EQ(ia.toText(), subnet->getLastAllocated(Lease::TYPE_NA).toText());
+
+    // TA and PD should be unchanged
+    EXPECT_EQ(last.toText(), subnet->getLastAllocated(Lease::TYPE_TA).toText());
+    EXPECT_EQ(last.toText(), subnet->getLastAllocated(Lease::TYPE_PD).toText());
+
+    // Now set TA and PD
+    EXPECT_NO_THROW(subnet->setLastAllocated(Lease::TYPE_TA, ta));
+    EXPECT_NO_THROW(subnet->setLastAllocated(Lease::TYPE_PD, pd));
+
+    EXPECT_EQ(ia.toText(), subnet->getLastAllocated(Lease::TYPE_NA).toText());
+    EXPECT_EQ(ta.toText(), subnet->getLastAllocated(Lease::TYPE_TA).toText());
+    EXPECT_EQ(pd.toText(), subnet->getLastAllocated(Lease::TYPE_PD).toText());
+
+    // No, you can't set the last allocated IPv4 address in IPv6 subnet
+    EXPECT_THROW(subnet->setLastAllocated(Lease::TYPE_V4, ia), BadValue);
+}
+
 // This test verifies that the IPv4 subnet can be fetched by id.
 TEST(SubnetFetcherTest, getSubnet4ById) {
     Subnet4Collection collection;
@@ -1694,10 +1816,10 @@ TEST(SubnetFetcherTest, getSubnet4ById) {
     EXPECT_FALSE(subnet);
 
     subnet.reset(new Subnet4(IOAddress("192.0.2.0"), 24, 1, 2, 3, 1024));
-    EXPECT_NO_THROW(collection.push_back(subnet));
+    EXPECT_NO_THROW(collection.insert(subnet));
 
     subnet.reset(new Subnet4(IOAddress("192.0.3.0"), 24, 1, 2, 3, 2048));
-    EXPECT_NO_THROW(collection.push_back(subnet));
+    EXPECT_NO_THROW(collection.insert(subnet));
 
     subnet = SubnetFetcher4::get(collection, SubnetID(1024));
     ASSERT_TRUE(subnet);
@@ -1720,10 +1842,10 @@ TEST(SubnetFetcherTest, getSubnet6ById) {
     EXPECT_FALSE(subnet);
 
     subnet.reset(new Subnet6(IOAddress("2001:db8:1::"), 64, 1, 2, 3, 4, 1024));
-    EXPECT_NO_THROW(collection.push_back(subnet));
+    EXPECT_NO_THROW(collection.insert(subnet));
 
     subnet.reset(new Subnet6(IOAddress("2001:db8:2::"), 64, 1, 2, 3, 4, 2048));
-    EXPECT_NO_THROW(collection.push_back(subnet));
+    EXPECT_NO_THROW(collection.insert(subnet));
 
     subnet = SubnetFetcher6::get(collection, SubnetID(1024));
     ASSERT_TRUE(subnet);
