@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -207,6 +207,131 @@ protected:
     std::string value_;
 };
 
+/// @brief Token representing an IP address as a string
+///
+/// This token holds the value of an IP address as a string, for instance
+/// 10.0.0.1 is '10.0.0.1'
+class TokenIpAddressToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenIpAddressToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented IP address as a string will be pushed here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token representing an 8 bit integer as a string
+///
+/// This token holds the value of an 8 bit integer as a string, for instance
+/// 0xff is '-1'
+class TokenInt8ToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenInt8ToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented 8 bit integer as a string will be pushed
+    /// here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token representing a 16 bit integer as a string
+///
+/// This token holds the value of a 16 bit integer as a string, for instance
+/// 0xffff is '-1'
+class TokenInt16ToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenInt16ToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented 16 bit integer as a string will be pushed
+    /// here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token representing a 32 bit integer as a string
+///
+/// This token holds the value of a 32 bit integer as a string, for instance
+/// 0xffffffff is '-1'
+class TokenInt32ToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenInt32ToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented 32 bit integer as a string will be pushed
+    /// here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token representing an 8 bit unsigned integer as a string
+///
+/// This token holds the value of an 8 bit unsigned integer as a string, for
+/// instance 0xff is '255'
+class TokenUInt8ToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenUInt8ToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented 8 bit unsigned integer as a string will be
+    /// pushed here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token representing a 16 bit unsigned integer as a string
+///
+/// This token holds the value of a 16 bit unsigned integer as a string, for
+/// instance 0xffff is '65535'
+class TokenUInt16ToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenUInt16ToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented 16 bit unsigned integer as a string will be
+    /// pushed here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token representing a 32 bit unsigned integer as a string
+///
+/// This token holds the value of a 32 bit unsigned integer as a string, for
+/// instance 0xffffffff is '4294967295'
+class TokenUInt32ToText : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenUInt32ToText() {}
+
+    /// @brief Token evaluation (puts value of the string on the stack after
+    /// decoding)
+    ///
+    /// @param pkt (ignored)
+    /// @param values (represented 32 bit unsigned integer as a string will be
+    /// pushed here)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
 /// @brief Token that represents a value of an option
 ///
 /// This represents a reference to a given option, e.g. in the expression
@@ -233,7 +358,6 @@ public:
     };
 
     /// @brief Constructor that takes an option code as a parameter
-    /// @param option_code code of the option
     ///
     /// Note: There is no constructor that takes option_name, as it would
     /// introduce complex dependency of the libkea-eval on libdhcpsrv.
@@ -338,16 +462,17 @@ protected:
 /// exist or the option is not found an empty string ("") is returned
 /// (or "false" when the representation is EXISTS).
 ///
-/// The nesting level can go from 0 (closest to the server) to 31
+/// The nesting level can go from 0 (closest to the server) to 31,
+/// or from -1 (closest to the client) to -32
 class TokenRelay6Option : public TokenOption {
 public:
     /// @brief Constructor that takes a nesting level and an option
-    /// code as paramaters.
+    /// code as parameters.
     ///
     /// @param nest_level the nesting for which relay to examine.
     /// @param option_code code of the option.
     /// @param rep_type Token representation type.
-    TokenRelay6Option(const uint8_t nest_level, const uint16_t option_code,
+    TokenRelay6Option(const int8_t nest_level, const uint16_t option_code,
                       const RepresentationType& rep_type)
         :TokenOption(option_code, rep_type), nest_level_(nest_level) {}
 
@@ -358,7 +483,7 @@ public:
     ///
     /// @return nest-level of the relay block this token expects to use
     /// for extraction.
-    uint8_t getNest() const {
+    int8_t getNest() const {
         return (nest_level_);
     }
 
@@ -368,7 +493,7 @@ protected:
     /// @return option instance if available
     virtual OptionPtr getOption(Pkt& pkt);
 
-    uint8_t nest_level_; ///< nesting level of the relay block to use
+    int8_t nest_level_; ///< nesting level of the relay block to use
 };
 
 /// @brief Token that represents meta data of a DHCP packet.
@@ -398,10 +523,10 @@ public:
 
     /// @brief Gets a value from the specified packet.
     ///
-    /// Evaluation uses metadatas available in the packet. It does not
+    /// Evaluation uses metadata available in the packet. It does not
     /// require any values to be present on the stack.
     ///
-    /// @param pkt - metadatas will be extracted from here
+    /// @param pkt - metadata will be extracted from here
     /// @param values - stack of values (1 result will be pushed)
     void evaluate(Pkt& pkt, ValueStack& values);
 
@@ -499,7 +624,7 @@ public:
 
     /// @brief Gets a value of the specified packet.
     ///
-    /// The evaluation uses fields that are availabe in the packet.  It does not
+    /// The evaluation uses fields that are available in the packet.  It does not
     /// require any values to be present on the stack.
     ///
     /// @throw EvalTypeError when called for a DHCPv4 packet
@@ -534,7 +659,8 @@ private:
 /// is always returned as a 16 byte IPv6 address.  As the relay may not have
 /// set the field it may be 0s.
 ///
-/// The nesting level can go from 0 (closest to the server) to 31.
+/// The nesting level can go from 0 (closest to the server) to 31,
+/// or from -1 (closest to the client) to -32
 class TokenRelay6Field : public Token {
 public:
 
@@ -549,7 +675,7 @@ public:
     ///
     /// @param nest_level the nesting level for which relay to examine.
     /// @param type which field to extract.
-    TokenRelay6Field(const uint8_t nest_level, const FieldType type)
+    TokenRelay6Field(const int8_t nest_level, const FieldType type)
       : nest_level_(nest_level), type_(type) {}
 
     /// @brief Extracts the specified field from the requested relay
@@ -568,7 +694,7 @@ public:
     ///
     /// @return nest-level of the relay block this token expects to use
     /// for extraction.
-    uint8_t getNest() const {
+    int8_t getNest() const {
         return (nest_level_);
     }
 
@@ -584,7 +710,7 @@ public:
 
 protected:
     /// @brief Specifies field of the DHCPv6 relay option to get
-    uint8_t nest_level_; ///< nesting level of the relay block to use
+    int8_t nest_level_; ///< nesting level of the relay block to use
     FieldType type_; ///< field to get
 };
 
@@ -635,7 +761,7 @@ public:
     /// str is the string to extract a substring from.  If it is empty, an empty
     /// string is pushed onto the value stack.
     ///
-    /// start is the postion from which the code starts extracting the substring.
+    /// start is the position from which the code starts extracting the substring.
     /// 0 is the first character and a negative number starts from the end, with
     /// -1 being the last character.  If the starting point is outside of the
     /// original string an empty string is pushed onto the value stack.
@@ -673,6 +799,7 @@ public:
 ///
 /// For example in the sub-expression "concat('foo','bar')" the result
 /// of the evaluation is "foobar"
+/// For user convenience the "'foo' + 'bar'" alternative does the same.
 class TokenConcat : public Token {
 public:
     /// @brief Constructor (does nothing)
@@ -692,6 +819,76 @@ public:
     void evaluate(Pkt& pkt, ValueStack& values);
 };
 
+/// @brief Token that represents an alternative
+///
+/// For example in the sub-expression "ifelse(cond, iftrue, iffalse)"
+/// the boolean "cond" expression is evaluated, if it is true then
+/// the "iftrue" value is returned else the "iffalse" value is returned.
+/// Please note that "iftrue" and "iffalse" must be plain string (vs. boolean)
+/// expressions and they are always evaluated. If you want a similar
+/// operator on boolean expressions it can be built from "and", "or" and
+/// "not" boolean operators.
+class TokenIfElse : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenIfElse() { }
+
+    /// @brief Alternative.
+    ///
+    /// Evaluation does not use packet information, but rather consumes the
+    /// last three results. It does a simple string comparison on the
+    /// condition (third value on the stack) which is required to be
+    /// either "true" or "false", and leaves the second and first
+    /// value if the condition is "true" or "false".
+    ///
+    /// @throw EvalBadStack if there are less than 3 values on stack
+    /// @throw EvalTypeError if the third value (the condition) is not
+    ///        either "true" or "false"
+    ///
+    /// @param pkt (unused)
+    /// @param values - stack of values (two items are removed)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
+/// @brief Token that converts to hexadecimal string
+///
+/// For example in the sub-expression "hexstring(pkt4.mac, ':')"
+/// the binary MAC address is converted to its usual hexadecimal
+/// representation as a list of (6) pairs of hexadecimal digits
+/// separated by colons (':').
+/// Please note the token is named TokenToHexString when the syntax
+/// use the hexstring name without a leading "to".
+class TokenToHexString : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenToHexString() { }
+
+    /// @brief Convert a binary value to its hexadecimal string representation
+    ///
+    /// Evaluation does not use packet information. It requires at least
+    /// two values to be present on the stack. It will consume the top
+    /// two values on the stack as parameters and push the resulting
+    /// hexadecimal string onto the stack.
+    /// From the top it expects the values on the stack as:
+    /// - separator
+    /// - binary
+    ///
+    /// binary is the binary value (note it can be any value, i.e.
+    /// it is not checked to really be not printable).
+    /// separator is literal for instance '-' or ':'. The empty separator
+    /// means no separator.
+    ///
+    /// The following example use a binary MAC address 06:ce:8f:55:b3:33:
+    /// - mac, '-' => "06-ce-8f-55-b3-33"
+    ///
+    /// @throw EvalBadStack if there are less than 2 values on stack
+    ///
+    /// @param pkt (unused)
+    /// @param values - stack of values (2 arguments will be popped, 1 result
+    ///        will be pushed)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
 /// @brief Token that represents logical negation operator
 ///
 /// For example in the expression "not(option[vendor-class].text == 'MSF')"
@@ -767,6 +964,40 @@ public:
     void evaluate(Pkt& pkt, ValueStack& values);
 };
 
+/// @brief Token that represents client class membership
+///
+/// For example "not member('foo')" is the complement of class foo
+class TokenMember : public Token {
+public:
+    /// @brief Constructor
+    ///
+    /// @param client_class client class name
+    TokenMember(const std::string& client_class)
+        :client_class_(client_class){
+    }
+
+    /// @brief Token evaluation (check if client_class_ was added to
+    /// packet client classes)
+    ///
+    /// @param pkt the class name will be check from this packet's client classes
+    /// @param values true (if found) or false (if not found) will be pushed here
+    void evaluate(Pkt& pkt, ValueStack& values);
+
+    /// @brief Returns client class name
+    ///
+    /// This method is used in testing to determine if the parser had
+    /// instantiated TokenMember with correct parameters.
+    ///
+    /// @return client class name the token expects to check membership.
+    const ClientClass& getClientClass() const {
+        return (client_class_);
+    }
+
+protected:
+    /// @brief The client class name
+    ClientClass client_class_;
+};
+
 /// @brief Token that represents vendor options in DHCPv4 and DHCPv6.
 ///
 /// It covers vendor independent vendor information option (125, DHCPv4)
@@ -952,7 +1183,69 @@ protected:
     uint16_t index_;
 };
 
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+/// @brief Token that represents sub-options in DHCPv4 and DHCPv6.
+///
+/// It covers any options which encapsulate sub-options, for instance
+/// dhcp-agent-options (82, DHCPv4) or rsoo (66, DHCPv6).
+/// This class is derived from TokenOption and leverages its ability
+/// to operate on sub-options. It also adds additional capabilities.
+///
+/// It can represent the following expressions:
+/// option[149].exists - check if option 149 exists
+/// option[149].option[1].exists - check if suboption 1 exists in the option 149
+/// option[149].option[1].hex - return content of suboption 1 for option 149
+class TokenSubOption : public TokenOption {
+public:
+
+    /// @note Does not define its own representation type:
+    /// simply use the @c TokenOption::RepresentationType
+
+    /// @brief Constructor that takes an option and sub-option codes as parameter
+    ///
+    /// Note: There is no constructor that takes names.
+    ///
+    /// @param option_code code of the parent option.
+    /// @param sub_option_code code of the sub-option to be represented.
+    /// @param rep_type Token representation type.
+    TokenSubOption(const uint16_t option_code,
+                   const uint16_t sub_option_code,
+                   const RepresentationType& rep_type)
+        : TokenOption(option_code, rep_type), sub_option_code_(sub_option_code) {}
+
+    /// @brief This is a method for evaluating a packet.
+    ///
+    /// This token represents a value of the sub-option, so this method
+    /// attempts to extract the parent option from the packet and when
+    /// it succeeds to extract the sub-option from the option and
+    /// its value on the stack.
+    /// If the parent option or the sub-option is not there, an empty
+    /// string ("") is put on the stack.
+    ///
+    /// @param pkt specified parent option will be extracted from this packet
+    /// @param values value of the sub-option will be pushed here (or "")
+    virtual void evaluate(Pkt& pkt, ValueStack& values);
+
+    /// @brief Returns sub-option-code
+    ///
+    /// This method is used in testing to determine if the parser had
+    /// instantiated TokenSubOption with correct parameters.
+    ///
+    /// @return option-code of the sub-option this token expects to extract.
+    uint16_t getSubCode() const {
+        return (sub_option_code_);
+    }
+
+protected:
+    /// @brief Attempts to retrieve a sub-option.
+    ///
+    /// @param parent the sub-option will be retrieved from here
+    /// @return sub-option instance (or NULL if not found)
+    virtual OptionPtr getSubOption(const OptionPtr& parent);
+
+    uint16_t sub_option_code_; ///< Code of the sub-option to be extracted
+};
+
+} // end of isc::dhcp namespace
+} // end of isc namespace
 
 #endif
