@@ -1,4 +1,4 @@
-// Copyright (C) 2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,6 +10,7 @@
 #include <config_backend/base_config_backend_pool.h>
 #include <process/cb_ctl_base.h>
 #include <boost/date_time/posix_time/posix_time.hpp>
+#include <boost/date_time/gregorian/gregorian.hpp>
 #include <boost/shared_ptr.hpp>
 #include <gtest/gtest.h>
 #include <map>
@@ -39,19 +40,23 @@ public:
     ///
     /// @param modification_time The lower bound time for which audit
     /// entries should be returned.
+    /// @param modification_id The lower bound id for which audit
+    /// entries should be returned.
     ///
     /// @return Collection of audit entries later than specified time.
     virtual db::AuditEntryCollection
     getRecentAuditEntries(const db::ServerSelector&,
-                          const boost::posix_time::ptime& modification_time) const {
+                          const boost::posix_time::ptime& modification_time,
+                          const uint64_t modification_id) const {
         db::AuditEntryCollection filtered_entries;
 
         // Use the index which orders the audit entries by timestamps.
-        const auto& index = audit_entries_.get<AuditEntryModificationTimeTag>();
+        const auto& index = audit_entries_.get<AuditEntryModificationTimeIdTag>();
 
         // Locate the first audit entry after the last one having the
-        // specified modification time.
-        auto first_entry = index.upper_bound(modification_time);
+        // specified modification time and id.
+        auto modification = boost::make_tuple(modification_time, modification_id);
+        auto first_entry = index.upper_bound(modification);
 
         // If there are any entries found return them.
         if (first_entry != index.end()) {
@@ -66,15 +71,18 @@ public:
     /// @param object_type Object type to be stored in the audit entry.
     /// @param object_id Object id to be stored in the audit entry.
     /// @param modification_time Audit entry modification time to be set.
+    /// @param modification_id Audit entry modification id to be set.
     void addAuditEntry(const ServerSelector&,
                        const std::string& object_type,
                        const uint64_t object_id,
-                       const boost::posix_time::ptime& modification_time) {
+                       const boost::posix_time::ptime& modification_time,
+                       const uint64_t modification_id) {
         // Create new audit entry from the specified parameters.
         AuditEntryPtr audit_entry(new AuditEntry(object_type,
                                                  object_id,
                                                  AuditEntry::ModificationType::CREATE,
                                                  modification_time,
+                                                 modification_id,
                                                  "added audit entry"));
 
         // The audit entries are held in the static variable so as they
@@ -143,15 +151,17 @@ public:
     /// @param object_type Object type to be stored in the audit entry.
     /// @param object_id Object id to be stored in the audit entry.
     /// @param modification_time Audit entry modification time to be set.
+    /// @param modification_id Audit entry modification id to be set.
     void addAuditEntry(const BackendSelector& backend_selector,
                        const ServerSelector& server_selector,
                        const std::string& object_type,
                        const uint64_t object_id,
-                       const boost::posix_time::ptime& modification_time) {
+                       const boost::posix_time::ptime& modification_time,
+                       const uint64_t modification_id) {
         createUpdateDeleteProperty<void, const std::string&, uint64_t,
-                                   const boost::posix_time::ptime&>
+                                   const boost::posix_time::ptime&, uint64_t>
             (&CBControlBackend::addAuditEntry, backend_selector, server_selector,
-             object_type, object_id, modification_time);
+             object_type, object_id, modification_time, modification_id);
     }
 
     /// @brief Retrieves the audit entries later than specified time.
@@ -160,16 +170,20 @@ public:
     /// @param server_selector Server selector.
     /// @param modification_time The lower bound time for which audit
     /// entries should be returned.
+    /// @param modification_id The lower bound id for which audit
+    /// entries should be returned.
     ///
     /// @return Collection of audit entries later than specified time.
     virtual db::AuditEntryCollection
     getRecentAuditEntries(const BackendSelector& backend_selector,
                           const ServerSelector& server_selector,
-                          const boost::posix_time::ptime& modification_time) const {
+                          const boost::posix_time::ptime& modification_time,
+                          const uint64_t modification_id) const {
         AuditEntryCollection audit_entries;
         getMultiplePropertiesConst<AuditEntryCollection, const boost::posix_time::ptime&>
             (&CBControlBackend::getRecentAuditEntries, backend_selector,
-             server_selector, audit_entries, modification_time);
+             server_selector, audit_entries, modification_time,
+             modification_id);
         return (audit_entries);
     }
 };
@@ -223,7 +237,7 @@ public:
 
     using CBControlBase<CBControlBackendMgr>::fetchConfigElement;
     using CBControlBase<CBControlBackendMgr>::getMgr;
-    using CBControlBase<CBControlBackendMgr>::getInitialAuditEntryTime;
+    using CBControlBase<CBControlBackendMgr>::getInitialAuditRevisionTime;
 
     /// @brief Constructor.
     CBControl()
@@ -282,15 +296,27 @@ public:
     }
 
     /// @brief Returns the recorded time of last audit entry.
-    boost::posix_time::ptime getLastAuditEntryTime() const {
-        return (last_audit_entry_time_);
+    boost::posix_time::ptime getLastAuditRevisionTime() const {
+        return (last_audit_revision_time_);
+    }
+
+    /// @brief Returns the recorded id of last audit entry.
+    uint64_t getLastAuditRevisionId() const {
+        return (last_audit_revision_id_);
     }
 
     /// @brief Overwrites the last audit entry time.
     ///
-    /// @param last_audit_entry_time New time to be set.
-    void setLastAuditEntryTime(const boost::posix_time::ptime& last_audit_entry_time) {
-        last_audit_entry_time_ = last_audit_entry_time;
+    /// @param last_audit_revision_time New time to be set.
+    void setLastAuditRevisionTime(const boost::posix_time::ptime& last_audit_revision_time) {
+        last_audit_revision_time_ = last_audit_revision_time;
+    }
+
+    /// @brief Overwrites the last audit revision id.
+    ///
+    /// @param last_audit_revision_id New id to be set.
+    void setLastAuditRevisionId(const uint64_t& last_audit_revision_id) {
+        last_audit_revision_id_ = last_audit_revision_id;
     }
 
     /// @brief Enables the @c databaseConfigApply function to throw.
@@ -394,22 +420,37 @@ TEST_F(CBControlBaseTest, getMgr) {
     EXPECT_EQ(TEST_INSTANCE_ID, mgr.getInstanceId());
 }
 
+// This test verifies that the initial audit revision time is set to
+// local time of 2000-01-01.
+TEST_F(CBControlBaseTest, getInitialAuditRevisionTime) {
+    auto initial_time = cb_ctl_.getInitialAuditRevisionTime();
+    ASSERT_FALSE(initial_time.is_not_a_date_time());
+    auto tm = boost::posix_time::to_tm(initial_time);
+    EXPECT_EQ(100, tm.tm_year);
+    EXPECT_EQ(0, tm.tm_mon);
+    EXPECT_EQ(0, tm.tm_yday);
+    EXPECT_EQ(0, tm.tm_hour);
+    EXPECT_EQ(0, tm.tm_min);
+    EXPECT_EQ(0, tm.tm_sec);
+}
+
 // This test verifies that last audit entry time is reset upon the
 // call to CBControlBase::reset().
 TEST_F(CBControlBaseTest, reset) {
-    cb_ctl_.setLastAuditEntryTime(timestamps_["tomorrow"]);
+    cb_ctl_.setLastAuditRevisionTime(timestamps_["tomorrow"]);
     cb_ctl_.reset();
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(0, cb_ctl_.getLastAuditRevisionId());
 }
 
-// This test verifies that it is correctly determined whether the
-// server should fetch the particular configuration element.
+// This test verifies that it is correctly determined what entries the
+// server should fetch for the particular configuration element.
 TEST_F(CBControlBaseTest, fetchConfigElement) {
     db::AuditEntryCollection audit_entries;
-    // When audit entries collection is empty it indicates that this
-    // is the case of the full server reconfiguration. Always indicate
-    // that the configuration elements must be fetched.
-    EXPECT_TRUE(cb_ctl_.fetchConfigElement(audit_entries, "my_object_type"));
+    db::AuditEntryCollection updated;
+    // When audit entries collection is empty any subset is empty too.
+    updated = cb_ctl_.fetchConfigElement(audit_entries, "my_object_type");
+    EXPECT_TRUE(updated.empty());
 
     // Now test the case that there is a DELETE audit entry. In this case
     // our function should indicate that the configuration should not be
@@ -418,24 +459,55 @@ TEST_F(CBControlBaseTest, fetchConfigElement) {
     // no reason to fetch the data from the database.
     AuditEntryPtr audit_entry(new AuditEntry("dhcp4_subnet", 1234 ,
                                              AuditEntry::ModificationType::DELETE,
-                                             "added audit entry"));
+                                             2345, "added audit entry"));
     audit_entries.insert(audit_entry);
-    EXPECT_FALSE(cb_ctl_.fetchConfigElement(audit_entries, "my_object_type"));
+    updated = cb_ctl_.fetchConfigElement(audit_entries, "my_object_type");
+    EXPECT_TRUE(updated.empty());
+    EXPECT_TRUE(hasObjectId(audit_entries, 1234));
+    EXPECT_FALSE(hasObjectId(audit_entries, 5678));
+    EXPECT_FALSE(hasObjectId(updated, 1234));
 
     // Add another audit entry which indicates creation of the configuration element.
-    // This time we should get 'true'.
+    // This time we should get it.
     audit_entry.reset(new AuditEntry("my_object_type", 5678,
                                      AuditEntry::ModificationType::CREATE,
-                                     "added audit entry"));
+                                     6789, "added audit entry"));
     audit_entries.insert(audit_entry);
-    EXPECT_TRUE(cb_ctl_.fetchConfigElement(audit_entries, "my_object_type"));
+    updated = cb_ctl_.fetchConfigElement(audit_entries, "my_object_type");
+    ASSERT_EQ(1, updated.size());
+    AuditEntryPtr updated_entry = (*updated.begin());
+    ASSERT_TRUE(updated_entry);
+    EXPECT_EQ("my_object_type", updated_entry->getObjectType());
+    EXPECT_EQ(5678, updated_entry->getObjectId());
+    EXPECT_EQ(AuditEntry::ModificationType::CREATE, updated_entry->getModificationType());
+    EXPECT_TRUE(hasObjectId(audit_entries, 5678));
+    EXPECT_TRUE(hasObjectId(updated, 5678));
+    EXPECT_FALSE(hasObjectId(updated, 1234));
 
     // Also we should get 'true' for the UPDATE case.
-    audit_entry.reset(new AuditEntry("another_object_type",
+    audit_entry.reset(new AuditEntry("my_object_type",
                                      5678, AuditEntry::ModificationType::UPDATE,
-                                     "added audit entry"));
+                                     6790, "added audit entry"));
     audit_entries.insert(audit_entry);
-    EXPECT_TRUE(cb_ctl_.fetchConfigElement(audit_entries, "another_object_type"));
+    updated = cb_ctl_.fetchConfigElement(audit_entries, "my_object_type");
+    EXPECT_EQ(2, updated.size());
+    bool saw_create = false;
+    bool saw_update =  false;
+    for (auto entry : updated) {
+        EXPECT_EQ("my_object_type", entry->getObjectType());
+        EXPECT_EQ(5678, entry->getObjectId());
+        if (AuditEntry::ModificationType::CREATE == entry->getModificationType()) {
+            EXPECT_FALSE(saw_create);
+            saw_create = true;
+        } else if (AuditEntry::ModificationType::UPDATE == entry->getModificationType()) {
+            EXPECT_FALSE(saw_update);
+            saw_update = true;
+        }
+    }
+    EXPECT_TRUE(saw_create);
+    EXPECT_TRUE(saw_update);
+    EXPECT_TRUE(hasObjectId(updated, 5678));
+    EXPECT_FALSE(hasObjectId(updated, 1234));
 }
 
 // This test verifies that true is return when the server successfully
@@ -462,13 +534,15 @@ TEST_F(CBControlBaseTest, fetchAll) {
                                               ServerSelector::ALL(),
                                               "sql_table_2",
                                               1234,
-                                              timestamps_["yesterday"]);
+                                              timestamps_["yesterday"],
+                                              2345);
 
     cb_ctl_.getMgr().getPool()->addAuditEntry(BackendSelector::UNSPEC(),
                                               ServerSelector::ALL(),
                                               "sql_table_1",
                                               3456,
-                                              timestamps_["today"]);
+                                              timestamps_["today"],
+                                              4567);
 
     // Disconnect from the database in order to check that the
     // databaseConfigFetch reconnects.
@@ -479,7 +553,8 @@ TEST_F(CBControlBaseTest, fetchAll) {
     ASSERT_EQ(BackendSelector::Type::MYSQL, cb_ctl_.getBackendSelector().getBackendType());
     ASSERT_EQ(ServerSelector::Type::UNASSIGNED, cb_ctl_.getServerSelector().getType());
     ASSERT_EQ(-1, cb_ctl_.getAuditEntriesNum());
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(0, cb_ctl_.getLastAuditRevisionId());
 
     // Connect to the database and fetch the configuration.
     ASSERT_NO_THROW(cb_ctl_.databaseConfigFetch(config_base));
@@ -494,7 +569,8 @@ TEST_F(CBControlBaseTest, fetchAll) {
     // Make sure that the internal timestamp is set to the most recent
     // audit entry, so as the server will only later fetch config
     // updates after this timestamp.
-    EXPECT_EQ(timestamps_["today"], cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(timestamps_["today"], cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(4567, cb_ctl_.getLastAuditRevisionId());
 }
 
 // This test verifies that the configuration can be fetched for a
@@ -509,7 +585,8 @@ TEST_F(CBControlBaseTest, fetchFromServer) {
     ASSERT_EQ(BackendSelector::Type::MYSQL, cb_ctl_.getBackendSelector().getBackendType());
     ASSERT_EQ(ServerSelector::Type::UNASSIGNED, cb_ctl_.getServerSelector().getType());
     ASSERT_EQ(-1, cb_ctl_.getAuditEntriesNum());
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(0, cb_ctl_.getLastAuditRevisionId());
 
     ASSERT_NO_THROW(cb_ctl_.databaseConfigFetch(config_base));
 
@@ -518,7 +595,7 @@ TEST_F(CBControlBaseTest, fetchFromServer) {
     EXPECT_EQ(BackendSelector::Type::UNSPEC, cb_ctl_.getBackendSelector().getBackendType());
     // An explicit server selector should have been used this time.
     ASSERT_EQ(ServerSelector::Type::SUBSET, cb_ctl_.getServerSelector().getType());
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
 
     // Make sure that the server selector used in databaseConfigFetch is
     // correct.
@@ -540,14 +617,16 @@ TEST_F(CBControlBaseTest, fetchUpdates) {
                                               ServerSelector::ALL(),
                                               "sql_table_1",
                                               3456,
-                                              timestamps_["today"]);
+                                              timestamps_["today"],
+                                              4567);
 
     // Verify that various indicators are set to their initial values.
     ASSERT_EQ(0, cb_ctl_.getMergesNum());
     ASSERT_EQ(BackendSelector::Type::MYSQL, cb_ctl_.getBackendSelector().getBackendType());
     ASSERT_EQ(ServerSelector::Type::UNASSIGNED, cb_ctl_.getServerSelector().getType());
     ASSERT_EQ(-1, cb_ctl_.getAuditEntriesNum());
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(0, cb_ctl_.getLastAuditRevisionId());
 
     ASSERT_NO_THROW(cb_ctl_.databaseConfigFetch(config_base,
                                                 CBControl::FetchMode::FETCH_UPDATE));
@@ -559,7 +638,8 @@ TEST_F(CBControlBaseTest, fetchUpdates) {
     EXPECT_EQ(BackendSelector::Type::UNSPEC, cb_ctl_.getBackendSelector().getBackendType());
     EXPECT_EQ(ServerSelector::Type::ALL, cb_ctl_.getServerSelector().getType());
     // The last audit entry time should be set to the latest audit entry.
-    EXPECT_EQ(timestamps_["today"], cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(timestamps_["today"], cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(4567, cb_ctl_.getLastAuditRevisionId());
 }
 
 // Check that the databaseConfigApply function is not called when there
@@ -570,7 +650,8 @@ TEST_F(CBControlBaseTest, fetchNoUpdates) {
     // Set last audit entry time to the timestamp of the audit
     // entry we are going to add. That means that there will be
     // no new audit entries to fetch.
-    cb_ctl_.setLastAuditEntryTime(timestamps_["yesterday"]);
+    cb_ctl_.setLastAuditRevisionTime(timestamps_["yesterday"]);
+    cb_ctl_.setLastAuditRevisionId(4567);
 
     ASSERT_TRUE(cb_ctl_.databaseConfigConnect(config_base));
 
@@ -578,7 +659,8 @@ TEST_F(CBControlBaseTest, fetchNoUpdates) {
                                               ServerSelector::ALL(),
                                               "sql_table_1",
                                               3456,
-                                              timestamps_["yesterday"]);
+                                              timestamps_["yesterday"],
+                                              4567);
 
     ASSERT_EQ(0, cb_ctl_.getMergesNum());
 
@@ -603,7 +685,8 @@ TEST_F(CBControlBaseTest, fetchFailure) {
                                               ServerSelector::ALL(),
                                               "sql_table_1",
                                               3456,
-                                              timestamps_["today"]);
+                                              timestamps_["today"],
+                                              4567);
 
     // Configure the CBControl to always throw simulating the failure
     // during configuration merge.
@@ -614,7 +697,8 @@ TEST_F(CBControlBaseTest, fetchFailure) {
     ASSERT_EQ(BackendSelector::Type::MYSQL, cb_ctl_.getBackendSelector().getBackendType());
     ASSERT_EQ(ServerSelector::Type::UNASSIGNED, cb_ctl_.getServerSelector().getType());
     ASSERT_EQ(-1, cb_ctl_.getAuditEntriesNum());
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(0, cb_ctl_.getLastAuditRevisionId());
 
     ASSERT_THROW(cb_ctl_.databaseConfigFetch(config_base, CBControl::FetchMode::FETCH_UPDATE),
                  isc::Unexpected);
@@ -627,8 +711,8 @@ TEST_F(CBControlBaseTest, fetchFailure) {
     EXPECT_EQ(ServerSelector::Type::ALL, cb_ctl_.getServerSelector().getType());
     // The last audit entry time should not be modified because there was a merge
     // error.
-    EXPECT_EQ(cb_ctl_.getInitialAuditEntryTime(), cb_ctl_.getLastAuditEntryTime());
-
+    EXPECT_EQ(cb_ctl_.getInitialAuditRevisionTime(), cb_ctl_.getLastAuditRevisionTime());
+    EXPECT_EQ(0, cb_ctl_.getLastAuditRevisionId());
 }
 
 }
