@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,10 +10,14 @@
 #include <testutils/user_context_utils.h>
 #include <yang/translator_config.h>
 #include <yang/yang_models.h>
-#include <yang/tests/yang_configs.h>
 #include <yang/tests/json_configs.h>
+#include <yang/tests/yang_configs.h>
+#include <yang/tests/sysrepo_setup.h>
+
 #include <boost/algorithm/string.hpp>
+
 #include <gtest/gtest.h>
+
 #include <iostream>
 
 using namespace std;
@@ -21,9 +25,7 @@ using namespace isc;
 using namespace isc::data;
 using namespace isc::yang;
 using namespace isc::yang::test;
-#ifndef HAVE_PRE_0_7_6_SYSREPO
 using namespace sysrepo;
-#endif
 
 namespace {
 
@@ -50,39 +52,38 @@ std::string generateDiff(std::string, std::string) {
 }
 #endif
 
-
 /// @brief Test Fixture class for Yang <-> JSON configs.
 class ConfigTest : public ::testing::Test {
 public:
+    virtual ~ConfigTest() = default;
 
-    /// @brief Constructor.
-    ConfigTest() {
-        createSession();
+    void SetUp() override {
+        SysrepoSetup::cleanSharedMemory();
+        connection_ = std::make_shared<Connection>();
+        session_.reset(new Session(connection_, SR_DS_CANDIDATE));
+        translator_.reset(new TranslatorBasic(session_, model_));
+        cleanModelData();
     }
 
-    /// @brief Virtual destructor.
-    virtual ~ConfigTest() {
+    void TearDown() override {
+        cleanModelData();
+        translator_.reset();
         session_.reset();
         connection_.reset();
-        model_.clear();
+        SysrepoSetup::cleanSharedMemory();
     }
 
-    /// @brief Set model.
-    ///
-    /// @param model The model name.
-    void setModel(const string model) {
-        model_ = model;
-    }
-
-    /// @brief Create session.
-    void createSession() {
-        connection_.reset(new Connection("configs unittests"));
-        session_.reset(new Session(connection_, SR_DS_CANDIDATE));
+    void cleanModelData() {
+        std::string toplevel_node("config");
+        if (model_ == IETF_DHCPV6_SERVER) {
+            toplevel_node = "server";
+        }
+        translator_->delItem("/" + model_ + ":" + toplevel_node);
     }
 
     /// @brief Reset session.
     void resetSession() {
-        session_.reset(new Session(connection_, SR_DS_CANDIDATE));
+        SetUp();
     }
 
     /// @brief Loads YANG configuration from specified tree.
@@ -105,8 +106,8 @@ public:
     ///
     /// @param config The JSON tree to load in textual format.
     void load(const string& config) {
-        ConstElementPtr json;
-        ASSERT_NO_THROW(json = Element::fromJSON(config));
+        ElementPtr json;
+        ASSERT_NO_THROW_LOG(json = Element::fromJSON(config));
         load(json);
     }
 
@@ -171,7 +172,7 @@ public:
     /// @param expected The expected JSON tree in textual format.
     bool verify(const string& config) {
         ConstElementPtr expected;
-        expected= Element::fromJSON(config);
+        expected = Element::fromJSON(config);
         return (verify(expected));
     }
 
@@ -192,88 +193,90 @@ public:
 
     /// @brief The sysrepo session.
     S_Session session_;
+
+    std::unique_ptr<TranslatorBasic> translator_;
+};
+
+struct ConfigTestKeaV4 : ConfigTest {
+    ConfigTestKeaV4() {
+        model_ = KEA_DHCP4_SERVER;
+    }
+};
+struct ConfigTestKeaV6 : ConfigTest {
+    ConfigTestKeaV6() {
+        model_ = KEA_DHCP6_SERVER;
+    }
+};
+struct ConfigTestIetfV6 : ConfigTest {
+    ConfigTestIetfV6() {
+        model_ = IETF_DHCPV6_SERVER;
+    }
 };
 
 // Check empty config with ietf-dhcpv6-server model.
-TEST_F(ConfigTest, emptyIetf6) {
-    // First set the model.
-    setModel(IETF_DHCPV6_SERVER);
-
+TEST_F(ConfigTestIetfV6, emptyIetf6) {
     YRTree tree;
-    ASSERT_NO_THROW(load(tree));
+    ASSERT_NO_THROW_LOG(load(tree));
     EXPECT_TRUE(verify(tree));
 
     ConstElementPtr json = Element::fromJSON(emptyJson6);
     EXPECT_TRUE(verify(json));
-    ASSERT_NO_THROW(load(json));
+    ASSERT_NO_THROW_LOG(load(json));
     EXPECT_TRUE(verify(emptyJson6));
     EXPECT_TRUE(verify(tree));
 }
 
 // Check empty config with kea-dhcp4-server:config model.
-TEST_F(ConfigTest, emptyKeaDhcp4) {
-    // First set the model.
-    setModel(KEA_DHCP4_SERVER);
-
+TEST_F(ConfigTestKeaV4, emptyKeaDhcp4) {
     YRTree tree;
-    ASSERT_NO_THROW(load(tree));
-    EXPECT_TRUE(verify(tree));
+    ASSERT_NO_THROW_LOG(load(tree));
+    EXPECT_TRUE(verify(emptyTreeKeaDhcp4));
 
     ConstElementPtr json = Element::fromJSON(emptyJson4);
     EXPECT_TRUE(verify(json));
-    ASSERT_NO_THROW(load(json));
+    ASSERT_NO_THROW_LOG(load(json));
     EXPECT_TRUE(verify(emptyJson4));
-    EXPECT_TRUE(verify(tree));
+    EXPECT_TRUE(verify(emptyTreeKeaDhcp4));
 }
 
 // Check empty config with kea-dhcp6-server:config model.
-TEST_F(ConfigTest, emptyKeaDhcp6) {
-    // First set the model.
-    setModel(KEA_DHCP6_SERVER);
-
+TEST_F(ConfigTestKeaV6, emptyKeaDhcp6) {
     YRTree tree;
-    ASSERT_NO_THROW(load(tree));
-    EXPECT_TRUE(verify(tree));
+    ASSERT_NO_THROW_LOG(load(tree));
+    EXPECT_TRUE(verify(emptyTreeKeaDhcp6));
 
     ConstElementPtr json = Element::fromJSON(emptyJson6);
     EXPECT_TRUE(verify(json));
-    ASSERT_NO_THROW(load(json));
+    ASSERT_NO_THROW_LOG(load(json));
     EXPECT_TRUE(verify(emptyJson6));
-    EXPECT_TRUE(verify(tree));
+    EXPECT_TRUE(verify(emptyTreeKeaDhcp6));
 }
 
 // Check subnet with two pools with ietf-dhcpv6-server model.
 // Validation will fail because the current model has a vendor-info
 // container with a mandatory ent-num leaf and no presence flag,
 // and of course the candidate YANG tree has nothing for this.
-TEST_F(ConfigTest, subnetTwoPoolsIetf6) {
-    // First set the model.
-    setModel(subnetTwoPoolsModelIetf6);
-
-    ASSERT_NO_THROW(load(subnetTwoPoolsTreeIetf6));
+TEST_F(ConfigTestIetfV6, subnetTwoPoolsIetf6) {
+    ASSERT_NO_THROW_LOG(load(subnetTwoPoolsTreeIetf6));
     EXPECT_TRUE(verify(subnetTwoPoolsJson6));
 
     resetSession();
 
-    ASSERT_NO_THROW(load(subnetTwoPoolsJson6));
+    ASSERT_NO_THROW_LOG(load(subnetTwoPoolsJson6));
     EXPECT_TRUE(verify(subnetTwoPoolsTreeIetf6));
 
-    cout << "validation is expected to fail: please ignore messages" << endl;
     EXPECT_FALSE(validate());
 }
 
 // Check subnet with a pool and option data lists with
 // kea-dhcp4-server:config model.
-TEST_F(ConfigTest, subnetOptionsKeaDhcp4) {
-    // First set the model.
-    setModel(subnetOptionsModelKeaDhcp4);
-
-    ASSERT_NO_THROW(load(subnetOptionsTreeKeaDhcp4));
+TEST_F(ConfigTestKeaV4, subnetOptionsKeaDhcp4) {
+    ASSERT_NO_THROW_LOG(load(subnetOptionsTreeKeaDhcp4));
     EXPECT_TRUE(verify(subnetOptionsJson4));
 
     resetSession();
 
-    ASSERT_NO_THROW(load(subnetOptionsJson4));
+    ASSERT_NO_THROW_LOG(load(subnetOptionsJson4));
     EXPECT_TRUE(verify(subnetOptionsTreeKeaDhcp4));
 
     EXPECT_TRUE(validate());
@@ -281,41 +284,32 @@ TEST_F(ConfigTest, subnetOptionsKeaDhcp4) {
 
 // Check subnet with a pool and option data lists with
 // kea-dhcp6-server:config model.
-TEST_F(ConfigTest, subnetOptionsKeaDhcp6) {
-    // First set the model.
-    setModel(subnetOptionsModelKeaDhcp6);
-
-    ASSERT_NO_THROW(load(subnetOptionsTreeKeaDhcp6));
+TEST_F(ConfigTestKeaV6, subnetOptionsKeaDhcp6) {
+    ASSERT_NO_THROW_LOG(load(subnetOptionsTreeKeaDhcp6));
     EXPECT_TRUE(verify(subnetOptionsJson6));
 
     resetSession();
 
-    ASSERT_NO_THROW(load(subnetOptionsJson6));
+    ASSERT_NO_THROW_LOG(load(subnetOptionsJson6));
     EXPECT_TRUE(verify(subnetOptionsTreeKeaDhcp6));
 
     EXPECT_TRUE(validate());
 }
 
 // Check with timers.
-TEST_F(ConfigTest, subnetTimersIetf6) {
-    // First set the model.
-    setModel(subnetTimersModel);
-
-    ASSERT_NO_THROW(load(subnetTimersIetf6));
+TEST_F(ConfigTestIetfV6, subnetTimersIetf6) {
+    ASSERT_NO_THROW_LOG(load(subnetTimersIetf6));
     EXPECT_TRUE(verify(subnetTimersJson6));
 
     resetSession();
 
-    ASSERT_NO_THROW(load(subnetTimersJson6));
+    ASSERT_NO_THROW_LOG(load(subnetTimersJson6));
     EXPECT_TRUE(verify(subnetTimersIetf6));
 }
 
 // Check a ietf-dhcpv6-server configuration which validates.
-TEST_F(ConfigTest, validateIetf6) {
-    // First set the model.
-    setModel(validModelIetf6);
-
-    ASSERT_NO_THROW(load(validTreeIetf6));
+TEST_F(ConfigTestIetfV6, validateIetf6) {
+    ASSERT_NO_THROW_LOG(load(validTreeIetf6));
     EXPECT_TRUE(verify(validTreeIetf6));
 
     // If this validation fails, make sure you have the model *and its
@@ -328,10 +322,7 @@ TEST_F(ConfigTest, validateIetf6) {
 }
 
 // Check Kea4 example files.
-TEST_F(ConfigTest, examples4) {
-    // First set the model.
-    setModel(KEA_DHCP4_SERVER);
-
+TEST_F(ConfigTestKeaV4, examples4) {
     vector<string> examples = {
         "advanced.json",
         "all-keys-netconf.json",
@@ -342,7 +333,7 @@ TEST_F(ConfigTest, examples4) {
         "comments.json",
         "config-backend.json",
         "dhcpv4-over-dhcpv6.json",
-        "global-reservations.json",
+        // "global-reservations.json" removed for #1405
         "ha-load-balancing-primary.json",
         "hooks.json",
         "hooks-radius.json",
@@ -350,18 +341,18 @@ TEST_F(ConfigTest, examples4) {
         "multiple-options.json",
         "mysql-reservations.json",
         "pgsql-reservations.json",
-        "reservations.json",
+        // "reservations.json" removed for #1405
         "several-subnets.json",
-        "shared-network.json",
+        // "shared-network.json" removed for #1405
         "single-subnet.json",
-        "with-ddns.json"
+        // "with-ddns.json" removed for #35
     };
     for (string file : examples) {
         resetSession();
         string path = string(CFG_EXAMPLES) + "/kea4/" + file;
         cout << "Testing file " << path << endl;
         ConstElementPtr json;
-        ASSERT_NO_THROW(json = loadFile(path));
+        ASSERT_NO_THROW_LOG(json = loadFile(path));
         json = isc::test::moveComments(json);
         EXPECT_TRUE(verify(json));
         EXPECT_TRUE(validate());
@@ -369,10 +360,7 @@ TEST_F(ConfigTest, examples4) {
 }
 
 // Check Kea6 example files.
-TEST_F(ConfigTest, examples6) {
-    // First set the model.
-    setModel(KEA_DHCP6_SERVER);
-
+TEST_F(ConfigTestKeaV6, examples6) {
     vector<string> examples = {
         "advanced.json",
         "all-keys-netconf.json",
@@ -384,7 +372,7 @@ TEST_F(ConfigTest, examples6) {
         "config-backend.json",
         "dhcpv4-over-dhcpv6.json",
         "duid.json",
-        "global-reservations.json",
+        // "global-reservations.json" removed for #1405
         "ha-hot-standby.json",
         "hooks.json",
         "iPXE.json",
@@ -392,21 +380,21 @@ TEST_F(ConfigTest, examples6) {
         "multiple-options.json",
         "mysql-reservations.json",
         "pgsql-reservations.json",
-        "reservations.json",
+        // "reservations.json" removed for #1405
         "several-subnets.json",
-        "shared-network.json",
+        // "shared-network.json" removed for #1405
         "simple.json",
         "softwire46.json",
         "stateless.json",
         "tee-times.json",
-        "with-ddns.json"
+        // "with-ddns.json" removed for #35
     };
     for (string file : examples) {
         resetSession();
         string path = string(CFG_EXAMPLES) + "/kea6/" + file;
         cout << "Testing file " << path << endl;
         ConstElementPtr json;
-        ASSERT_NO_THROW(json = loadFile(path));
+        ASSERT_NO_THROW_LOG(json = loadFile(path));
         json = isc::test::moveComments(json);
         EXPECT_TRUE(verify(json));
         EXPECT_TRUE(validate());
@@ -414,17 +402,14 @@ TEST_F(ConfigTest, examples6) {
 }
 
 // Check the example in the design document.
-TEST_F(ConfigTest, designExample) {
-    // First set the model.
-    setModel(designExampleModel);
-
-    ASSERT_NO_THROW(load(designExampleTree));
+TEST_F(ConfigTestIetfV6, designExample) {
+    ASSERT_NO_THROW_LOG(load(designExampleTree));
     EXPECT_TRUE(verify(designExampleJson));
 
     resetSession();
 
-    ASSERT_NO_THROW(load(designExampleJson));
+    ASSERT_NO_THROW_LOG(load(designExampleJson));
     EXPECT_TRUE(verify(designExampleTree));
 }
 
-}; // end of anonymous namespace
+}  // namespace
