@@ -1,4 +1,4 @@
-// Copyright (C) 2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -16,9 +16,8 @@ namespace test {
 
 /// @brief The test module from sysrepo tests.
 const std::string testModel = "keatest-module";
-const YRTree testTree = {
-    { "/keatest-module:container",      "", SR_CONTAINER_T, false },
-    { "/keatest-module:container/list", "", SR_LIST_T,      true },
+const YRTree testTree = YangRepr::buildTreeFromVector({
+    { "/keatest-module:container", "", SR_CONTAINER_T, false },
     { "/keatest-module:main", "", SR_CONTAINER_T, false },
     { "/keatest-module:main/string", "str", SR_STRING_T, true },
     { "/keatest-module:main/boolean", "true", SR_BOOL_T, true },
@@ -33,29 +32,26 @@ const YRTree testTree = {
     { "/keatest-module:main/enum", "maybe", SR_ENUM_T, true },
     { "/keatest-module:main/raw", "Zm9vYmFy", SR_BINARY_T, true },
     { "/keatest-module:kernel-modules", "", SR_CONTAINER_T, false }
-};
+});
 
 /// @brief A subnet with two pools with ietf-dhcpv6-server model.
 const std::string subnetTwoPoolsModelIetf6 = IETF_DHCPV6_SERVER;
-const YRTree subnetTwoPoolsTreeIetf6 = {
+const YRTree subnetTwoPoolsTreeIetf6 = YangRepr::buildTreeFromVector({
     { "/ietf-dhcpv6-server:server", "", SR_CONTAINER_PRESENCE_T, false },
     { "/ietf-dhcpv6-server:server/server-config", "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges",
       "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='111']", "", SR_LIST_T, true },
+      "network-range[network-range-id='111']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/network-range-id",
       "111", SR_UINT32_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/network-prefix",
       "2001:db8::/48", SR_STRING_T, true },
-    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='111']/address-pools",
-      "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
-      "address-pool[pool-id='0']", "", SR_LIST_T, true },
+      "address-pool[pool-id='0']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
       "address-pool[pool-id='0']/pool-id", "0", SR_UINT32_T, false },
@@ -77,7 +73,7 @@ const YRTree subnetTwoPoolsTreeIetf6 = {
       "disabled", SR_ENUM_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
-      "address-pool[pool-id='1']", "", SR_LIST_T, true },
+      "address-pool[pool-id='1']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
       "address-pool[pool-id='1']/pool-id", "1", SR_UINT32_T, false },
@@ -96,30 +92,52 @@ const YRTree subnetTwoPoolsTreeIetf6 = {
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
       "address-pool[pool-id='1']/max-address-count",
-      "disabled", SR_ENUM_T, true }
-};
+      "disabled", SR_ENUM_T, true },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid/type-code",
+      "65535", SR_UINT16_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/address-pools",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/pd-pools",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/host-reservations",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/lease-storage",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/vendor-info",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/option-sets",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/relay-opaque-paras",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/rsoo-enabled-options",
+      "", SR_CONTAINER_T, false },
+});
 
 /// @brief A subnet with timers with ietf-dhcpv6-server model.
 const std::string subnetTimersModel = IETF_DHCPV6_SERVER;
-const YRTree subnetTimersIetf6 = {
+const YRTree subnetTimersIetf6 = YangRepr::buildTreeFromVector({
     { "/ietf-dhcpv6-server:server", "", SR_CONTAINER_PRESENCE_T, false },
     { "/ietf-dhcpv6-server:server/server-config", "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges",
       "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='111']", "", SR_LIST_T, true },
+      "network-range[network-range-id='111']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/network-range-id",
       "111", SR_UINT32_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/network-prefix",
       "2001:db8::/48", SR_STRING_T, true },
-    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='111']/address-pools",
-      "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
-      "address-pool[pool-id='0']", "", SR_LIST_T, true },
+      "address-pool[pool-id='0']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
       "address-pool[pool-id='0']/pool-id", "0", SR_UINT32_T, false },
@@ -147,7 +165,7 @@ const YRTree subnetTimersIetf6 = {
       "disabled", SR_ENUM_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
-      "address-pool[pool-id='1']", "", SR_LIST_T, true },
+      "address-pool[pool-id='1']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
       "address-pool[pool-id='1']/pool-id", "1", SR_UINT32_T, false },
@@ -172,13 +190,38 @@ const YRTree subnetTimersIetf6 = {
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/address-pools/"
       "address-pool[pool-id='1']/max-address-count",
-      "disabled", SR_ENUM_T, true }
-};
+      "disabled", SR_ENUM_T, true },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid/type-code",
+      "65535", SR_UINT16_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/address-pools",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/pd-pools",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/host-reservations",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/lease-storage",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/vendor-info",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/option-sets",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/relay-opaque-paras",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/rsoo-enabled-options",
+      "", SR_CONTAINER_T, false },
+});
 
 /// @brief A subnet with two pools with ietf-dhcpv6-server model
 /// which validates.
 const std::string validModelIetf6 = IETF_DHCPV6_SERVER;
-const YRTree validTreeIetf6 = {
+const YRTree validTreeIetf6 = YangRepr::buildTreeFromVector({
     { "/ietf-dhcpv6-server:server", "", SR_CONTAINER_PRESENCE_T, false },
     { "/ietf-dhcpv6-server:server/server-config", "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/serv-attributes",
@@ -190,14 +233,14 @@ const YRTree validTreeIetf6 = {
     { "/ietf-dhcpv6-server:server/server-config/option-sets",
       "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/option-sets"
-      "/option-set[option-set-id='0']", "", SR_LIST_T, true },
+      "/option-set[option-set-id='0']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/option-sets"
       "/option-set[option-set-id='0']/option-set-id",
-      "0", SR_UINT32_T, false },
+      "0", SR_UINT32_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges",
       "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='111']", "", SR_LIST_T, true },
+      "network-range[network-range-id='111']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/network-range-id",
       "111", SR_UINT32_T, false },
@@ -207,12 +250,9 @@ const YRTree validTreeIetf6 = {
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/network-prefix",
       "2001:db8::/48", SR_STRING_T, true },
-    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='111']/pd-pools",
-      "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/pd-pools/"
-      "pd-pool[pool-id='0']", "", SR_LIST_T, true },
+      "pd-pool[pool-id='0']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/pd-pools/"
       "pd-pool[pool-id='0']/pool-id", "0", SR_UINT32_T, false },
@@ -248,7 +288,7 @@ const YRTree validTreeIetf6 = {
       "disabled", SR_ENUM_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/pd-pools/"
-      "pd-pool[pool-id='1']", "", SR_LIST_T, true },
+      "pd-pool[pool-id='1']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/pd-pools/"
       "pd-pool[pool-id='1']/pool-id", "1", SR_UINT32_T, false },
@@ -281,20 +321,45 @@ const YRTree validTreeIetf6 = {
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='111']/pd-pools/"
       "pd-pool[pool-id='1']/max-pd-space-utilization",
-      "disabled", SR_ENUM_T, true }
-};
+      "disabled", SR_ENUM_T, true },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid/type-code",
+      "65535", SR_UINT16_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/address-pools",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/pd-pools",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='111']/host-reservations",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/lease-storage",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/vendor-info",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/option-sets",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/relay-opaque-paras",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/rsoo-enabled-options",
+      "", SR_CONTAINER_T, false },
+});
 
 /// @brief A subnet with a pool and option data lists with
 /// kea-dhcp4-server:config model.
 const std::string subnetOptionsModelKeaDhcp4 = KEA_DHCP4_SERVER;
-const YRTree subnetOptionsTreeKeaDhcp4 = {
+const YRTree subnetOptionsTreeKeaDhcp4 = YangRepr::buildTreeFromVector({
     { "/kea-dhcp4-server:config", "", SR_CONTAINER_T, false },
     { "/kea-dhcp4-server:config/subnet4[id='111']", "",
-      SR_LIST_T, true },
+      SR_LIST_T, false },
     { "/kea-dhcp4-server:config/subnet4[id='111']/id",
       "111", SR_UINT32_T, false },
     { "/kea-dhcp4-server:config/subnet4[id='111']/"
-      "option-data[code='100'][space='dns']", "", SR_LIST_T, true },
+      "option-data[code='100'][space='dns']", "", SR_LIST_T, false },
     { "/kea-dhcp4-server:config/subnet4[id='111']/"
       "option-data[code='100'][space='dns']/code",
       "100", SR_UINT8_T, false },
@@ -312,7 +377,7 @@ const YRTree subnetOptionsTreeKeaDhcp4 = {
       "false", SR_BOOL_T, true },
     { "/kea-dhcp4-server:config/subnet4[id='111']/"
       "pool[start-address='10.0.1.0'][end-address='10.0.1.255']",
-      "", SR_LIST_T, true },
+      "", SR_LIST_T, false },
     { "/kea-dhcp4-server:config/subnet4[id='111']/"
       "pool[start-address='10.0.1.0'][end-address='10.0.1.255']/start-address",
       "10.0.1.0", SR_STRING_T, false },
@@ -323,21 +388,33 @@ const YRTree subnetOptionsTreeKeaDhcp4 = {
       "pool[start-address='10.0.1.0'][end-address='10.0.1.255']/prefix",
       "10.0.1.0/24", SR_STRING_T, true },
     { "/kea-dhcp4-server:config/subnet4[id='111']/subnet",
-      "10.0.0.0/8", SR_STRING_T, true }
-};
+      "10.0.0.0/8", SR_STRING_T, true },
+    { "/kea-dhcp4-server:config/expired-leases-processing",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/dhcp-ddns",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/config-control",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/sanity-checks",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/interfaces-config",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/subnet4[id='111']/relay",
+      "", SR_CONTAINER_T, false },
+});
 
 /// @brief A subnet with a pool and option data lists with
 /// kea-dhcp6-server:config model.
 const std::string subnetOptionsModelKeaDhcp6 = KEA_DHCP6_SERVER;
-const YRTree subnetOptionsTreeKeaDhcp6 = {
+const YRTree subnetOptionsTreeKeaDhcp6 = YangRepr::buildTreeFromVector({
     { "/kea-dhcp6-server:config", "", SR_CONTAINER_T, false },
     { "/kea-dhcp6-server:config/subnet6[id='111']", "",
-      SR_LIST_T, true },
+      SR_LIST_T, false },
     { "/kea-dhcp6-server:config/subnet6[id='111']/id",
       "111", SR_UINT32_T, false },
     { "/kea-dhcp6-server:config/subnet6[id='111']/"
       "pool[start-address='2001:db8::1:0'][end-address='2001:db8::1:ffff']",
-      "", SR_LIST_T, true },
+      "", SR_LIST_T, false },
     { "/kea-dhcp6-server:config/subnet6[id='111']/"
       "pool[start-address='2001:db8::1:0'][end-address='2001:db8::1:ffff']/"
       "start-address", "2001:db8::1:0", SR_STRING_T, false },
@@ -350,7 +427,7 @@ const YRTree subnetOptionsTreeKeaDhcp6 = {
     { "/kea-dhcp6-server:config/subnet6[id='111']/"
       "pool[start-address='2001:db8::1:0'][end-address='2001:db8::1:ffff']/"
       "option-data[code='100'][space='dns']",
-      "", SR_LIST_T, true },
+      "", SR_LIST_T, false },
     { "/kea-dhcp6-server:config/subnet6[id='111']/"
       "pool[start-address='2001:db8::1:0'][end-address='2001:db8::1:ffff']/"
       "option-data[code='100'][space='dns']/code",
@@ -372,48 +449,112 @@ const YRTree subnetOptionsTreeKeaDhcp6 = {
       "option-data[code='100'][space='dns']/always-send",
       "false", SR_BOOL_T, true },
     { "/kea-dhcp6-server:config/subnet6[id='111']/subnet",
-      "2001:db8::/48", SR_STRING_T, true }
-};
+      "2001:db8::/48", SR_STRING_T, true },
+    { "/kea-dhcp6-server:config/expired-leases-processing",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/dhcp-ddns",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/config-control",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/sanity-checks",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/interfaces-config",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/subnet6[id='111']/relay",
+      "", SR_CONTAINER_T, false },
+});
 
 /// @brief Example from the design document.
 const std::string designExampleModel = IETF_DHCPV6_SERVER;
-const YRTree designExampleTree = {
+const YRTree designExampleTree = YangRepr::buildTreeFromVector({
     { "/ietf-dhcpv6-server:server", "", SR_CONTAINER_PRESENCE_T, false },
     { "/ietf-dhcpv6-server:server/server-config", "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges",
       "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='1']", "", SR_LIST_T, true },
+      "network-range[network-range-id='1']", "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='1']/network-range-id",
       "1", SR_UINT32_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='1']/network-description",
       "example", SR_STRING_T, true },
-    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='1']/network-prefix",
-      "2001:db8:20:b00::/56", SR_STRING_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='1']/pd-pools",
       "", SR_CONTAINER_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='1']/pd-pools/"
-      "pd-pool[pool-id='0']", "", SR_LIST_T, true },
+      "network-range[network-range-id='1']/pd-pools/pd-pool[pool-id='0']",
+      "", SR_LIST_T, false },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='1']/pd-pools/"
-      "pd-pool[pool-id='0']/pool-id", "0", SR_UINT32_T, false },
+      "network-range[network-range-id='1']/pd-pools/pd-pool[pool-id='0']"
+      "/pool-id",
+      "0", SR_UINT32_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='1']/pd-pools/"
-      "pd-pool[pool-id='0']/prefix",
+      "network-range[network-range-id='1']/pd-pools/pd-pool[pool-id='0']"
+      "/prefix",
       "2001:db8:20:b00::/57", SR_STRING_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
-      "network-range[network-range-id='1']/pd-pools/"
-      "pd-pool[pool-id='0']/prefix-length", "57", SR_UINT8_T, true },
+      "network-range[network-range-id='1']/pd-pools/pd-pool[pool-id='0']"
+      "/prefix-length",
+      "57", SR_UINT8_T, true },
     { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
       "network-range[network-range-id='1']/pd-pools/"
       "pd-pool[pool-id='0']/max-pd-space-utilization",
-      "disabled", SR_ENUM_T, true }
-};
+      "disabled", SR_ENUM_T, true },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='1']/network-prefix",
+      "2001:db8:20:b00::/56", SR_STRING_T, true },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid/type-code",
+      "65535", SR_UINT16_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='1']/host-reservations",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/duid",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/lease-storage",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/serv-attributes/vendor-info",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/option-sets",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/relay-opaque-paras",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/rsoo-enabled-options",
+      "", SR_CONTAINER_T, false },
+    { "/ietf-dhcpv6-server:server/server-config/network-ranges/"
+      "network-range[network-range-id='1']/address-pools",
+      "", SR_CONTAINER_T, false },
+});
+
+const YRTree emptyTreeKeaDhcp4 = YangRepr::buildTreeFromVector({
+    { "/kea-dhcp4-server:config", "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/expired-leases-processing",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/dhcp-ddns",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/config-control",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/sanity-checks",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp4-server:config/interfaces-config",
+      "", SR_CONTAINER_T, false },
+});
+
+const YRTree emptyTreeKeaDhcp6 = YangRepr::buildTreeFromVector({
+    { "/kea-dhcp6-server:config", "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/expired-leases-processing",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/dhcp-ddns",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/config-control",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/sanity-checks",
+      "", SR_CONTAINER_T, false },
+    { "/kea-dhcp6-server:config/interfaces-config",
+      "", SR_CONTAINER_T, false },
+});
 
 /// @brief Set of example configurations.
 const std::vector<std::pair<std::string, YRTree> > test_configs =
@@ -427,8 +568,8 @@ const std::vector<std::pair<std::string, YRTree> > test_configs =
     { designExampleModel,          designExampleTree }
 };
 
-}; // end of namespace isc::yang::test
-}; // end of namespace isc::yang
-}; // end of namespace isc
+}  // namespace test
+}  // namespace yang
+}  // namespace isc
 
 #endif // ISC_YANG_CONFIGS_H
