@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,9 +13,7 @@
 
 using namespace std;
 using namespace isc::data;
-#ifndef HAVE_PRE_0_7_6_SYSREPO
 using namespace sysrepo;
-#endif
 
 namespace isc {
 namespace yang {
@@ -279,24 +277,8 @@ TranslatorDatabases::getDatabases(const string& xpath) {
 
 ElementPtr
 TranslatorDatabases::getDatabasesKea(const string& xpath) {
-    S_Iter_Value iter = getIter(xpath);
-    if (!iter) {
-        // Can't happen.
-        isc_throw(Unexpected, "getDatabasesKea can't get iterator: " << xpath);
-    }
-    ElementPtr result = Element::createList();
-    for (;;) {
-        const string& database = getNext(iter);
-        if (database.empty()) {
-            break;
-        }
-        result->add(getDatabase(database));
-    }
-    if (result->size() > 0) {
-        return (result);
-    } else {
-        return (ElementPtr());
-    }
+    return getList<TranslatorDatabase>(xpath, *this,
+                                       &TranslatorDatabase::getDatabase);
 }
 
 void
@@ -336,5 +318,5 @@ TranslatorDatabases::setDatabasesKea(const string& xpath,
     }
 }
 
-}; // end of namespace isc::yang
-}; // end of namespace isc
+}  // namespace yang
+}  // namespace isc
